@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -804,7 +804,6 @@ is the global server default. */
 /*
   Errorcodes given by handler functions
 
-  opt_sum_query() assumes these codes are > 1
   Do not add error numbers before HA_ERR_FIRST.
   If necessary to add lower numbers, change HA_ERR_FIRST accordingly.
 */
@@ -925,14 +924,16 @@ is the global server default. */
 #define HA_ERR_DISK_FULL_NOWAIT 204 /* No more room in disk */
 #define HA_ERR_NO_SESSION_TEMP 205  /* No session temporary space available */
 #define HA_ERR_WRONG_TABLE_NAME 206 /* Wrong or Invalid table name */
-#define HA_ERR_DEST_SCHEMA_NOT_EXIST 207 /* Destination schema does not exist \
-                                          */
-#define HA_ERR_LAST 207                  /* Copy of last error nr */
+#define HA_ERR_DEST_SCHEMA_NOT_EXIST                         \
+  207                   /* Destination schema does not exist \
+                         */
+#define HA_ERR_LAST 207 /* Copy of last error nr */
 
 /* Number of different errors */
 #define HA_ERR_ERRORS (HA_ERR_LAST - HA_ERR_FIRST + 1)
 
 #define HA_ERR_DECRYPTION_FAILED 500 /* Table encrypted but decypt failed */
+#define HA_ERR_ENCRYPTION_KEY_MISSING 501
 
 /* Other constants */
 
