@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -73,7 +73,7 @@
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/xcom_ssl_transport.h"
 #endif
 
-#define MY_XCOM_PROTO x_1_5
+#define MY_XCOM_PROTO x_1_6
 
 xcom_proto const my_min_xcom_version =
     x_1_0; /* The minimum protocol version I am able to understand */
@@ -762,6 +762,12 @@ static server *mksrv(char *srv, xcom_port port) {
                                 "reply_handler_task", XCOM_THREAD_DEBUG);
   }
   reset_srv_buf(&s->out_buf);
+  /*
+   Keep the server from being freed if the acceptor_learner_task calls
+   srv_unref on the server before the {local_,}server_task and
+   reply_handler_task begin.
+  */
+  srv_ref(s);
   return s;
 }
 
@@ -867,6 +873,9 @@ static void shut_srv(server *s) {
   /* Tasks will free the server object when they terminate */
   if (s->sender) task_terminate(s->sender);
   if (s->reply_handler) task_terminate(s->reply_handler);
+
+  // Allow the server to be freed. This unref pairs with the ref from mksrv.
+  srv_unref(s);
 }
 
 int srv_ref(server *s) {
@@ -2077,6 +2086,7 @@ bool_t xdr_node_list_1_1(XDR *xdrs, node_list_1_1 *objp) {
     case x_1_3:
     case x_1_4:
     case x_1_5:
+    case x_1_6:
       retval = xdr_array(xdrs, &x, (u_int *)&objp->node_list_len, NSERVERS,
                          sizeof(node_address), (xdrproc_t)xdr_node_address);
       objp->node_list_val = (node_address *)x;
@@ -2108,20 +2118,37 @@ bool_t xdr_pax_msg(XDR *xdrs, pax_msg *objp) {
     case x_1_1:
       if (!xdr_pax_msg_1_1(xdrs, (pax_msg_1_1 *)objp)) return FALSE;
       if (xdrs->x_op == XDR_DECODE) {
-        objp->delivered_msg = get_delivered_msg(); /* Use our own minimum */
-        objp->event_horizon = 0; /* Won't be used, so use 0 to spot if it is */
+        /* Use our own minimum */
+        objp->delivered_msg = get_delivered_msg();
+        /* Won't be used, so use 0 to spot if it is */
+        objp->event_horizon = 0;
+        /* Won't be used, so set as empty */
+        objp->requested_synode_app_data.synode_app_data_array_len = 0;
+        objp->requested_synode_app_data.synode_app_data_array_val = NULL;
       }
       return TRUE;
     case x_1_2:
     case x_1_3:
       if (!xdr_pax_msg_1_2(xdrs, (pax_msg_1_2 *)objp)) return FALSE;
       if (xdrs->x_op == XDR_DECODE) {
-        objp->event_horizon = 0; /* Won't be used, so use 0 to spot if it is */
+        /* Won't be used, so use 0 to spot if it is */
+        objp->event_horizon = 0;
+        /* Won't be used, so set as empty */
+        objp->requested_synode_app_data.synode_app_data_array_len = 0;
+        objp->requested_synode_app_data.synode_app_data_array_val = NULL;
       }
       return TRUE;
     case x_1_4:
     case x_1_5:
-      return xdr_pax_msg_1_4(xdrs, objp);
+      if (!xdr_pax_msg_1_4(xdrs, (pax_msg_1_4 *)objp)) return FALSE;
+      if (xdrs->x_op == XDR_DECODE) {
+        /* Won't be used, so set as empty */
+        objp->requested_synode_app_data.synode_app_data_array_len = 0;
+        objp->requested_synode_app_data.synode_app_data_array_val = NULL;
+      }
+      return TRUE;
+    case x_1_6:
+      return xdr_pax_msg_1_6(xdrs, objp);
     default:
       return FALSE;
   }
@@ -2140,6 +2167,7 @@ bool_t xdr_config(XDR *xdrs, config *objp) {
       return TRUE;
     case x_1_4:
     case x_1_5:
+    case x_1_6:
       return xdr_config_1_4(xdrs, objp);
     default:
       return FALSE;
