@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,7 @@
 /* clang-format off */
 PSI_mutex_key key_GR_LOCK_applier_module_run,
     key_GR_LOCK_applier_module_suspend,
+    key_GR_LOCK_autorejoin_module,
     key_GR_LOCK_cert_broadcast_run,
     key_GR_LOCK_cert_broadcast_dispatcher_run,
     key_GR_LOCK_certification_info,
@@ -49,6 +50,7 @@ PSI_mutex_key key_GR_LOCK_applier_module_run,
     key_GR_LOCK_pipeline_continuation,
     key_GR_LOCK_pipeline_stats_flow_control,
     key_GR_LOCK_pipeline_stats_transactions_waiting_apply,
+    key_GR_LOCK_plugin_modules_termination,
     key_GR_LOCK_plugin_online, key_GR_LOCK_plugin_running,
     key_GR_LOCK_primary_election_action_phase,
     key_GR_LOCK_primary_election_action_notification,
@@ -67,6 +69,7 @@ PSI_mutex_key key_GR_LOCK_applier_module_run,
     key_GR_LOCK_synchronized_queue,
     key_GR_LOCK_trx_unlocking,
     key_GR_LOCK_group_member_info_manager_update_lock,
+    key_GR_LOCK_group_member_info_update_lock,
     key_GR_LOCK_view_modification_wait,
     key_GR_LOCK_wait_ticket,
     key_GR_LOCK_write_lock_protection;
@@ -74,6 +77,7 @@ PSI_mutex_key key_GR_LOCK_applier_module_run,
 PSI_cond_key key_GR_COND_applier_module_run,
     key_GR_COND_applier_module_suspend,
     key_GR_COND_applier_module_wait,
+    key_GR_COND_autorejoin_module,
     key_GR_COND_cert_broadcast_dispatcher_run,
     key_GR_COND_cert_broadcast_run,
     key_GR_COND_count_down_latch,
@@ -103,6 +107,7 @@ PSI_cond_key key_GR_COND_applier_module_run,
     key_GR_COND_write_lock_protection;
 
 PSI_thread_key key_GR_THD_applier_module_receiver,
+    key_GR_THD_autorejoin,
     key_GR_THD_cert_broadcast,
     key_GR_THD_delayed_init,
     key_GR_THD_group_action_coordinator,
@@ -127,6 +132,9 @@ PSI_rwlock_key key_GR_RWLOCK_cert_stable_gtid_set,
 
 #ifdef HAVE_PSI_INTERFACE
 
+PSI_stage_info info_GR_STAGE_autorejoin = {
+    0, "Undergoing auto-rejoin procedure", PSI_FLAG_STAGE_PROGRESS,
+    PSI_DOCUMENT_ME};
 PSI_stage_info info_GR_STAGE_multi_primary_mode_switch_pending_transactions = {
     0, "Multi-primary Switch: waiting for pending transactions to finish",
     PSI_FLAG_STAGE_PROGRESS, PSI_DOCUMENT_ME};
@@ -177,11 +185,23 @@ PSI_stage_info info_GR_STAGE_single_primary_mode_switch_completion = {
     "Single-primary Switch: waiting for operation to complete on all members",
     PSI_FLAG_STAGE_PROGRESS, PSI_DOCUMENT_ME};
 
+PSI_stage_info info_GR_STAGE_module_executing = {
+    0, "Group Replication Module: Executing", 0, PSI_DOCUMENT_ME};
+PSI_stage_info info_GR_STAGE_module_suspending = {
+    0, "Group Replication Module: Suspending", 0, PSI_DOCUMENT_ME};
+PSI_stage_info info_GR_STAGE_recovery_connecting_to_donor = {
+    0, "Group Replication Recovery: Connecting to donor", 0, PSI_DOCUMENT_ME};
+PSI_stage_info info_GR_STAGE_recovery_transferring_state = {
+    0, "Group Replication Recovery: Transferring state from donor", 0,
+    PSI_DOCUMENT_ME};
+
 static PSI_mutex_info all_group_replication_psi_mutex_keys[] = {
     {&key_GR_LOCK_applier_module_run, "LOCK_applier_module_run",
      PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
     {&key_GR_LOCK_applier_module_suspend, "LOCK_applier_module_suspend",
      PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
+    {&key_GR_LOCK_autorejoin_module, "LOCK_autorejoin_module",
+     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
     {&key_GR_LOCK_cert_broadcast_run, "LOCK_certifier_broadcast_run",
      PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
     {&key_GR_LOCK_cert_broadcast_dispatcher_run,
@@ -231,6 +251,8 @@ static PSI_mutex_info all_group_replication_psi_mutex_keys[] = {
     {&key_GR_LOCK_pipeline_stats_transactions_waiting_apply,
      "LOCK_pipeline_stats_transactions_waiting_apply", PSI_FLAG_SINGLETON, 0,
      PSI_DOCUMENT_ME},
+    {&key_GR_LOCK_plugin_modules_termination, "LOCK_plugin_modules_termination",
+     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
     {&key_GR_LOCK_plugin_online, "LOCK_plugin_online", PSI_FLAG_SINGLETON, 0,
      PSI_DOCUMENT_ME},
     {&key_GR_LOCK_plugin_running, "LOCK_plugin_running", PSI_FLAG_SINGLETON, 0,
@@ -277,6 +299,9 @@ static PSI_mutex_info all_group_replication_psi_mutex_keys[] = {
     {&key_GR_LOCK_group_member_info_manager_update_lock,
      "LOCK_group_member_info_manager_update_lock", PSI_FLAG_SINGLETON, 0,
      PSI_DOCUMENT_ME},
+    {&key_GR_LOCK_group_member_info_update_lock,
+     "LOCK_group_member_info_update_lock", PSI_FLAG_SINGLETON, 0,
+     PSI_DOCUMENT_ME},
     {&key_GR_LOCK_view_modification_wait, "LOCK_view_modification_wait",
      PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
     {&key_GR_LOCK_wait_ticket, "LOCK_wait_ticket", PSI_FLAG_SINGLETON, 0,
@@ -355,7 +380,8 @@ static PSI_cond_info all_group_replication_psi_condition_keys[] = {
      PSI_DOCUMENT_ME},
     {&key_GR_COND_primary_promotion_policy, "COND_primary_promotion_policy",
      PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
-};
+    {&key_GR_COND_autorejoin_module, "COND_autorejoin_module",
+     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME}};
 
 static PSI_thread_info all_group_replication_psi_thread_keys[] = {
     {&key_GR_THD_applier_module_receiver, "THD_applier_module_receiver",
@@ -377,6 +403,8 @@ static PSI_thread_info all_group_replication_psi_thread_keys[] = {
     {&key_GR_THD_group_partition_handler, "THD_group_partition_handler",
      PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
     {&key_GR_THD_recovery, "THD_recovery", PSI_FLAG_SINGLETON, 0,
+     PSI_DOCUMENT_ME},
+    {&key_GR_THD_autorejoin, "THD_autorejoin", PSI_FLAG_SINGLETON, 0,
      PSI_DOCUMENT_ME}};
 
 static PSI_rwlock_info all_group_replication_psi_rwlock_keys[] = {
@@ -411,6 +439,7 @@ static PSI_rwlock_info all_group_replication_psi_rwlock_keys[] = {
      PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME}};
 
 static PSI_stage_info *all_group_replication_stages_keys[] = {
+    &info_GR_STAGE_autorejoin,
     &info_GR_STAGE_multi_primary_mode_switch_pending_transactions,
     &info_GR_STAGE_multi_primary_mode_switch_step_completion,
     &info_GR_STAGE_multi_primary_mode_switch_buffered_transactions,
@@ -426,7 +455,11 @@ static PSI_stage_info *all_group_replication_stages_keys[] = {
     &info_GR_STAGE_primary_switch_completion,
     &info_GR_STAGE_single_primary_mode_switch_checks,
     &info_GR_STAGE_single_primary_mode_switch_election,
-    &info_GR_STAGE_single_primary_mode_switch_completion};
+    &info_GR_STAGE_single_primary_mode_switch_completion,
+    &info_GR_STAGE_module_executing,
+    &info_GR_STAGE_module_suspending,
+    &info_GR_STAGE_recovery_connecting_to_donor,
+    &info_GR_STAGE_recovery_transferring_state};
 
 void register_group_replication_mutex_psi_keys(PSI_mutex_info mutexes[],
                                                size_t mutex_count) {
