@@ -36,7 +36,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 #define BACKUP_H
 
 
-#ident "$Id: 4d98ecc8d3e5108458a45c0f32fa4e0e6a164b4f $"
+#ident "$Id: de4836cd07a5ef2f38c7f14066a3198d110346e2 $"
 
 extern "C" {
 
@@ -63,10 +63,10 @@ int tokubackup_create_backup(const char *source_dirs[],
                              void *error_extra,
                              backup_exclude_copy_fun_t check_fun,
                              void *exclude_copy_extra,
-                             backup_before_stop_capt_fun_t bsc_fun = nullptr,
-                             void *bsc_extra = nullptr,
-                             backup_after_stop_capt_fun_t asc_fun = nullptr,
-                             void *asc_extra = nullptr)
+                             backup_before_stop_capt_fun_t bsc_fun,
+                             void *bsc_extra,
+                             backup_after_stop_capt_fun_t asc_fun,
+                             void *asc_extra)
     throw() __attribute__((visibility("default")));
 // Effect: Backup the directories in source_dirs into correspnding dest_dirs.
 // Periodically call poll_fun.
