@@ -32,7 +32,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
     along with Percona TokuBackup.  If not, see <http://www.gnu.org/licenses/>.
 ======= */
 
-#ident "$Id: d48a1ed63ede522ba95c9ee3a7b854e64a2a56c9 $"
+#ident "$Id: ed36c5762915ad7c792cd3ad610a17f4c3d1d8b7 $"
 
 #include "backup_debug.h"
 #include "check.h"
@@ -58,7 +58,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 
 template class std::vector<char *>;
 
-#if defined(DEBUG_HOTBACKUP)
+#if defined(DEBUG_HOTBACKUP) && DEBUG_HOTBACKUP
 #define WARN(string, arg) HotBackup::CopyWarn(string, arg)
 #define TRACE(string, arg) HotBackup::CopyTrace(string, arg)
 #define ERROR(string, arg) HotBackup::CopyError(string, arg)
