@@ -32,7 +32,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
     along with Percona TokuBackup.  If not, see <http://www.gnu.org/licenses/>.
 ======= */
 
-#ident "$Id: 07129478e78250144f9f29b42a6fa381fe8553fc $"
+#ident "$Id: 73bbbc42baf632b31e89143d70f82644e2eae598 $"
 
 #include "backup_debug.h"
 #include "file_hash_table.h"
@@ -56,7 +56,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 #include <unistd.h>
 #include "backup_helgrind.h"
 
-#if defined(DEBUG_HOTBACKUP)
+#if defined(DEBUG_HOTBACKUP) && DEBUG_HOTBACKUP
 #define WARN(string, arg) HotBackup::CaptureWarn(string, arg)
 #define TRACE(string, arg) HotBackup::CaptureTrace(string, arg)
 #define ERROR(string, arg) HotBackup::CaptureError(string, arg)
