@@ -32,7 +32,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
     along with Percona TokuBackup.  If not, see <http://www.gnu.org/licenses/>.
 ======= */
 
-#ident "$Id: afb476b5d0f985480843f449f7a147aad63279f9 $"
+#ident "$Id: 6043b111a55688d85c50b461bef20c5dbf8eeaf0 $"
 
 #include <errno.h>
 #include <pthread.h>
@@ -104,11 +104,6 @@ source_file::~source_file(void) throw() {
             check(r==0);
         }
     }
-
-    if (m_destination_file != NULL) {
-        delete m_destination_file;
-        m_destination_file = NULL;
-    }
 }
 
 ////////////////////////////////////////////////////////
@@ -221,7 +216,7 @@ int source_file::rename(const char * new_name) throw() {
 ////////////////////////////////////////////////////////
 //
 void source_file::add_reference(void) throw() {
-    m_reference_count.fetch_add(1);
+    __sync_fetch_and_add(&m_reference_count, 1);
 }
 
 ////////////////////////////////////////////////////////
@@ -229,7 +224,7 @@ void source_file::add_reference(void) throw() {
 void source_file::remove_reference(void) throw() {
     // TODO.  How can the code that decremented a reference count only if it was positive be right?  Under what conditions could someone be decrementing a refcount when they don't know that it's positive?
     check(m_reference_count>0);
-    m_reference_count.fetch_add(-1);
+    __sync_fetch_and_add(&m_reference_count, -1);
 }
 
 ////////////////////////////////////////////////////////
