@@ -32,7 +32,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
     along with Percona TokuBackup.  If not, see <http://www.gnu.org/licenses/>.
 ======= */
 
-#ident "$Id: 8e48e17f21fde2e239bd3c50cc28c83a93282f65 $"
+#ident "$Id: edd7273b00febd5b212494713d2120c5c7386195 $"
 
 #include <fcntl.h>
 #include <limits.h>
@@ -150,7 +150,7 @@ struct backup_thread_extra_t {
     int                expect_return_result;
 };
 
-static std::atomic_bool backup_is_done = {false};
+static volatile bool backup_is_done = false;
 
 static void* start_backup_thread_fun(void *backup_extra_v) {
     backup_thread_extra_t *backup_extra = (backup_thread_extra_t*)backup_extra_v;
@@ -350,7 +350,6 @@ int main (int argc, const char *argv[]) {
             argnum++;
         } else {
             new_argv[new_argc++] = argv[argnum];
-            argnum++;
         }
     }
     if (test_name==NULL) test_name=argv[0]; // make the function work with no arguments.
