@@ -1,6 +1,6 @@
 
 /*
- * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -37,10 +37,12 @@
 #include "plugin/x/src/xpl_log.h"
 #include "plugin/x/src/xpl_resultset.h"
 
+#include "my_systime.h"  // my_sleep()
+
 namespace xpl {
 
 ngs::Error_code Sql_data_context::init(const int client_port,
-                                       const ngs::Connection_type type) {
+                                       const Connection_type type) {
   ngs::Error_code error = init();
   if (error) return error;
 
@@ -138,8 +140,8 @@ bool Sql_data_context::kill() {
 }
 
 ngs::Error_code Sql_data_context::set_connection_type(
-    const ngs::Connection_type type) {
-  enum_vio_type vio_type = ngs::Connection_type_helper::convert_type(type);
+    const Connection_type type) {
+  enum_vio_type vio_type = Connection_type_helper::convert_type(type);
 
   if (NO_VIO_TYPE == vio_type)
     return ngs::Error(ER_X_SESSION, "Connection type not known. type=%i",
@@ -152,7 +154,7 @@ ngs::Error_code Sql_data_context::set_connection_type(
   return ngs::Error_code();
 }
 
-bool Sql_data_context::wait_api_ready(ngs::function<bool()> exiting) {
+bool Sql_data_context::wait_api_ready(std::function<bool()> exiting) {
   bool result = is_api_ready();
 
   while (!result && !exiting()) {
@@ -238,6 +240,19 @@ ngs::Error_code Sql_data_context::authenticate(
     std::string user_name = get_user_name();
     std::string host_or_ip = get_host_or_ip();
 
+    /*
+      Instead of modifying the current security context in switch_user()
+      method above, we must create a security_context to do the
+      security_context_lookup() on newly created security_context then set
+      that in the THD. Until that happens, we have to get the existing security
+      context and set that again in the THD. The latter opertion is nedded as
+      it may toggle the system_user flag in THD iff security_context has
+      SYSTEM_USER privilege.
+    */
+    MYSQL_SECURITY_CONTEXT scontext;
+    thd_get_security_context(get_thd(), &scontext);
+    thd_set_security_context(get_thd(), scontext);
+
 #ifdef HAVE_PSI_THREAD_INTERFACE
     PSI_THREAD_CALL(set_thread_account)
     (user_name.c_str(), static_cast<int>(user_name.length()),
@@ -453,8 +468,6 @@ ngs::Error_code Sql_data_context::attach() {
     return ngs::Error_code(ER_X_SERVICE_ERROR, "Internal error attaching");
   }
 
-  DBUG_ASSERT(nullptr == previous_thd);
-
   return {};
 }
 
@@ -511,4 +524,21 @@ ngs::Error_code Sql_data_context::execute_server_command(
   return error;
 }
 
+ngs::Error_code Sql_data_context::reset() {
+  COM_DATA data;
+  Callback_command_delegate deleg;
+  if (command_service_run_command(m_mysql_session, COM_RESET_CONNECTION, &data,
+                                  mysqld::get_charset_utf8mb4_general_ci(),
+                                  deleg.callbacks(), deleg.representation(),
+                                  &deleg)) {
+    return ngs::Error_code(ER_X_SERVICE_ERROR,
+                           "Internal error executing command");
+  }
+  const ngs::Error_code &error = deleg.get_error();
+  if (error)
+    log_debug("Error reseting sql session: (%i %s)", error.error,
+              error.message.c_str());
+  return error;
+}
+
 }  // namespace xpl
