@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -51,7 +51,7 @@ using std::runtime_error;
 using metadata_cache::ManagedInstance;
 IMPORT_LOG_FUNCTIONS()
 
-// if client wants a primary and there's none, we can wait up to this amount of
+// if client wants a PRIMARY and there's none, we can wait up to this amount of
 // seconds until giving up and disconnecting the client
 // TODO: possibly this should be made into a configurable option
 static const int kPrimaryFailoverTimeout = 10;
@@ -68,14 +68,14 @@ DestMetadataCacheGroup::ServerRole get_server_role_from_uri(
     throw runtime_error("Missing 'role' in routing destination specification");
 
   const std::string name = uri.at("role");
-  std::string name_lc = name;
-  std::transform(name.begin(), name.end(), name_lc.begin(), ::tolower);
+  std::string name_uc = name;
+  std::transform(name.begin(), name.end(), name_uc.begin(), ::toupper);
 
-  if (name_lc == "primary")
+  if (name_uc == "PRIMARY")
     return DestMetadataCacheGroup::ServerRole::Primary;
-  else if (name_lc == "secondary")
+  else if (name_uc == "SECONDARY")
     return DestMetadataCacheGroup::ServerRole::Secondary;
-  else if (name_lc == "primary_and_secondary")
+  else if (name_uc == "PRIMARY_AND_SECONDARY")
     return DestMetadataCacheGroup::ServerRole::PrimaryAndSecondary;
 
   throw std::runtime_error("Invalid server role in metadata cache routing '" +
@@ -86,11 +86,11 @@ std::string get_server_role_name(
     const DestMetadataCacheGroup::ServerRole role) {
   switch (role) {
     case DestMetadataCacheGroup::ServerRole::Primary:
-      return "primary";
+      return "PRIMARY";
     case DestMetadataCacheGroup::ServerRole::Secondary:
-      return "secondary";
+      return "SECONDARY";
     case DestMetadataCacheGroup::ServerRole::PrimaryAndSecondary:
-      return "primary_and_secondary";
+      return "PRIMARY_AND_SECONDARY";
   }
 
   return "unknown";
@@ -228,7 +228,6 @@ DestMetadataCacheGroup::get_available(
 
     primary_fallback = secondary == managed_servers_vec.end();
   }
-
   // if we are gathering the nodes for the decision about keeping existing
   // connections we look also at the disconnect_on_promoted_to_primary_ setting
   // if set to 'no' we need to allow primaries for role=SECONDARY
@@ -391,7 +390,6 @@ size_t DestMetadataCacheGroup::get_next_server(
     default:
       assert(0);
       // impossible we verify this in init()
-      ;
   }
 
   return result;
@@ -408,7 +406,7 @@ int DestMetadataCacheGroup::get_server_socket(
         log_warning(
             "No available servers found for '%s' %s routing",
             ha_replicaset_.c_str(),
-            server_role_ == ServerRole::Primary ? "primary" : "secondary");
+            server_role_ == ServerRole::Primary ? "PRIMARY" : "SECONDARY");
         return -1;
       }
 
@@ -473,15 +471,8 @@ void DestMetadataCacheGroup::notify(
 }
 
 void DestMetadataCacheGroup::start(const mysql_harness::PluginFuncEnv *env) {
-  // before using metadata-cache we need to make sure it is initialized
-  // we probe for it max 1000 times each millisecond here, if that's not enough
-  // error out with an exception
-  unsigned count = 0;
+  // before using metadata-cache we need to wait for it to be initialized
   while (!cache_api_->is_initialized() && (!env || is_running(env))) {
-    if (count++ > 1000) {
-      throw std::runtime_error(
-          "Timed out waiting for metadata-cache to initialize.");
-    }
     std::this_thread::sleep_for(std::chrono::milliseconds(1));
   }
 
