@@ -1,4 +1,4 @@
--- Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+-- Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License, version 2.0,
@@ -49,70 +49,6 @@ INSERT IGNORE INTO mysql.user
 (host, user, select_priv, plugin, authentication_string, ssl_cipher, x509_issuer, x509_subject)
 VALUES ('localhost','mysql.infoschema','Y','caching_sha2_password','$A$005$THISISACOMBINATIONOFINVALIDSALTANDPASSWORDTHATMUSTNEVERBRBEUSED','','','');
 
-FLUSH PRIVILEGES;
-
-# Move distributed grant tables to default engine during upgrade, remember
-# which tables was moved so they can be moved back after upgrade
-SET @had_distributed_user =
-  (SELECT COUNT(table_name) FROM information_schema.tables
-     WHERE table_schema = 'mysql' AND table_name = 'user' AND
-           table_type = 'BASE TABLE' AND engine = 'NDBCLUSTER');
-SET @cmd="ALTER TABLE mysql.user ENGINE=InnoDB";
-SET @str = IF(@had_distributed_user > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
-SET @had_distributed_db =
-  (SELECT COUNT(table_name) FROM information_schema.tables
-     WHERE table_schema = 'mysql' AND table_name = 'db' AND
-           table_type = 'BASE TABLE' AND engine = 'NDBCLUSTER');
-SET @cmd="ALTER TABLE mysql.db ENGINE=InnoDB";
-SET @str = IF(@had_distributed_db > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
-SET @had_distributed_tables_priv =
-  (SELECT COUNT(table_name) FROM information_schema.tables
-     WHERE table_schema = 'mysql' AND table_name = 'tables_priv' AND
-           table_type = 'BASE TABLE' AND engine = 'NDBCLUSTER');
-SET @cmd="ALTER TABLE mysql.tables_priv ENGINE=InnoDB";
-SET @str = IF(@had_distributed_tables_priv > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
-SET @had_distributed_columns_priv =
-  (SELECT COUNT(table_name) FROM information_schema.tables
-     WHERE table_schema = 'mysql' AND table_name = 'columns_priv' AND
-           table_type = 'BASE TABLE' AND engine = 'NDBCLUSTER');
-SET @cmd="ALTER TABLE mysql.columns_priv ENGINE=InnoDB";
-SET @str = IF(@had_distributed_columns_priv > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
-SET @had_distributed_procs_priv =
-  (SELECT COUNT(table_name) FROM information_schema.tables
-     WHERE table_schema = 'mysql' AND table_name = 'procs_priv' AND
-           table_type = 'BASE TABLE' AND engine = 'NDBCLUSTER');
-SET @cmd="ALTER TABLE mysql.procs_priv ENGINE=InnoDB";
-SET @str = IF(@had_distributed_procs_priv > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
-SET @had_distributed_proxies_priv =
-  (SELECT COUNT(table_name) FROM information_schema.tables
-     WHERE table_schema = 'mysql' AND table_name = 'proxies_priv' AND
-           table_type = 'BASE TABLE' AND engine = 'NDBCLUSTER' );
-SET @cmd="ALTER TABLE mysql.proxies_priv ENGINE=InnoDB";
-SET @str = IF(@had_distributed_proxies_priv > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
 ALTER TABLE user add File_priv enum('N','Y') COLLATE utf8_general_ci NOT NULL;
 
 # Detect whether or not we had the Grant_priv column
@@ -290,8 +226,7 @@ ALTER TABLE db
   MODIFY  Create_tmp_table_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,
   MODIFY  Lock_tables_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL;
 
-ALTER TABLE func
-  ENGINE=MyISAM, CONVERT TO CHARACTER SET utf8 COLLATE utf8_bin;
+ALTER TABLE func CONVERT TO CHARACTER SET utf8 COLLATE utf8_bin;
 ALTER TABLE func
   MODIFY type enum ('function','aggregate') COLLATE utf8_general_ci NOT NULL;
 
@@ -701,12 +636,30 @@ INSERT INTO global_grants SELECT user, host, 'APPLICATION_PASSWORD_ADMIN', IF(gr
 FROM mysql.user WHERE Create_user_priv = 'Y' AND @hadApplicationPasswordAdminPriv = 0;
 COMMIT;
 
+-- Add the privilege SYSTEM_USER for every user who has privilege SET_USER_ID privilege
+SET @hadSystemUserPriv = (SELECT COUNT(*) FROM global_grants WHERE priv = 'SYSTEM_USER');
+INSERT INTO global_grants SELECT user, host, 'SYSTEM_USER',
+IF (WITH_GRANT_OPTION = 'Y', 'Y', 'N') FROM global_grants WHERE priv = 'SET_USER_ID' AND @hadSystemUserPriv = 0;
+COMMIT;
+
+-- Add the privilege SYSTEM_USER for every user who has the privilege SUPER
+-- provided that there isn't a user who already has the privilege SYSTEM_USER
+SET @hadSystemUserPriv = (SELECT COUNT(*) FROM global_grants WHERE priv = 'SYSTEM_USER');
+INSERT INTO global_grants SELECT user, host, 'SYSTEM_USER', IF(grant_priv = 'Y', 'Y', 'N')
+FROM mysql.user WHERE super_priv = 'Y' AND @hadSystemUserPriv = 0;
+COMMIT;
+
+-- Add the privilege TABLE_ENCRYPTION_ADMIN for every user who has the privilege SUPER
+-- provided that there isn't a user who already has the privilige TABLE_ENCRYPTION_ADMIN.
+SET @hadTableEncryptionAdminPriv = (SELECT COUNT(*) FROM global_grants WHERE priv = 'TABLE_ENCRYPTION_ADMIN');
+INSERT INTO global_grants SELECT user, host, 'TABLE_ENCRYPTION_ADMIN', IF(grant_priv = 'Y', 'Y', 'N')
+FROM mysql.user WHERE super_priv = 'Y' AND @hadTableEncryptionAdminPriv = 0;
+COMMIT;
+
 # Activate the new, possible modified privilege tables
 # This should not be needed, but gives us some extra testing that the above
 # changes was correct
 
-flush privileges;
-
 ALTER TABLE slave_master_info ADD Ssl_crl TEXT CHARACTER SET utf8 COLLATE utf8_bin COMMENT 'The file used for the Certificate Revocation List (CRL)';
 ALTER TABLE slave_master_info ADD Ssl_crlpath TEXT CHARACTER SET utf8 COLLATE utf8_bin COMMENT 'The path used for Certificate Revocation List (CRL) files';
 ALTER TABLE slave_master_info STATS_PERSISTENT=0;
@@ -748,6 +701,8 @@ ALTER TABLE slave_master_info ADD Public_key_path TEXT CHARACTER SET utf8 COLLAT
 # The Get_public_key field at slave_master_info should be added after the slave_master_info field
 ALTER TABLE slave_master_info ADD Get_public_key BOOLEAN NOT NULL COMMENT 'Preference to get public key from master.';
 
+ALTER TABLE slave_master_info ADD Network_namespace TEXT CHARACTER SET utf8 COLLATE utf8_bin COMMENT 'Network namespace used for communication with the master server.';
+
 # If the order of column Public_key_path, Get_public_key is wrong, this will correct the order in
 # slave_master_info table.
 ALTER TABLE slave_master_info
@@ -757,10 +712,25 @@ ALTER TABLE slave_master_info
   MODIFY COLUMN Get_public_key BOOLEAN NOT NULL COMMENT 'Preference to get public key from master.'
   AFTER Public_key_path;
 
+ALTER TABLE slave_master_info
+  MODIFY COLUMN Network_namespace TEXT CHARACTER SET utf8 COLLATE utf8_bin
+  COMMENT 'Network namespace used for communication with the master server.'
+  AFTER Get_public_key;
+
+#
+# Drop legacy NDB distributed privileges function & procedures
+#
+DROP function  IF EXISTS mysql.mysql_cluster_privileges_are_distributed;
+DROP procedure IF EXISTS mysql.mysql_cluster_backup_privileges;
+DROP procedure IF EXISTS mysql.mysql_cluster_move_grant_tables;
+DROP procedure IF EXISTS mysql.mysql_cluster_restore_local_privileges;
+DROP procedure IF EXISTS mysql.mysql_cluster_restore_privileges;
+DROP procedure IF EXISTS mysql.mysql_cluster_restore_privileges_from_local;
+DROP procedure IF EXISTS mysql.mysql_cluster_move_privileges;
+
 #
 # Alter mysql.ndb_binlog_index only if it exists already.
 #
-
 SET @cmd="ALTER TABLE ndb_binlog_index
   ADD COLUMN next_position BIGINT UNSIGNED NOT NULL";
 
@@ -847,43 +817,6 @@ ALTER TABLE columns_priv ENGINE=InnoDB STATS_PERSISTENT=0;
 ALTER TABLE procs_priv ENGINE=InnoDB STATS_PERSISTENT=0;
 ALTER TABLE proxies_priv ENGINE=InnoDB STATS_PERSISTENT=0;
 
-# Move any distributed grant tables back to NDB after upgrade
-SET @cmd="ALTER TABLE mysql.user ENGINE=NDB";
-SET @str = IF(@had_distributed_user > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
-SET @cmd="ALTER TABLE mysql.db ENGINE=NDB";
-SET @str = IF(@had_distributed_db > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
-SET @cmd="ALTER TABLE mysql.tables_priv ENGINE=NDB";
-SET @str = IF(@had_distributed_tables_priv > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
-SET @cmd="ALTER TABLE mysql.columns_priv ENGINE=NDB";
-SET @str = IF(@had_distributed_columns_priv > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
-SET @cmd="ALTER TABLE mysql.procs_priv ENGINE=NDB";
-SET @str = IF(@had_distributed_procs_priv > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
-SET @cmd="ALTER TABLE mysql.proxies_priv ENGINE=NDB";
-SET @str = IF(@had_distributed_proxies_priv > 0, @cmd, "SET @dummy = 0");
-PREPARE stmt FROM @str;
-EXECUTE stmt;
-DROP PREPARE stmt;
-
 --
 -- CREATE_ROLE_ACL and DROP_ROLE_ACL
 --
@@ -1063,71 +996,219 @@ INSERT IGNORE INTO mysql.global_grants VALUES ('mysql.session', 'localhost', 'SY
 
 INSERT IGNORE INTO mysql.global_grants VALUES ('mysql.session', 'localhost', 'SESSION_VARIABLES_ADMIN', 'N');
 
-FLUSH PRIVILEGES;
+# mysql.session is granted the SUPER and other administrative privileges.
+# This user should not be modified inadvertently. Therefore, server grants
+# the SYSTEM_USER privilege to this user at the time of initialization or
+# upgrade.
+INSERT IGNORE INTO mysql.global_grants VALUES ('mysql.session', 'localhost', 'SYSTEM_USER', 'N');
+
+set @is_mysql_encrypted = (select ENCRYPTION from information_schema.INNODB_TABLESPACES where NAME='mysql');
+
+SET @str="ALTER TABLE mysql.db ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 
 # Move all system tables with InnoDB storage engine to mysql tablespace.
-# Move privilege tables to InnoDB only if they were not in NDB.
 SET @cmd="ALTER TABLE mysql.db TABLESPACE = mysql";
-SET @str = IF(@had_distributed_db > 0, "SET @dummy = 0", @cmd);
-PREPARE stmt FROM @str;
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
+
+SET @str="ALTER TABLE mysql.user ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
 EXECUTE stmt;
 DROP PREPARE stmt;
 
 SET @cmd="ALTER TABLE mysql.user TABLESPACE = mysql";
-SET @str = IF(@had_distributed_user > 0, "SET @dummy = 0", @cmd);
-PREPARE stmt FROM @str;
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
+
+SET @str="ALTER TABLE mysql.tables_priv ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
 EXECUTE stmt;
 DROP PREPARE stmt;
 
 SET @cmd="ALTER TABLE mysql.tables_priv TABLESPACE = mysql";
-SET @str = IF(@had_distributed_tables_priv > 0, "SET @dummy = 0", @cmd);
-PREPARE stmt FROM @str;
+PREPARE stmt FROM @cmd;
 EXECUTE stmt;
 DROP PREPARE stmt;
 
+SET @str="ALTER TABLE mysql.columns_priv ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 
 SET @cmd="ALTER TABLE mysql.columns_priv TABLESPACE = mysql";
-SET @str = IF(@had_distributed_columns_priv > 0, "SET @dummy = 0", @cmd);
-PREPARE stmt FROM @str;
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
+
+SET @str="ALTER TABLE mysql.procs_priv ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
 EXECUTE stmt;
 DROP PREPARE stmt;
 
 SET @cmd="ALTER TABLE mysql.procs_priv TABLESPACE = mysql";
-SET @str = IF(@had_distributed_procs_priv > 0, "SET @dummy = 0", @cmd);
-PREPARE stmt FROM @str;
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
+
+SET @str="ALTER TABLE mysql.proxies_priv ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
 EXECUTE stmt;
 DROP PREPARE stmt;
 
 SET @cmd="ALTER TABLE mysql.proxies_priv TABLESPACE = mysql";
-SET @str = IF(@had_distributed_proxies_priv > 0, "SET @dummy = 0", @cmd);
-PREPARE stmt FROM @str;
+PREPARE stmt FROM @cmd;
 EXECUTE stmt;
 DROP PREPARE stmt;
 
 # Alter mysql.ndb_binlog_index only if it exists already.
+SET @str_enc="ALTER TABLE ndb_binlog_index ENCRYPTION='Y'";
+SET @str = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str_enc);
+SET @cmd = IF(@have_ndb_binlog_index = 1, @str, 'SET @dummy = 0');
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
+
 SET @cmd="ALTER TABLE ndb_binlog_index TABLESPACE = mysql";
 SET @str = IF(@have_ndb_binlog_index = 1, @cmd, 'SET @dummy = 0');
 PREPARE stmt FROM @str;
 EXECUTE stmt;
 DROP PREPARE stmt;
 
+SET @str="ALTER TABLE mysql.func ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.func TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.plugin ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.plugin TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.servers ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.servers TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.help_topic ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.help_topic TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.help_category ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.help_category TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.help_relation ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.help_relation TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.help_keyword ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.help_keyword TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.time_zone_name ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.time_zone_name TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.time_zone ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.time_zone TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.time_zone_transition ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.time_zone_transition TABLESPACE = mysql;
+
+SET @str ="ALTER TABLE mysql.time_zone_transition_type ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.time_zone_transition_type TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.time_zone_leap_second ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.time_zone_leap_second TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.slave_relay_log_info ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.slave_relay_log_info TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.slave_master_info ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.slave_master_info TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.slave_worker_info ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.slave_worker_info TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.gtid_executed ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.gtid_executed TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.server_cost ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.server_cost TABLESPACE = mysql;
+
+SET @str="ALTER TABLE mysql.engine_cost ENCRYPTION='Y'";
+SET @cmd = IF(STRCMP(@is_mysql_encrypted,'Y'), 'SET @dummy = 0', @str);
+PREPARE stmt FROM @cmd;
+EXECUTE stmt;
+DROP PREPARE stmt;
 ALTER TABLE mysql.engine_cost TABLESPACE = mysql;
 
 SET @@session.sql_mode = @old_sql_mode;
