@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -51,13 +51,20 @@ struct HA_CREATE_INFO;
 struct LEX_USER;
 template <class T>
 class List;
-
 typedef struct user_conn USER_CONN;
 class Security_context;
+class ACL_USER;
 struct TABLE;
+struct MEM_ROOT;
 struct TABLE_LIST;
 enum class role_enum;
 enum class Consumer_type;
+class LEX_GRANT_AS;
+
+namespace consts {
+extern const std::string mysql;
+extern const std::string system_user;
+}  // namespace consts
 
 /** user, host tuple which reference either acl_cache or g_default_roles */
 typedef std::pair<LEX_CSTRING, LEX_CSTRING> Auth_id_ref;
@@ -182,8 +189,7 @@ class IS_internal_schema_access : public ACL_internal_schema_access {
 
 /* Data Structures */
 
-extern const char *command_array[];
-extern uint command_lengths[];
+extern const std::vector<std::string> global_acls_vector;
 
 enum mysql_db_table_field {
   MYSQL_DB_FIELD_HOST = 0,
@@ -673,10 +679,12 @@ bool change_password(THD *thd, LEX_USER *user, char *password,
 bool mysql_create_user(THD *thd, List<LEX_USER> &list, bool if_not_exists,
                        bool is_role);
 bool mysql_alter_user(THD *thd, List<LEX_USER> &list, bool if_exists);
-bool mysql_drop_user(THD *thd, List<LEX_USER> &list, bool if_exists);
+bool mysql_drop_user(THD *thd, List<LEX_USER> &list, bool if_exists,
+                     bool drop_role);
 bool mysql_rename_user(THD *thd, List<LEX_USER> &list);
 
 /* sql_auth_cache */
+void init_acl_memory();
 int wild_case_compare(CHARSET_INFO *cs, const char *str, const char *wildstr);
 int wild_case_compare(CHARSET_INFO *cs, const char *str, size_t str_len,
                       const char *wildstr, size_t wildstr_len);
@@ -695,19 +703,18 @@ bool acl_getroot(THD *thd, Security_context *sctx, char *user, char *host,
 bool check_acl_tables_intact(THD *thd);
 bool check_acl_tables_intact(THD *thd, TABLE_LIST *tables);
 void notify_flush_event(THD *thd);
+bool wildcard_db_grant_exists();
 
 /* sql_authorization */
 bool skip_grant_tables();
-bool has_grant_role_privilege(THD *thd);
-bool has_revoke_role_privilege(THD *thd);
-int mysql_set_active_role_none(THD *thd);
-int mysql_set_role_default(THD *thd);
-int mysql_set_active_role_all(THD *thd, const List<LEX_USER> *except_users);
-int mysql_set_active_role(THD *thd, const List<LEX_USER> *role_list);
+bool mysql_set_active_role_none(THD *thd);
+bool mysql_set_role_default(THD *thd);
+bool mysql_set_active_role_all(THD *thd, const List<LEX_USER> *except_users);
+bool mysql_set_active_role(THD *thd, const List<LEX_USER> *role_list);
 bool mysql_grant(THD *thd, const char *db, List<LEX_USER> &list, ulong rights,
                  bool revoke_grant, bool is_proxy,
                  const List<LEX_CSTRING> &dynamic_privilege,
-                 bool grant_all_current_privileges);
+                 bool grant_all_current_privileges, LEX_GRANT_AS *grant_as);
 bool mysql_routine_grant(THD *thd, TABLE_LIST *table, bool is_proc,
                          List<LEX_USER> &user_list, ulong rights, bool revoke,
                          bool write_to_binlog);
@@ -774,6 +781,7 @@ bool check_access(THD *thd, ulong want_access, const char *db, ulong *save_priv,
 bool check_table_access(THD *thd, ulong requirements, TABLE_LIST *tables,
                         bool any_combination_of_privileges_will_do, uint number,
                         bool no_errors);
+bool check_table_encryption_admin_access(THD *thd);
 bool mysql_grant_role(THD *thd, const List<LEX_USER> *users,
                       const List<LEX_USER> *roles, bool with_admin_opt);
 bool mysql_revoke_role(THD *thd, const List<LEX_USER> *users,
@@ -791,8 +799,6 @@ bool has_grant_role_privilege(THD *thd, const LEX_CSTRING &role_name,
 Auth_id_ref create_authid_from(const LEX_USER *user);
 std::string create_authid_str_from(const LEX_USER *user);
 void append_identifier(String *packet, const char *name, size_t length);
-void append_identifier_with_q(int q, String *packet, const char *name,
-                              size_t length);
 bool is_role_id(LEX_USER *authid);
 void shutdown_acl_cache();
 bool is_granted_role(LEX_CSTRING user, LEX_CSTRING host, LEX_CSTRING role,
@@ -813,7 +819,8 @@ typedef enum ssl_artifacts_status {
 ulong get_global_acl_cache_size();
 #if defined(HAVE_OPENSSL) && !defined(HAVE_WOLFSSL)
 extern bool opt_auto_generate_certs;
-bool do_auto_cert_generation(ssl_artifacts_status auto_detection_status);
+bool do_auto_cert_generation(ssl_artifacts_status auto_detection_status,
+                             char **ssl_ca, char **ssl_key, char **ssl_cert);
 #endif /* HAVE_OPENSSL && !HAVE_WOLFSSL */
 
 #define DEFAULT_SSL_CA_CERT "ca.pem"
@@ -822,8 +829,8 @@ bool do_auto_cert_generation(ssl_artifacts_status auto_detection_status);
 #define DEFAULT_SSL_SERVER_KEY "server-key.pem"
 
 void update_mandatory_roles(void);
-bool check_authorization_id_string(const char *buffer, size_t length);
-void func_current_role(THD *thd, String *active_role);
+bool check_authorization_id_string(THD *thd, const char *buffer, size_t length);
+void func_current_role(const THD *thd, String *active_role);
 
 extern volatile uint32 global_password_history, global_password_reuse_interval;
 
@@ -898,9 +905,12 @@ class Security_context_factory {
         m_static_privileges(static_priv),
         m_drop_policy(drop_policy) {}
 
-  Sctx_ptr<Security_context> create();
+  Sctx_ptr<Security_context> create(MEM_ROOT *mem_root);
+  void apply_policies_to_security_ctx();
 
  private:
+  bool apply_pre_constructed_policies(Security_context *sctx);
+
   THD *m_thd;
   std::string m_user;
   std::string m_host;
@@ -920,6 +930,10 @@ class Default_local_authid : public Create_authid<Default_local_authid> {
   const THD *m_thd;
 };
 
+/**
+  Grant the privilege temporarily to the in-memory global privleges map.
+  This class is not thread safe.
+ */
 class Grant_temporary_dynamic_privileges
     : public Grant_privileges<Grant_temporary_dynamic_privileges> {
  public:
@@ -951,9 +965,54 @@ class Grant_temporary_static_privileges
   bool grant_privileges(Security_context *sctx);
 
  private:
+  /** THD handle */
   const THD *m_thd;
+
+  /** Privileges */
   const ulong m_privs;
 };
 
 bool operator==(const LEX_CSTRING &a, const LEX_CSTRING &b);
+bool is_partial_revoke_exists(THD *thd);
+void set_system_user_flag(THD *thd, bool check_for_main_security_ctx = false);
+
+/**
+  Storage container for default auth ids. Default roles are only weakly
+  depending on ACL_USERs. You can retain a default role even if the
+  corresponding ACL_USER is missing in the acl_cache.
+*/
+class Auth_id {
+ public:
+  Auth_id();
+  Auth_id(const char *user, size_t user_len, const char *host, size_t host_len);
+  Auth_id(const Auth_id_ref &id);
+  Auth_id(const LEX_CSTRING &user, const LEX_CSTRING &host);
+  Auth_id(const std::string &user, const std::string &host);
+  Auth_id(const LEX_USER *lex_user);
+  Auth_id(const ACL_USER *acl_user);
+
+  ~Auth_id();
+  Auth_id(const Auth_id &id);
+  Auth_id &operator=(const Auth_id &) = default;
+
+  bool operator<(const Auth_id &id) const;
+  void auth_str(std::string *out) const;
+  std::string auth_str() const;
+  const std::string &user() const;
+  const std::string &host() const;
+
+ private:
+  /** User part */
+  std::string m_user;
+  /** Host part */
+  std::string m_host;
+};
+
+/*
+  As of now Role_id is an alias of Auth_id.
+  We may extend the Auth_id as Role_id once
+  more substances are added to latter.
+*/
+using Role_id = Auth_id;
+
 #endif /* AUTH_COMMON_INCLUDED */
