@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2017, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -67,6 +67,7 @@ class Per_thread_connection_handler : public Connection_handler {
   // Status variables related to Per_thread_connection_handler
   static ulong blocked_pthread_count;  // Protected by LOCK_thread_cache.
   static ulong slow_launch_threads;
+  static bool shrink_cache;  // Protected by LOCK_thread_cache
   // System variable
   static ulong max_blocked_pthreads;
 
@@ -83,6 +84,13 @@ class Per_thread_connection_handler : public Connection_handler {
   */
   static Channel_info *block_until_new_connection();
 
+  /**
+    Modify the thread cache size.
+
+    @param thread_cache_size size of thread cache.
+  */
+  static void modify_thread_cache_size(const ulong thread_cache_size);
+
   Per_thread_connection_handler() {}
   virtual ~Per_thread_connection_handler() {}
 
