@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -835,6 +835,23 @@ class Dictionary_client {
                                 bool *exists)
       MY_ATTRIBUTE((warn_unused_result));
 
+  /**
+    Check if schema contains check constraint with specified name.
+
+    @param        schema            Schema containing the check constraint.
+    @param        check_cons_name   Name of the check constraint.
+    @param  [out] exists            Set to true if check constraint with
+                                    the name provided exists in the
+                                    schema, false otherwise.
+
+    @retval      false    No error.
+    @retval      true     Error.
+  */
+
+  bool check_constraint_exists(const Schema &schema,
+                               const String_type &check_cons_name,
+                               bool *exists);
+
   /**
     Fetch the names of the components in the schema. Hidden components are
     ignored. E.g., Object with dd::Table::hidden() == true will be ignored.
