@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,7 +28,6 @@
 #include <memory>
 #include <string>
 
-#include "binary_log_types.h"
 #include "lex_string.h"
 #include "my_alloc.h"
 #include "my_dbug.h"
@@ -380,31 +379,14 @@ static void fill_dd_view_tables(View *view_obj, const TABLE_LIST *view,
   for (const TABLE_LIST *table = query_tables; table != nullptr;
        table = table->next_global) {
     /*
-      Skip tables if
-        - It is not directly referred by the view OR
-        - It is a temporary table OR
-        - It is a data-directly table OR
-        - If it is not a user or information_schema schema table.
+      Skip if table is not directly referred by a view or if table is a
+      data-dictionary or temporary table.
     */
-    {
-      if (table->referencing_view && table->referencing_view != view)
-        continue;
-      else if (is_temporary_table(const_cast<TABLE_LIST *>(table)))
-        continue;
-      else if (get_dictionary()->is_dd_schema_name(table->get_db_name()))
-        continue;
-      else {
-        LEX_STRING db_name = {const_cast<char *>(table->get_db_name()),
-                              strlen(table->get_db_name())};
-        LEX_STRING table_name = {const_cast<char *>(table->get_table_name()),
-                                 strlen(table->get_table_name())};
-
-        TABLE_CATEGORY table_category = get_table_category(db_name, table_name);
-        if (table_category != TABLE_CATEGORY_USER &&
-            table_category != TABLE_CATEGORY_INFORMATION)
-          continue;
-      }
-    }
+    if ((table->referencing_view && table->referencing_view != view) ||
+        get_dictionary()->is_dd_table_name(table->get_db_name(),
+                                           table->get_table_name()) ||
+        is_temporary_table(const_cast<TABLE_LIST *>(table)))
+      continue;
 
     LEX_CSTRING db_name;
     LEX_CSTRING table_name;
