@@ -62,7 +62,7 @@ class Sdi_rcontext;
 class Sdi_wcontext;
 
 static const std::set<String_type> default_valid_option_keys = {
-    "column_format", "geom_type",         "interval_count",
+    "column_format", "geom_type",         "interval_count", "not_secondary",
     "storage",       "treat_bit_as_char", "zip_dict_id"};
 
 static const std::set<String_type> default_valid_se_private_data_keys = {
