@@ -1,7 +1,7 @@
 #ifndef ITEM_SUM_INCLUDED
 #define ITEM_SUM_INCLUDED
 
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,7 +32,7 @@
 #include <sys/types.h>
 #include <utility>  // std::forward
 
-#include "binary_log_types.h"
+#include "field_types.h"  // enum_field_types
 #include "m_ctype.h"
 #include "m_string.h"
 #include "my_compiler.h"
@@ -601,7 +601,8 @@ class Item_sum : public Item_result_field {
     used_tables_cache = 0;
     forced_const = true;
   }
-  void print(String *str, enum_query_type query_type) override;
+  void print(const THD *thd, String *str,
+             enum_query_type query_type) const override;
   void fix_num_length_and_dec();
   bool eq(const Item *item, bool binary_cmp) const override;
   /**
@@ -915,7 +916,7 @@ class Item_sum_num : public Item_sum {
   bool fix_fields(THD *, Item **) override;
   longlong val_int() override {
     DBUG_ASSERT(fixed == 1);
-    return (longlong)rint(val_real()); /* Real as default */
+    return llrint_with_overflow_check(val_real()); /* Real as default */
   }
   String *val_str(String *str) override;
   my_decimal *val_decimal(my_decimal *) override;
@@ -1109,7 +1110,7 @@ class Item_sum_num_field : public Item_sum_hybrid_field {
  public:
   longlong val_int() override {
     /* can't be fix_fields()ed */
-    return (longlong)rint(val_real());
+    return llrint_with_overflow_check(val_real());
   }
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override {
     return get_date_from_numeric(ltime, fuzzydate); /* Decimal or real */
@@ -1180,10 +1181,11 @@ class Item_sum_json : public Item_sum {
  public:
   /**
     Construct an Item_sum_json instance.
-    @param args  arguments to forward to Item_sum's constructor
+    @param parent_args arguments to forward to Item_sum's constructor
   */
   template <typename... Args>
-  Item_sum_json(Args &&... args) : Item_sum(std::forward<Args>(args)...) {
+  Item_sum_json(Args &&... parent_args)
+      : Item_sum(std::forward<Args>(parent_args)...) {
     set_data_type_json();
   }
 
@@ -1279,7 +1281,7 @@ class Item_sum_avg final : public Item_sum_sum {
   bool add() override;
   double val_real() override;
   // In SPs we might force the "wrong" type with select into a declare variable
-  longlong val_int() override { return llrint(val_real()); }
+  longlong val_int() override { return llrint_with_overflow_check(val_real()); }
   my_decimal *val_decimal(my_decimal *) override;
   String *val_str(String *str) override;
   void reset_field() override;
@@ -1881,7 +1883,8 @@ class Item_udf_sum : public Item_sum {
   void reset_field() override {}
   void update_field() override {}
   void cleanup() override;
-  void print(String *str, enum_query_type query_type) override;
+  void print(const THD *thd, String *str,
+             enum_query_type query_type) const override;
 };
 
 class Item_sum_udf_float final : public Item_udf_sum {
@@ -1945,7 +1948,7 @@ class Item_sum_udf_str final : public Item_udf_sum {
   String *val_str(String *) override;
   double val_real() override {
     int err_not_used;
-    char *end_not_used;
+    const char *end_not_used;
     String *res;
     res = val_str(&str_value);
     return res ? my_strntod(res->charset(), (char *)res->ptr(), res->length(),
@@ -1954,13 +1957,12 @@ class Item_sum_udf_str final : public Item_udf_sum {
   }
   longlong val_int() override {
     int err_not_used;
-    char *end;
     String *res;
     const CHARSET_INFO *cs;
 
     if (!(res = val_str(&str_value))) return 0; /* Null value */
     cs = res->charset();
-    end = (char *)res->ptr() + res->length();
+    const char *end = res->ptr() + res->length();
     return cs->cset->strtoll10(cs, res->ptr(), &end, &err_not_used);
   }
   my_decimal *val_decimal(my_decimal *dec) override;
@@ -2087,10 +2089,9 @@ class Item_func_group_concat final : public Item_sum {
   }
   longlong val_int() override {
     String *res;
-    char *end_ptr;
     int error;
     if (!(res = val_str(&str_value))) return (longlong)0;
-    end_ptr = (char *)res->ptr() + res->length();
+    const char *end_ptr = res->ptr() + res->length();
     return my_strtoll10(res->ptr(), &end_ptr, &error);
   }
   my_decimal *val_decimal(my_decimal *decimal_value) override {
@@ -2105,7 +2106,8 @@ class Item_func_group_concat final : public Item_sum {
   String *val_str(String *str) override;
   Item *copy_or_same(THD *thd) override;
   void no_rows_in_result() override {}
-  void print(String *str, enum_query_type query_type) override;
+  void print(const THD *thd, String *str,
+             enum_query_type query_type) const override;
   bool change_context_processor(uchar *cntx) override {
     context = reinterpret_cast<Name_resolution_context *>(cntx);
     return false;
@@ -2136,8 +2138,8 @@ class Item_non_framing_wf : public Item_sum {
   Item_non_framing_wf(const POS &pos, PT_window *w) : Item_sum(pos, w) {}
   Item_non_framing_wf(const POS &pos, Item *a, PT_window *w)
       : Item_sum(pos, a, w) {}
-  Item_non_framing_wf(const POS &pos, PT_item_list *args, PT_window *w)
-      : Item_sum(pos, args, w) {}
+  Item_non_framing_wf(const POS &pos, PT_item_list *opt_list, PT_window *w)
+      : Item_sum(pos, opt_list, w) {}
   Item_non_framing_wf(THD *thd, Item_non_framing_wf *i) : Item_sum(thd, i) {}
 
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override {
@@ -2382,9 +2384,9 @@ class Item_lead_lag : public Item_non_framing_wf {
 
  public:
   Item_lead_lag(const POS &pos, bool lead,
-                PT_item_list *args,  // [0] expr, [1] offset, [2] default
+                PT_item_list *opt_list,  // [0] expr, [1] offset, [2] default
                 enum_null_treatment null_treatment, PT_window *w)
-      : Item_non_framing_wf(pos, args, w),
+      : Item_non_framing_wf(pos, opt_list, w),
         m_null_treatment(null_treatment),
         m_is_lead(lead),
         m_n(0),
@@ -2590,14 +2592,15 @@ class Item_nth_value : public Item_sum {
 */
 class Item_func_grouping : public Item_int_func {
  public:
-  Item_func_grouping(const POS &pos, PT_item_list *a) : Item_int_func(pos, a) {}
+  Item_func_grouping(const POS &pos, PT_item_list *a) : Item_int_func(pos, a) {
+    set_grouping_func();
+  }
   const char *func_name() const override { return "grouping"; }
   enum Functype functype() const override { return GROUPING_FUNC; }
   longlong val_int() override;
   bool aggregate_check_group(uchar *arg) override;
   bool fix_fields(THD *thd, Item **ref) override;
   void update_used_tables() override;
-  bool has_grouping_func_processor(uchar *) override { return true; }
 };
 
 #endif /* ITEM_SUM_INCLUDED */
