@@ -1,7 +1,7 @@
 #ifndef LEX_INCLUDED
 #define LEX_INCLUDED
 
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -225,6 +225,7 @@ static const SYMBOL symbols[] = {
     {SYM("ENCLOSED", ENCLOSED)},
     {SYM("END", END)},
     {SYM("ENDS", ENDS_SYM)},
+    {SYM("ENFORCED", ENFORCED_SYM)},
     {SYM("ENGINE", ENGINE_SYM)},
     {SYM("ENGINES", ENGINES_SYM)},
     {SYM("ENUM", ENUM_SYM)},
@@ -442,6 +443,7 @@ static const SYMBOL symbols[] = {
     {SYM("NDBCLUSTER", NDBCLUSTER_SYM)},
     {SYM("NCHAR", NCHAR_SYM)},
     {SYM("NESTED", NESTED_SYM)},
+    {SYM("NETWORK_NAMESPACE", NETWORK_NAMESPACE_SYM)},
     {SYM("NEVER", NEVER_SYM)},
     {SYM("NEW", NEW_SYM)},
     {SYM("NEXT", NEXT_SYM)},
@@ -461,6 +463,7 @@ static const SYMBOL symbols[] = {
     {SYM("NVARCHAR", NVARCHAR_SYM)},
     {SYM("OF", OF_SYM)},
     {SYM("OFFSET", OFFSET_SYM)},
+    {SYM("OJ", OJ_SYM)},
     {SYM("OLD", OLD_SYM)},
     {SYM("ON", ON_SYM)},
     {SYM("ONE", ONE_SYM)},
@@ -590,6 +593,7 @@ static const SYMBOL symbols[] = {
     {SYM("SCHEMAS", DATABASES)},
     {SYM("SECOND", SECOND_SYM)},
     {SYM("SECOND_MICROSECOND", SECOND_MICROSECOND_SYM)},
+    {SYM("SECONDARY", SECONDARY_SYM)},
     {SYM("SECONDARY_ENGINE", SECONDARY_ENGINE_SYM)},
     {SYM("SECONDARY_LOAD", SECONDARY_LOAD_SYM)},
     {SYM("SECONDARY_UNLOAD", SECONDARY_UNLOAD_SYM)},
