@@ -1146,8 +1146,8 @@ bool Global_backup_lock::acquire(THD *thd) {
 
   DBUG_ENTER("Global_backup_lock::acquire");
 
-  DBUG_ASSERT(m_lock == nullptr &&
-              !thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "", "",
+  DBUG_ASSERT(m_lock == nullptr);
+  DBUG_ASSERT(!thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "", "",
                                                             MDL_SHARED));
 
   MDL_REQUEST_INIT(&mdl_request, m_namespace, "", "", MDL_SHARED, MDL_EXPLICIT);
@@ -1170,8 +1170,9 @@ bool Global_backup_lock::acquire(THD *thd) {
 void Global_backup_lock::release(THD *thd) noexcept {
   DBUG_ENTER("Global_backup_lock::release");
 
-  DBUG_ASSERT(m_lock != nullptr && thd->mdl_context.owns_equal_or_stronger_lock(
-                                       m_namespace, "", "", MDL_SHARED));
+  DBUG_ASSERT(m_lock != nullptr);
+  DBUG_ASSERT(thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "", "",
+                                                           MDL_SHARED));
 
   thd->mdl_context.release_lock(m_lock);
 
