@@ -1,6 +1,6 @@
 #ifndef MDL_H
 #define MDL_H
-/* Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -35,6 +35,7 @@
 #include "my_inttypes.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
+#include "my_systime.h"  // Timout_type
 #include "mysql/components/services/mysql_cond_bits.h"
 #include "mysql/components/services/mysql_mutex_bits.h"
 #include "mysql/components/services/mysql_rwlock_bits.h"
@@ -164,7 +165,7 @@ class MDL_context_owner {
     Get random seed specific to this THD to be used for initialization
     of PRNG for the MDL_context.
   */
-  virtual uint get_rand_seed() = 0;
+  virtual uint get_rand_seed() const = 0;
 };
 
 /**
@@ -374,6 +375,7 @@ struct MDL_key {
        and some administrative statements.
      - RESOURCE_GROUPS is for resource groups.
      - FOREIGN_KEY is for foreign key names.
+     - CHECK_CONSTRAINT is for check constraint names.
     Note that requests waiting for user-level locks get special
     treatment - waiting is aborted if connection to client is lost.
   */
@@ -395,6 +397,7 @@ struct MDL_key {
     BACKUP_LOCK, /* Oracle LOCK INSTANCE FOR BACKUP */
     RESOURCE_GROUPS,
     FOREIGN_KEY,
+    CHECK_CONSTRAINT,
     BACKUP_TABLES, /* Percona LOCK TABLES FOR BACKUP */
     /* This should be the last ! */
     NAMESPACE_END
@@ -1321,7 +1324,9 @@ class MDL_wait {
   MDL_wait();
   ~MDL_wait();
 
-  enum enum_wait_status { EMPTY = 0, GRANTED, VICTIM, TIMEOUT, KILLED };
+  // WS_EMPTY since EMPTY conflicts with #define in system headers on some
+  // platforms.
+  enum enum_wait_status { WS_EMPTY = 0, GRANTED, VICTIM, TIMEOUT, KILLED };
 
   bool set_status(enum_wait_status result_arg);
   enum_wait_status get_status();
@@ -1395,10 +1400,11 @@ class MDL_context {
   void destroy();
 
   bool try_acquire_lock(MDL_request *mdl_request);
-  bool acquire_lock(MDL_request *mdl_request, ulong lock_wait_timeout);
-  bool acquire_locks(MDL_request_list *requests, ulong lock_wait_timeout);
+  bool acquire_lock(MDL_request *mdl_request, Timeout_type lock_wait_timeout);
+  bool acquire_locks(MDL_request_list *requests,
+                     Timeout_type lock_wait_timeout);
   bool upgrade_shared_lock(MDL_ticket *mdl_ticket, enum_mdl_type new_type,
-                           ulong lock_wait_timeout);
+                           Timeout_type lock_wait_timeout);
 
   bool clone_ticket(MDL_request *mdl_request);
 
