@@ -58,42 +58,38 @@ class SELECT_LEX_UNIT;
   This array must be in sync with Extra_tag enum.
 */
 static const char *json_extra_tags[ET_total] = {
-    NULL,                             // ET_none
-    "using_temporary_table",          // ET_USING_TEMPORARY
-    "using_filesort",                 // ET_USING_FILESORT
-    "index_condition",                // ET_USING_INDEX_CONDITION
-    NULL,                             // ET_USING
-    "range_checked_for_each_record",  // ET_RANGE_CHECKED_FOR_EACH_RECORD
-    "pushed_condition",               // ET_USING_WHERE_WITH_PUSHED_CONDITION
-    "using_where",                    // ET_USING_WHERE
-    "not_exists",                     // ET_NOT_EXISTS
-    "using_MRR",                      // ET_USING_MRR
-    "using_index",                    // ET_USING_INDEX
-    "full_scan_on_NULL_key",          // ET_FULL_SCAN_ON_NULL_KEY
-    "skip_open_table",                // ET_SKIP_OPEN_TABLE
-    "open_frm_only",                  // ET_OPEN_FRM_ONLY
-    "open_full_table",                // ET_OPEN_FULL_TABLE
-    "scanned_databases",              // ET_SCANNED_DATABASES
-    "using_index_for_group_by",       // ET_USING_INDEX_FOR_GROUP_BY
-    "using_index_for_skip_scan",      // ET_USING_INDEX_FOR_SKIP_SCAN
-    "distinct",                       // ET_DISTINCT
-    "loosescan",                      // ET_LOOSESCAN
-    NULL,                             // ET_START_TEMPORARY
-    NULL,                             // ET_END_TEMPORARY
-    "first_match",                    // ET_FIRST_MATCH
-    NULL,                             // ET_MATERIALIZE
-    NULL,                             // ET_START_MATERIALIZE
-    NULL,                             // ET_END_MATERIALIZE
-    NULL,                             // ET_SCAN
-    "using_join_buffer",              // ET_USING_JOIN_BUFFER
-    "const_row_not_found",            // ET_CONST_ROW_NOT_FOUND
-    "unique_row_not_found",           // ET_UNIQUE_ROW_NOT_FOUND
-    "impossible_on_condition",        // ET_IMPOSSIBLE_ON_CONDITION
-    "pushed_join",                    // ET_PUSHED_JOIN
-    "ft_hints",                       // ET_FT_HINTS
-    "backward_index_scan",            // ET_BACKWARD_SCAN
-    "recursive",                      // ET_RECURSIVE
-    "table_function",                 // ET_TABLE_FUNCTION
+    NULL,                                  // ET_none
+    "using_temporary_table",               // ET_USING_TEMPORARY
+    "using_filesort",                      // ET_USING_FILESORT
+    "index_condition",                     // ET_USING_INDEX_CONDITION
+    NULL,                                  // ET_USING
+    "range_checked_for_each_record",       // ET_RANGE_CHECKED_FOR_EACH_RECORD
+    "pushed_condition",                    // ET_USING_PUSHED_CONDITION
+    "using_where",                         // ET_USING_WHERE
+    "not_exists",                          // ET_NOT_EXISTS
+    "using_MRR",                           // ET_USING_MRR
+    "using_index",                         // ET_USING_INDEX
+    "full_scan_on_NULL_key",               // ET_FULL_SCAN_ON_NULL_KEY
+    "using_index_for_group_by",            // ET_USING_INDEX_FOR_GROUP_BY
+    "using_index_for_skip_scan",           // ET_USING_INDEX_FOR_SKIP_SCAN
+    "distinct",                            // ET_DISTINCT
+    "loosescan",                           // ET_LOOSESCAN
+    NULL,                                  // ET_START_TEMPORARY
+    NULL,                                  // ET_END_TEMPORARY
+    "first_match",                         // ET_FIRST_MATCH
+    NULL,                                  // ET_MATERIALIZE
+    NULL,                                  // ET_START_MATERIALIZE
+    NULL,                                  // ET_END_MATERIALIZE
+    NULL,                                  // ET_SCAN
+    "using_join_buffer",                   // ET_USING_JOIN_BUFFER
+    "const_row_not_found",                 // ET_CONST_ROW_NOT_FOUND
+    "unique_row_not_found",                // ET_UNIQUE_ROW_NOT_FOUND
+    "impossible_on_condition",             // ET_IMPOSSIBLE_ON_CONDITION
+    "pushed_join",                         // ET_PUSHED_JOIN
+    "ft_hints",                            // ET_FT_HINTS
+    "backward_index_scan",                 // ET_BACKWARD_SCAN
+    "recursive",                           // ET_RECURSIVE
+    "table_function",                      // ET_TABLE_FUNCTION
     "skip_records_in_range_due_to_force",  // ET_SKIP_RECORDS_IN_RANGE
     "using_secondary_engine",              // ET_USING_SECONDARY_ENGINE
     "rematerialize"                        // ET_REMATERIALIZE
@@ -1226,7 +1222,8 @@ class window_ctx : public join_ctx {
         for (; ord != NULL; ord = ord->next) {
           String str;
           (*ord->item)
-              ->print_for_order(&str, (enum_query_type)(QT_NO_DB | QT_NO_TABLE),
+              ->print_for_order(current_thd, &str,
+                                (enum_query_type)(QT_NO_DB | QT_NO_TABLE),
                                 ord->used_alias);
           if (ord->direction == ORDER_DESC)
             str.append(STRING_WITH_LEN(" desc"));
