@@ -2,7 +2,7 @@
 #define PARTITION_HANDLER_INCLUDED
 
 /*
-   Copyright (c) 2005, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -855,7 +855,7 @@ class Partition_helper {
   */
   virtual int read_range_first_in_part(uint part, uchar *buf,
                                        const key_range *start_key,
-                                       const key_range *end_key, bool eq_range,
+                                       const key_range *end_key,
                                        bool sorted) = 0;
   /**
     Do read_range_next in the specified partition.
