@@ -1037,7 +1037,7 @@ int Binlog_sender::send_format_description_event(File_reader *reader,
   // Let's check if next event is Start encryption event
   // If we go outside the file read_event will also return an error
   const auto binlog_pos_after_fdle = reader->position();
-  if (read_event(reader, &event_ptr, &event_len)) {
+  if (read_event(reader, &event_ptr, &event_len, true)) {
     reader->seek(binlog_pos_after_fdle);
     set_last_pos(binlog_pos_after_fdle);
     DBUG_RETURN(0);
@@ -1121,7 +1121,8 @@ const char *Binlog_sender::log_read_error_msg(
 }
 
 inline int Binlog_sender::read_event(File_reader *reader, uchar **event_ptr,
-                                     uint32 *event_len) {
+                                     uint32 *event_len,
+                                     bool readahead MY_ATTRIBUTE((unused))) {
   DBUG_ENTER("Binlog_sender::read_event");
 
   if (reset_transmit_packet(0, 0)) DBUG_RETURN(1);
@@ -1159,7 +1160,7 @@ inline int Binlog_sender::read_event(File_reader *reader, uchar **event_ptr,
   DBUG_PRINT("info", ("Read event %s", Log_event::get_type_str(Log_event_type(
                                            (*event_ptr)[EVENT_TYPE_OFFSET]))));
 #ifndef DBUG_OFF
-  if (check_event_count()) DBUG_RETURN(1);
+  if (!readahead && check_event_count()) DBUG_RETURN(1);
 #endif
   DBUG_RETURN(0);
 }
