@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -233,9 +233,8 @@ const uint info_slave_worker_table_pk_field_indexes[] = {
     0,
 };
 
-Slave_worker::Slave_worker(Relay_log_info *rli
+Slave_worker::Slave_worker(Relay_log_info *rli,
 #ifdef HAVE_PSI_INTERFACE
-                           ,
                            PSI_mutex_key *param_key_info_run_lock,
                            PSI_mutex_key *param_key_info_data_lock,
                            PSI_mutex_key *param_key_info_sleep_lock,
@@ -243,19 +242,16 @@ Slave_worker::Slave_worker(Relay_log_info *rli
                            PSI_mutex_key *param_key_info_data_cond,
                            PSI_mutex_key *param_key_info_start_cond,
                            PSI_mutex_key *param_key_info_stop_cond,
-                           PSI_mutex_key *param_key_info_sleep_cond
+                           PSI_mutex_key *param_key_info_sleep_cond,
 #endif
-                           ,
                            uint param_id, const char *param_channel)
-    : Relay_log_info(false
+    : Relay_log_info(false,
 #ifdef HAVE_PSI_INTERFACE
-                     ,
                      param_key_info_run_lock, param_key_info_data_lock,
                      param_key_info_sleep_lock, param_key_info_thd_lock,
                      param_key_info_data_cond, param_key_info_start_cond,
-                     param_key_info_stop_cond, param_key_info_sleep_cond
+                     param_key_info_stop_cond, param_key_info_sleep_cond,
 #endif
-                     ,
                      param_id + 1, param_channel, true),
       c_rli(rli),
       curr_group_exec_parts(key_memory_db_worker_hash_entry),
