@@ -411,7 +411,7 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
         key read.
       */
 
-      m_lex->unit->cleanup(true);
+      m_lex->unit->cleanup(thd, true);
 
       /* Here we also commit or rollback the current statement. */
 
@@ -956,7 +956,7 @@ bool sp_instr_stmt::execute(THD *thd, uint *nextp) {
   return rc || thd->is_error();
 }
 
-void sp_instr_stmt::print(String *str) {
+void sp_instr_stmt::print(const THD *, String *str) {
   /* stmt CMD "..." */
   if (str->reserve(SP_STMT_PRINT_MAXLEN + SP_INSTR_UINT_MAXLEN + 8)) return;
   qs_append(STRING_WITH_LEN("stmt"), str);
@@ -1021,7 +1021,7 @@ bool sp_instr_set::exec_core(THD *thd, uint *nextp) {
   return true;
 }
 
-void sp_instr_set::print(String *str) {
+void sp_instr_set::print(const THD *thd, String *str) {
   /* set name@offset ... */
   size_t rsrv = SP_INSTR_UINT_MAXLEN + 6;
   sp_variable *var = m_parsing_ctx->find_variable(m_offset);
@@ -1036,7 +1036,7 @@ void sp_instr_set::print(String *str) {
   }
   qs_append(m_offset, str);
   qs_append(' ', str);
-  m_value_item->print(str, QT_TO_ARGUMENT_CHARSET);
+  m_value_item->print(thd, str, QT_TO_ARGUMENT_CHARSET);
 }
 
   ///////////////////////////////////////////////////////////////////////////
@@ -1067,11 +1067,11 @@ bool sp_instr_set_trigger_field::exec_core(THD *thd, uint *nextp) {
   return error;
 }
 
-void sp_instr_set_trigger_field::print(String *str) {
+void sp_instr_set_trigger_field::print(const THD *thd, String *str) {
   str->append(STRING_WITH_LEN("set_trigger_field "));
-  m_trigger_field->print(str, QT_ORDINARY);
+  m_trigger_field->print(thd, str, QT_ORDINARY);
   str->append(STRING_WITH_LEN(":="));
-  m_value_item->print(str, QT_TO_ARGUMENT_CHARSET);
+  m_value_item->print(thd, str, QT_TO_ARGUMENT_CHARSET);
 }
 
 bool sp_instr_set_trigger_field::on_after_expr_parsing(THD *thd) {
@@ -1109,7 +1109,7 @@ void sp_instr_set_trigger_field::cleanup_before_parsing(THD *thd) {
 PSI_statement_info sp_instr_jump::psi_info = {0, "jump", 0, PSI_DOCUMENT_ME};
 #endif
 
-void sp_instr_jump::print(String *str) {
+void sp_instr_jump::print(const THD *, String *str) {
   /* jump dest */
   if (str->reserve(SP_INSTR_UINT_MAXLEN + 5)) return;
   qs_append(STRING_WITH_LEN("jump "), str);
@@ -1168,7 +1168,7 @@ bool sp_instr_jump_if_not::exec_core(THD *thd, uint *nextp) {
   return false;
 }
 
-void sp_instr_jump_if_not::print(String *str) {
+void sp_instr_jump_if_not::print(const THD *thd, String *str) {
   /* jump_if_not dest(cont) ... */
   if (str->reserve(2 * SP_INSTR_UINT_MAXLEN + 14 +
                    32))  // Add some for the expr. too
@@ -1178,7 +1178,7 @@ void sp_instr_jump_if_not::print(String *str) {
   qs_append('(', str);
   qs_append(m_cont_dest, str);
   qs_append(STRING_WITH_LEN(") "), str);
-  m_expr_item->print(str, QT_ORDINARY);
+  m_expr_item->print(thd, str, QT_ORDINARY);
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1250,7 +1250,7 @@ bool sp_instr_jump_case_when::exec_core(THD *thd, uint *nextp) {
   return false;
 }
 
-void sp_instr_jump_case_when::print(String *str) {
+void sp_instr_jump_case_when::print(const THD *thd, String *str) {
   /* jump_if_not dest(cont) ... */
   if (str->reserve(2 * SP_INSTR_UINT_MAXLEN + 14 +
                    32))  // Add some for the expr. too
@@ -1260,7 +1260,7 @@ void sp_instr_jump_case_when::print(String *str) {
   qs_append('(', str);
   qs_append(m_cont_dest, str);
   qs_append(STRING_WITH_LEN(") "), str);
-  m_eq_item->print(str, QT_ORDINARY);
+  m_eq_item->print(thd, str, QT_ORDINARY);
 }
 
 bool sp_instr_jump_case_when::on_after_expr_parsing(THD *thd) {
@@ -1329,14 +1329,14 @@ bool sp_instr_freturn::exec_core(THD *thd, uint *nextp) {
   return thd->sp_runtime_ctx->set_return_value(thd, &m_expr_item);
 }
 
-void sp_instr_freturn::print(String *str) {
+void sp_instr_freturn::print(const THD *thd, String *str) {
   /* freturn type expr... */
   if (str->reserve(1024 + 8 + 32))  // Add some for the expr. too
     return;
   qs_append(STRING_WITH_LEN("freturn "), str);
   qs_append((uint)m_return_field_type, str);
   qs_append(' ', str);
-  m_expr_item->print(str, QT_ORDINARY);
+  m_expr_item->print(thd, str, QT_ORDINARY);
 }
 
   ///////////////////////////////////////////////////////////////////////////
@@ -1372,7 +1372,7 @@ bool sp_instr_hpush_jump::execute(THD *thd, uint *nextp) {
   return thd->sp_runtime_ctx->push_handler(m_handler, get_ip() + 1);
 }
 
-void sp_instr_hpush_jump::print(String *str) {
+void sp_instr_hpush_jump::print(const THD *, String *str) {
   /* hpush_jump dest fsize type */
   if (str->reserve(SP_INSTR_UINT_MAXLEN * 2 + 21)) return;
 
@@ -1461,7 +1461,7 @@ bool sp_instr_hreturn::execute(THD *thd, uint *nextp) {
   return false;
 }
 
-void sp_instr_hreturn::print(String *str) {
+void sp_instr_hreturn::print(const THD *, String *str) {
   /* hreturn framesize dest */
   if (str->reserve(SP_INSTR_UINT_MAXLEN * 2 + 9)) return;
   qs_append(STRING_WITH_LEN("hreturn "), str);
@@ -1517,7 +1517,7 @@ bool sp_instr_cpush::exec_core(THD *thd, uint *) {
   return c ? c->open(thd) : true;
 }
 
-void sp_instr_cpush::print(String *str) {
+void sp_instr_cpush::print(const THD *, String *str) {
   const LEX_STRING *cursor_name = m_parsing_ctx->find_cursor(m_cursor_idx);
 
   size_t rsrv = SP_INSTR_UINT_MAXLEN + 7 + m_cursor_query.length + 1;
@@ -1550,7 +1550,7 @@ bool sp_instr_cpop::execute(THD *thd, uint *nextp) {
   return false;
 }
 
-void sp_instr_cpop::print(String *str) {
+void sp_instr_cpop::print(const THD *, String *str) {
   /* cpop count */
   if (str->reserve(SP_INSTR_UINT_MAXLEN + 5)) return;
   qs_append(STRING_WITH_LEN("cpop "), str);
@@ -1606,7 +1606,7 @@ bool sp_instr_copen::execute(THD *thd, uint *nextp) {
   return rc;
 }
 
-void sp_instr_copen::print(String *str) {
+void sp_instr_copen::print(const THD *, String *str) {
   const LEX_STRING *cursor_name = m_parsing_ctx->find_cursor(m_cursor_idx);
 
   /* copen name@offset */
@@ -1642,7 +1642,7 @@ bool sp_instr_cclose::execute(THD *thd, uint *nextp) {
   return c ? c->close() : true;
 }
 
-void sp_instr_cclose::print(String *str) {
+void sp_instr_cclose::print(const THD *, String *str) {
   const LEX_STRING *cursor_name = m_parsing_ctx->find_cursor(m_cursor_idx);
 
   /* cclose name@offset */
@@ -1678,7 +1678,7 @@ bool sp_instr_cfetch::execute(THD *thd, uint *nextp) {
   return c ? c->fetch(&m_varlist) : true;
 }
 
-void sp_instr_cfetch::print(String *str) {
+void sp_instr_cfetch::print(const THD *, String *str) {
   List_iterator_fast<sp_variable> li(m_varlist);
   sp_variable *pv;
   const LEX_STRING *cursor_name = m_parsing_ctx->find_cursor(m_cursor_idx);
@@ -1711,7 +1711,7 @@ void sp_instr_cfetch::print(String *str) {
 PSI_statement_info sp_instr_error::psi_info = {0, "error", 0, PSI_DOCUMENT_ME};
 #endif
 
-void sp_instr_error::print(String *str) {
+void sp_instr_error::print(const THD *, String *str) {
   /* error code */
   if (str->reserve(SP_INSTR_UINT_MAXLEN + 6)) return;
   qs_append(STRING_WITH_LEN("error "), str);
@@ -1750,7 +1750,7 @@ bool sp_instr_set_case_expr::exec_core(THD *thd, uint *nextp) {
   return false;
 }
 
-void sp_instr_set_case_expr::print(String *str) {
+void sp_instr_set_case_expr::print(const THD *thd, String *str) {
   /* set_case_expr (cont) id ... */
   str->reserve(2 * SP_INSTR_UINT_MAXLEN + 18 +
                32);  // Add some extra for expr too
@@ -1759,7 +1759,7 @@ void sp_instr_set_case_expr::print(String *str) {
   qs_append(STRING_WITH_LEN(") "), str);
   qs_append(m_case_expr_id, str);
   qs_append(' ', str);
-  m_expr_item->print(str, QT_ORDINARY);
+  m_expr_item->print(thd, str, QT_ORDINARY);
 }
 
 uint sp_instr_set_case_expr::opt_mark(sp_head *sp, List<sp_instr> *leads) {
