@@ -1,4 +1,4 @@
-/* Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,7 +27,7 @@
 #include <string.h>
 #include <sys/types.h>
 
-#include "binary_log_types.h"
+#include "field_types.h"
 #include "lex_string.h"
 #include "m_string.h"
 #include "my_alloc.h"
@@ -63,7 +63,7 @@ struct TABLE_LIST;
 */
 class sp_printable {
  public:
-  virtual void print(String *str) = 0;
+  virtual void print(const THD *thd, String *str) = 0;
 
   virtual ~sp_printable() {}
 };
@@ -485,7 +485,7 @@ class sp_instr_stmt : public sp_lex_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
@@ -539,7 +539,7 @@ class sp_instr_set : public sp_lex_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
@@ -599,7 +599,7 @@ class sp_instr_set_trigger_field : public sp_lex_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
@@ -656,7 +656,7 @@ class sp_instr_freturn : public sp_lex_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -732,7 +732,7 @@ class sp_instr_jump : public sp_instr, public sp_branch_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -885,7 +885,7 @@ class sp_instr_jump_if_not : public sp_lex_branch_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
@@ -929,7 +929,7 @@ class sp_instr_set_case_expr : public sp_lex_branch_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -1009,7 +1009,7 @@ class sp_instr_jump_case_when : public sp_lex_branch_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_lex_instr implementation.
@@ -1086,7 +1086,7 @@ class sp_instr_hpush_jump : public sp_instr_jump {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -1140,7 +1140,9 @@ class sp_instr_hpop : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str) { str->append(STRING_WITH_LEN("hpop")); }
+  virtual void print(const THD *, String *str) {
+    str->append(STRING_WITH_LEN("hpop"));
+  }
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -1166,7 +1168,7 @@ class sp_instr_hreturn : public sp_instr_jump {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -1232,7 +1234,7 @@ class sp_instr_cpush : public sp_lex_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -1293,7 +1295,7 @@ class sp_instr_cpop : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -1327,7 +1329,7 @@ class sp_instr_copen : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -1363,7 +1365,7 @@ class sp_instr_cclose : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -1399,7 +1401,7 @@ class sp_instr_cfetch : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
@@ -1441,7 +1443,7 @@ class sp_instr_error : public sp_instr {
   // sp_printable implementation.
   /////////////////////////////////////////////////////////////////////////
 
-  virtual void print(String *str);
+  virtual void print(const THD *thd, String *str);
 
   /////////////////////////////////////////////////////////////////////////
   // sp_instr implementation.
