@@ -83,7 +83,7 @@
 static bool parse_int(longlong *to, const char *from, size_t from_length)
 {
   int error;
-  char *end= const_cast<char *>(from + from_length);
+  const char *end= from + from_length;
   *to= my_strtoll10(from, &end, &error);
   return error != 0 || end != from + from_length;
 }
@@ -513,15 +513,15 @@ static const yytype_uint8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   164,   164,   166,   168,   173,   179,   187,   188,   189,
-     190,   191,   192,   193,   198,   218,   219,   223,   229,   238,
-     239,   243,   249,   258,   259,   263,   269,   278,   282,   290,
-     298,   299,   307,   308,   312,   319,   326,   333,   340,   347,
-     354,   361,   368,   375,   384,   385,   389,   396,   397,   398,
-     399,   403,   405,   410,   416,   423,   429,   439,   446,   456,
-     460,   464,   471,   475,   479,   486,   490,   494,   498,   505,
-     509,   513,   517,   524,   533,   542,   551,   552,   556,   571,
-     606,   607,   611,   619,   620
+       0,   163,   163,   165,   167,   172,   178,   186,   187,   188,
+     189,   190,   191,   192,   197,   217,   218,   222,   228,   237,
+     238,   242,   248,   257,   258,   262,   268,   277,   281,   289,
+     297,   298,   306,   307,   311,   318,   325,   332,   339,   346,
+     353,   360,   367,   374,   383,   384,   388,   395,   396,   397,
+     398,   402,   404,   409,   415,   422,   428,   438,   445,   455,
+     459,   463,   470,   474,   478,   485,   489,   493,   497,   504,
+     508,   512,   516,   523,   532,   541,   550,   551,   555,   570,
+     605,   606,   610,   618,   619
 };
 #endif
 
@@ -1423,25 +1423,25 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 165 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 164 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { *ret= (yyvsp[-1].hint_list); }
 #line 1429 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 3:
-#line 167 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 166 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { *ret= (yyvsp[-2].hint_list); }
 #line 1435 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 4:
-#line 169 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 168 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { *ret= NULL; }
 #line 1441 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 5:
-#line 174 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 173 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_list)= NEW_PTN PT_hint_list(thd->mem_root);
             if ((yyval.hint_list) == NULL || (yyval.hint_list)->push_back((yyvsp[0].hint)))
@@ -1451,7 +1451,7 @@ yyreduce:
     break;
 
   case 6:
-#line 180 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 179 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyvsp[-1].hint_list)->push_back((yyvsp[0].hint));
             (yyval.hint_list)= (yyvsp[-1].hint_list);
@@ -1460,10 +1460,10 @@ yyreduce:
     break;
 
   case 14:
-#line 199 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 198 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             longlong n;
-            if (parse_int(&n, (yyvsp[-1].hint_string).str, (yyvsp[-1].hint_string).length) || n > UINT_MAX32)
+            if (parse_int(&n, (yyvsp[-1].lexer.hint_string).str, (yyvsp[-1].lexer.hint_string).length) || n > UINT_MAX32)
             {
               scanner->syntax_warning(ER_THD(thd,
                                              ER_WARN_BAD_MAX_EXECUTION_TIME));
@@ -1480,13 +1480,13 @@ yyreduce:
     break;
 
   case 15:
-#line 218 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 217 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.hint_param_table_list).init(thd->mem_root); }
 #line 1486 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 17:
-#line 224 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 223 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_param_table_list).init(thd->mem_root);
             if ((yyval.hint_param_table_list).push_back((yyvsp[0].hint_param_table)))
@@ -1496,7 +1496,7 @@ yyreduce:
     break;
 
   case 18:
-#line 230 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 229 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             if ((yyvsp[-2].hint_param_table_list).push_back((yyvsp[0].hint_param_table)))
               YYABORT; // OOM
@@ -1506,13 +1506,13 @@ yyreduce:
     break;
 
   case 19:
-#line 238 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 237 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.hint_param_table_list).init(thd->mem_root); }
 #line 1512 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 21:
-#line 244 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 243 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_param_table_list).init(thd->mem_root);
             if ((yyval.hint_param_table_list).push_back((yyvsp[0].hint_param_table)))
@@ -1522,7 +1522,7 @@ yyreduce:
     break;
 
   case 22:
-#line 250 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 249 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             if ((yyvsp[-2].hint_param_table_list).push_back((yyvsp[0].hint_param_table)))
               YYABORT; // OOM
@@ -1532,25 +1532,25 @@ yyreduce:
     break;
 
   case 23:
-#line 258 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 257 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.hint_param_index_list).init(thd->mem_root); }
 #line 1538 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 25:
-#line 264 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 263 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_param_index_list).init(thd->mem_root);
-            if ((yyval.hint_param_index_list).push_back((yyvsp[0].hint_string)))
+            if ((yyval.hint_param_index_list).push_back((yyvsp[0].lexer.hint_string)))
               YYABORT; // OOM
           }
 #line 1548 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 26:
-#line 270 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 269 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            if ((yyvsp[-2].hint_param_index_list).push_back((yyvsp[0].hint_string)))
+            if ((yyvsp[-2].hint_param_index_list).push_back((yyvsp[0].lexer.hint_string)))
               YYABORT; // OOM
             (yyval.hint_param_index_list)= (yyvsp[-2].hint_param_index_list);
           }
@@ -1558,42 +1558,42 @@ yyreduce:
     break;
 
   case 28:
-#line 283 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 282 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint_param_table).table= (yyvsp[0].hint_string);
+            (yyval.hint_param_table).table= (yyvsp[0].lexer.hint_string);
             (yyval.hint_param_table).opt_query_block= NULL_CSTR;
           }
 #line 1567 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 29:
-#line 291 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 290 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint_param_table).table= (yyvsp[-1].hint_string);
-            (yyval.hint_param_table).opt_query_block= (yyvsp[0].hint_string);
+            (yyval.hint_param_table).table= (yyvsp[-1].lexer.hint_string);
+            (yyval.hint_param_table).opt_query_block= (yyvsp[0].lexer.hint_string);
           }
 #line 1576 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 31:
-#line 300 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 299 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint_param_table).table= (yyvsp[0].hint_string);
-            (yyval.hint_param_table).opt_query_block= (yyvsp[-1].hint_string);
+            (yyval.hint_param_table).table= (yyvsp[0].lexer.hint_string);
+            (yyval.hint_param_table).opt_query_block= (yyvsp[-1].lexer.hint_string);
           }
 #line 1585 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 32:
-#line 307 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
-    { (yyval.hint_string)= NULL_CSTR; }
+#line 306 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+    { (yyval.lexer.hint_string)= NULL_CSTR; }
 #line 1591 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 34:
-#line 313 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 312 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].hint_string), true, SEMIJOIN_HINT_ENUM, (yyvsp[-1].ulong_num));
+            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].lexer.hint_string), true, SEMIJOIN_HINT_ENUM, (yyvsp[-1].ulong_num));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1601,9 +1601,9 @@ yyreduce:
     break;
 
   case 35:
-#line 320 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 319 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].hint_string), false, SEMIJOIN_HINT_ENUM, (yyvsp[-1].ulong_num));
+            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].lexer.hint_string), false, SEMIJOIN_HINT_ENUM, (yyvsp[-1].ulong_num));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1611,9 +1611,9 @@ yyreduce:
     break;
 
   case 36:
-#line 327 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 326 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].hint_string), true, SUBQUERY_HINT_ENUM, (yyvsp[-1].ulong_num));
+            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].lexer.hint_string), true, SUBQUERY_HINT_ENUM, (yyvsp[-1].ulong_num));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1621,7 +1621,7 @@ yyreduce:
     break;
 
   case 37:
-#line 334 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 333 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_qb_level_hint(NULL_CSTR, true, JOIN_PREFIX_HINT_ENUM, (yyvsp[-1].hint_param_table_list));
             if ((yyval.hint) == NULL)
@@ -1631,9 +1631,9 @@ yyreduce:
     break;
 
   case 38:
-#line 341 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 340 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].hint_string), true, JOIN_PREFIX_HINT_ENUM, (yyvsp[-1].hint_param_table_list));
+            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].lexer.hint_string), true, JOIN_PREFIX_HINT_ENUM, (yyvsp[-1].hint_param_table_list));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1641,7 +1641,7 @@ yyreduce:
     break;
 
   case 39:
-#line 348 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 347 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_qb_level_hint(NULL_CSTR, true, JOIN_SUFFIX_HINT_ENUM, (yyvsp[-1].hint_param_table_list));
             if ((yyval.hint) == NULL)
@@ -1651,9 +1651,9 @@ yyreduce:
     break;
 
   case 40:
-#line 355 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 354 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].hint_string), true, JOIN_SUFFIX_HINT_ENUM, (yyvsp[-1].hint_param_table_list));
+            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].lexer.hint_string), true, JOIN_SUFFIX_HINT_ENUM, (yyvsp[-1].hint_param_table_list));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1661,7 +1661,7 @@ yyreduce:
     break;
 
   case 41:
-#line 362 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 361 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_qb_level_hint(NULL_CSTR, true, JOIN_ORDER_HINT_ENUM, (yyvsp[-1].hint_param_table_list));
             if ((yyval.hint) == NULL)
@@ -1671,9 +1671,9 @@ yyreduce:
     break;
 
   case 42:
-#line 369 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 368 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].hint_string), true, JOIN_ORDER_HINT_ENUM, (yyvsp[-1].hint_param_table_list));
+            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-2].lexer.hint_string), true, JOIN_ORDER_HINT_ENUM, (yyvsp[-1].hint_param_table_list));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1681,9 +1681,9 @@ yyreduce:
     break;
 
   case 43:
-#line 376 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 375 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-1].hint_string), true, JOIN_FIXED_ORDER_HINT_ENUM, 0);
+            (yyval.hint)= NEW_PTN PT_qb_level_hint((yyvsp[-1].lexer.hint_string), true, JOIN_FIXED_ORDER_HINT_ENUM, 0);
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1691,13 +1691,13 @@ yyreduce:
     break;
 
   case 44:
-#line 384 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 383 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= 0; }
 #line 1697 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 45:
-#line 386 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 385 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.ulong_num)= (yyvsp[0].ulong_num);
           }
@@ -1705,7 +1705,7 @@ yyreduce:
     break;
 
   case 46:
-#line 390 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 389 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.ulong_num)= (yyvsp[-2].ulong_num) | (yyvsp[0].ulong_num);
           }
@@ -1713,44 +1713,44 @@ yyreduce:
     break;
 
   case 47:
-#line 396 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 395 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= OPTIMIZER_SWITCH_FIRSTMATCH; }
 #line 1719 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 48:
-#line 397 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 396 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= OPTIMIZER_SWITCH_LOOSE_SCAN; }
 #line 1725 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 49:
-#line 398 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 397 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= OPTIMIZER_SWITCH_MATERIALIZATION; }
 #line 1731 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 50:
-#line 399 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 398 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= OPTIMIZER_SWITCH_DUPSWEEDOUT; }
 #line 1737 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 51:
-#line 403 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 402 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)=
                                    Item_exists_subselect::EXEC_MATERIALIZATION; }
 #line 1744 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 52:
-#line 405 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 404 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     { (yyval.ulong_num)= Item_exists_subselect::EXEC_EXISTS; }
 #line 1750 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy.cc" /* yacc.c:1646  */
     break;
 
   case 53:
-#line 411 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 410 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_table_level_hint(NULL_CSTR, (yyvsp[-1].hint_param_table_list), true, (yyvsp[-3].hint_type));
             if ((yyval.hint) == NULL)
@@ -1760,9 +1760,9 @@ yyreduce:
     break;
 
   case 54:
-#line 418 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 417 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_table_level_hint((yyvsp[-2].hint_string), (yyvsp[-1].hint_param_table_list), true, (yyvsp[-4].hint_type));
+            (yyval.hint)= NEW_PTN PT_table_level_hint((yyvsp[-2].lexer.hint_string), (yyvsp[-1].hint_param_table_list), true, (yyvsp[-4].hint_type));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1770,7 +1770,7 @@ yyreduce:
     break;
 
   case 55:
-#line 424 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 423 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_table_level_hint(NULL_CSTR, (yyvsp[-1].hint_param_table_list), false, (yyvsp[-3].hint_type));
             if ((yyval.hint) == NULL)
@@ -1780,9 +1780,9 @@ yyreduce:
     break;
 
   case 56:
-#line 431 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 430 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_table_level_hint((yyvsp[-2].hint_string), (yyvsp[-1].hint_param_table_list), false, (yyvsp[-4].hint_type));
+            (yyval.hint)= NEW_PTN PT_table_level_hint((yyvsp[-2].lexer.hint_string), (yyvsp[-1].hint_param_table_list), false, (yyvsp[-4].hint_type));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1790,7 +1790,7 @@ yyreduce:
     break;
 
   case 57:
-#line 441 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 440 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_key_level_hint((yyvsp[-2].hint_param_table), (yyvsp[-1].hint_param_index_list), true, (yyvsp[-4].hint_type));
             if ((yyval.hint) == NULL)
@@ -1800,7 +1800,7 @@ yyreduce:
     break;
 
   case 58:
-#line 448 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 447 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint)= NEW_PTN PT_key_level_hint((yyvsp[-2].hint_param_table), (yyvsp[-1].hint_param_index_list), false, (yyvsp[-4].hint_type));
             if ((yyval.hint) == NULL)
@@ -1810,7 +1810,7 @@ yyreduce:
     break;
 
   case 59:
-#line 457 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 456 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= BKA_HINT_ENUM;
           }
@@ -1818,7 +1818,7 @@ yyreduce:
     break;
 
   case 60:
-#line 461 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 460 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= BNL_HINT_ENUM;
           }
@@ -1826,7 +1826,7 @@ yyreduce:
     break;
 
   case 61:
-#line 465 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 464 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= DERIVED_MERGE_HINT_ENUM;
           }
@@ -1834,7 +1834,7 @@ yyreduce:
     break;
 
   case 62:
-#line 472 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 471 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= BKA_HINT_ENUM;
           }
@@ -1842,7 +1842,7 @@ yyreduce:
     break;
 
   case 63:
-#line 476 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 475 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= BNL_HINT_ENUM;
           }
@@ -1850,7 +1850,7 @@ yyreduce:
     break;
 
   case 64:
-#line 480 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 479 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= DERIVED_MERGE_HINT_ENUM;
           }
@@ -1858,7 +1858,7 @@ yyreduce:
     break;
 
   case 65:
-#line 487 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 486 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= MRR_HINT_ENUM;
           }
@@ -1866,7 +1866,7 @@ yyreduce:
     break;
 
   case 66:
-#line 491 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 490 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= NO_RANGE_HINT_ENUM;
           }
@@ -1874,7 +1874,7 @@ yyreduce:
     break;
 
   case 67:
-#line 495 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 494 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= INDEX_MERGE_HINT_ENUM;
           }
@@ -1882,7 +1882,7 @@ yyreduce:
     break;
 
   case 68:
-#line 499 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 498 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= SKIP_SCAN_HINT_ENUM;
           }
@@ -1890,7 +1890,7 @@ yyreduce:
     break;
 
   case 69:
-#line 506 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 505 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= ICP_HINT_ENUM;
           }
@@ -1898,7 +1898,7 @@ yyreduce:
     break;
 
   case 70:
-#line 510 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 509 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= MRR_HINT_ENUM;
           }
@@ -1906,7 +1906,7 @@ yyreduce:
     break;
 
   case 71:
-#line 514 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 513 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= INDEX_MERGE_HINT_ENUM;
           }
@@ -1914,7 +1914,7 @@ yyreduce:
     break;
 
   case 72:
-#line 518 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 517 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             (yyval.hint_type)= SKIP_SCAN_HINT_ENUM;
           }
@@ -1922,9 +1922,9 @@ yyreduce:
     break;
 
   case 73:
-#line 525 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 524 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_hint_qb_name((yyvsp[-1].hint_string));
+            (yyval.hint)= NEW_PTN PT_hint_qb_name((yyvsp[-1].lexer.hint_string));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1932,9 +1932,9 @@ yyreduce:
     break;
 
   case 74:
-#line 534 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 533 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-            (yyval.hint)= NEW_PTN PT_hint_sys_var((yyvsp[-3].hint_string), (yyvsp[-1].item));
+            (yyval.hint)= NEW_PTN PT_hint_sys_var((yyvsp[-3].lexer.hint_string), (yyvsp[-1].item));
             if ((yyval.hint) == NULL)
               YYABORT; // OOM
           }
@@ -1942,9 +1942,9 @@ yyreduce:
     break;
 
   case 75:
-#line 543 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 542 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-           (yyval.hint)= NEW_PTN PT_hint_resource_group((yyvsp[-1].hint_string));
+           (yyval.hint)= NEW_PTN PT_hint_resource_group((yyvsp[-1].lexer.hint_string));
            if ((yyval.hint) == nullptr)
               YYABORT; // OOM
          }
@@ -1952,10 +1952,10 @@ yyreduce:
     break;
 
   case 78:
-#line 557 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 556 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             longlong n;
-            if (parse_int(&n, (yyvsp[0].hint_string).str, (yyvsp[0].hint_string).length))
+            if (parse_int(&n, (yyvsp[0].lexer.hint_string).str, (yyvsp[0].lexer.hint_string).length))
             {
               scanner->syntax_warning(ER_THD(thd, ER_WRONG_SIZE_NUMBER));
               (yyval.item)= NULL;
@@ -1971,10 +1971,10 @@ yyreduce:
     break;
 
   case 79:
-#line 572 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 571 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
             longlong n;
-            if (parse_int(&n, (yyvsp[0].hint_string).str, (yyvsp[0].hint_string).length - 1))
+            if (parse_int(&n, (yyvsp[0].lexer.hint_string).str, (yyvsp[0].lexer.hint_string).length - 1))
             {
               scanner->syntax_warning(ER_THD(thd, ER_WRONG_SIZE_NUMBER));
               (yyval.item)= NULL;
@@ -1982,7 +1982,7 @@ yyreduce:
             else
             {
               int multiplier;
-              switch ((yyvsp[0].hint_string).str[(yyvsp[0].hint_string).length - 1]) {
+              switch ((yyvsp[0].lexer.hint_string).str[(yyvsp[0].lexer.hint_string).length - 1]) {
               case 'K': multiplier= 1024; break;
               case 'M': multiplier= 1024 * 1024; break;
               case 'G': multiplier= 1024 * 1024 * 1024; break;
@@ -2007,9 +2007,9 @@ yyreduce:
     break;
 
   case 82:
-#line 612 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
+#line 611 "/mnt/workspace/percona-server-8.0-source-tarballs/test/percona-server/sql/sql_hints.yy" /* yacc.c:1646  */
     {
-          (yyval.item)= NEW_PTN Item_string((yyvsp[0].hint_string).str, (yyvsp[0].hint_string).length, thd->charset());
+          (yyval.item)= NEW_PTN Item_string((yyvsp[0].lexer.hint_string).str, (yyvsp[0].lexer.hint_string).length, thd->charset());
           if ((yyval.item) == NULL)
             YYABORT; // OOM
         }
