@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -20,25 +20,41 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include "sql_reload.h"
-#include "mysqld.h"      // select_errors
-#include "sql_class.h"   // THD
-#include "auth_common.h" // acl_reload, grant_reload
-#include "sql_servers.h" // servers_reload
-#include "sql_connect.h" // reset_mqh
-#include "sql_base.h"    // close_cached_tables
-#include "sql_db.h"      // my_dbopt_cleanup
-#include "hostname.h"    // hostname_cache_refresh
-#include "rpl_master.h"  // reset_master
-#include "rpl_slave.h"   // reset_slave
-#include "rpl_rli.h"     // rotate_relay_log
-#include "rpl_mi.h"
-#include "rpl_msr.h"     /* multisource replication */
-#include "debug_sync.h"
-#include "connection_handler_impl.h"
-#include "opt_costconstantcache.h"     // reload_optimizer_cost_constants
-#include "log.h"         // query_logger
-#include "des_key_file.h"
+#include "sql/sql_reload.h"
+
+#include <stddef.h>
+#include <atomic>
+
+#include "lex_string.h"
+#include "map_helpers.h"
+#include "my_base.h"
+#include "my_dbug.h"
+#include "my_inttypes.h"
+#include "my_sys.h"
+#include "mysql_com.h"
+#include "mysqld_error.h"
+#include "sql/auth/auth_acls.h"
+#include "sql/auth/auth_common.h"  // acl_reload, grant_reload
+#include "sql/binlog.h"
+#include "sql/conn_handler/connection_handler_impl.h"
+#include "sql/current_thd.h"  // my_thread_set_THR_THD
+#include "sql/debug_sync.h"
+#include "sql/handler.h"
+#include "sql/hostname.h"  // hostname_cache_refresh
+#include "sql/log.h"       // query_logger
+#include "sql/mdl.h"
+#include "sql/mysqld.h"                 // select_errors
+#include "sql/opt_costconstantcache.h"  // reload_optimizer_cost_constants
+#include "sql/query_options.h"
+#include "sql/rpl_master.h"   // reset_master
+#include "sql/rpl_slave.h"    // reset_slave
+#include "sql/sql_base.h"     // close_cached_tables
+#include "sql/sql_class.h"    // THD
+#include "sql/sql_connect.h"  // reset_mqh
+#include "sql/sql_const.h"
+#include "sql/sql_servers.h"  // servers_reload
+#include "sql/system_variables.h"
+#include "sql/table.h"
 
 /**
   Reload/resets privileges and the different caches.
@@ -46,15 +62,14 @@
   @param thd Thread handler (can be NULL!)
   @param options What should be reset/reloaded (tables, privileges, slave...)
   @param tables Tables to flush (if any)
-  @param write_to_binlog < 0 if there was an error while interacting with the binary log inside
-                         reload_acl_and_cache,
-                         0 if we should not write to the binary log,
-                         > 0 if we can write to the binlog.
+  @param write_to_binlog < 0 if there was an error while interacting with the
+  binary log inside handle_reload_request, 0 if we should not write to the
+  binary log, > 0 if we can write to the binlog.
+
 
-               
   @note Depending on 'options', it may be very bad to write the
     query to the binlog (e.g. FLUSH SLAVE); this is a
-    pointer where reload_acl_and_cache() will put 0 if
+    pointer where handle_reload_request() will put 0 if
     it thinks we really should not write to the binlog.
     Otherwise it will put 1.
 
@@ -63,38 +78,31 @@
     @retval !=0  Error; thd->killed is set or thd->is_error() is true
 */
 
-bool reload_acl_and_cache(THD *thd, unsigned long options,
-                          TABLE_LIST *tables, int *write_to_binlog)
-{
-  bool result=0;
-  select_errors=0;				/* Write if more errors */
-  int tmp_write_to_binlog= *write_to_binlog= 1;
+bool handle_reload_request(THD *thd, unsigned long options, TABLE_LIST *tables,
+                           int *write_to_binlog) {
+  bool result = 0;
+  select_errors = 0; /* Write if more errors */
+  int tmp_write_to_binlog = *write_to_binlog = 1;
 
-  assert(!thd || !thd->in_sub_stmt);
+  DBUG_ASSERT(!thd || !thd->in_sub_stmt);
 
-#ifndef NO_EMBEDDED_ACCESS_CHECKS
-  if (options & REFRESH_GRANT)
-  {
-    THD *tmp_thd= 0;
+  if (options & REFRESH_GRANT) {
+    THD *tmp_thd = 0;
     /*
-      If reload_acl_and_cache() is called from SIGHUP handler we have to
+      If handle_reload_request() is called from SIGHUP handler we have to
       allocate temporary THD for execution of acl_reload()/grant_reload().
     */
-    if (!thd && (thd= (tmp_thd= new THD)))
-    {
-      thd->thread_stack= (char*) &tmp_thd;
+    if (!thd && (thd = (tmp_thd = new THD))) {
+      thd->thread_stack = (char *)&tmp_thd;
       thd->store_globals();
     }
 
-    if (thd)
-    {
-      bool reload_acl_failed= acl_reload(thd);
-      bool reload_grants_failed= grant_reload(thd);
-      bool reload_servers_failed= servers_reload(thd);
-
-      if (reload_acl_failed || reload_grants_failed || reload_servers_failed)
-      {
-        result= 1;
+    if (thd) {
+      bool reload_acl_failed = reload_acl_caches(thd);
+      bool reload_servers_failed = servers_reload(thd);
+      notify_flush_event(thd);
+      if (reload_acl_failed || reload_servers_failed) {
+        result = 1;
         /*
           When an error is returned, my_message may have not been called and
           the client will hang waiting for a response.
@@ -103,186 +111,152 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
       }
     }
 
-    if (tmp_thd)
-    {
+    reset_mqh(thd, (LEX_USER *)NULL, true);
+    if (tmp_thd) {
       delete tmp_thd;
-      thd= 0;
+      thd = 0;
     }
-    reset_mqh((LEX_USER *)NULL, TRUE);
   }
-#endif
-  if (options & REFRESH_LOG)
-  {
+
+  if (options & REFRESH_LOG) {
     /*
       Flush the normal query log, the update log, the binary log,
       the slow query log, the relay log (if it exists) and the log
       tables.
     */
 
-    options|= REFRESH_BINARY_LOG;
-    options|= REFRESH_RELAY_LOG;
-    options|= REFRESH_SLOW_LOG;
-    options|= REFRESH_GENERAL_LOG;
-    options|= REFRESH_ENGINE_LOG;
-    options|= REFRESH_ERROR_LOG;
+    options |= REFRESH_BINARY_LOG;
+    options |= REFRESH_RELAY_LOG;
+    options |= REFRESH_SLOW_LOG;
+    options |= REFRESH_GENERAL_LOG;
+    options |= REFRESH_ENGINE_LOG;
+    options |= REFRESH_ERROR_LOG;
   }
 
-  if (options & REFRESH_ERROR_LOG)
-    if (reopen_error_log())
-      result= 1;
+  if (options & REFRESH_ERROR_LOG) {
+    if (reopen_error_log()) result = 1;
+  }
 
   if ((options & REFRESH_SLOW_LOG) && opt_slow_log)
-    if (query_logger.reopen_log_file(QUERY_LOG_SLOW))
-      result= 1;
+    if (query_logger.reopen_log_file(QUERY_LOG_SLOW)) result = 1;
 
   if ((options & REFRESH_GENERAL_LOG) && opt_general_log)
-    if (query_logger.reopen_log_file(QUERY_LOG_GENERAL))
-      result= 1;
+    if (query_logger.reopen_log_file(QUERY_LOG_GENERAL)) result = 1;
 
-  if (options & REFRESH_ENGINE_LOG)
-    if (ha_flush_logs(NULL))
-      result= 1;
-  if ((options & REFRESH_BINARY_LOG) || (options & REFRESH_RELAY_LOG ))
-  {
+  if (options & REFRESH_ENGINE_LOG) {
+    if (ha_flush_logs()) {
+      result = 1;
+    }
+  }
+  if ((options & REFRESH_BINARY_LOG) || (options & REFRESH_RELAY_LOG)) {
     /*
-      If reload_acl_and_cache() is called from SIGHUP handler we have to
+      If handle_reload_request() is called from SIGHUP handler we have to
       allocate temporary THD for execution of binlog/relay log rotation.
      */
-    THD *tmp_thd= 0;
-    if (!thd && (thd= (tmp_thd= new THD)))
-    {
-      thd->thread_stack= (char *) (&tmp_thd);
+    THD *tmp_thd = 0;
+    if (!thd && (thd = (tmp_thd = new THD))) {
+      thd->thread_stack = (char *)(&tmp_thd);
       thd->store_globals();
     }
 
-    if (options & REFRESH_BINARY_LOG)
-    {
+    if (options & REFRESH_BINARY_LOG) {
       /*
         Writing this command to the binlog may result in infinite loops
         when doing mysqlbinlog|mysql, and anyway it does not really make
         sense to log it automatically (would cause more trouble to users
         than it would help them)
        */
-      tmp_write_to_binlog= 0;
-      if (mysql_bin_log.is_open())
-      {
-        if (mysql_bin_log.rotate_and_purge(thd, true))
-          *write_to_binlog= -1;
+      tmp_write_to_binlog = 0;
+      if (mysql_bin_log.is_open()) {
+        if (mysql_bin_log.rotate_and_purge(thd, true)) *write_to_binlog = -1;
       }
     }
-    if (options & REFRESH_RELAY_LOG)
-    {
-#ifdef HAVE_REPLICATION
-      if (flush_relay_logs_cmd(thd))
-        *write_to_binlog= -1;
-#endif
+    if (options & REFRESH_RELAY_LOG) {
+      if (flush_relay_logs_cmd(thd)) *write_to_binlog = -1;
     }
-    if (tmp_thd)
-    {
+    if (tmp_thd) {
       delete tmp_thd;
       /* Remember that we don't have a THD */
-      my_thread_set_THR_THD(NULL);
-      thd= 0;
+      current_thd = nullptr;
+      thd = 0;
     }
   }
-  if (options & REFRESH_QUERY_CACHE_FREE)
-  {
-    query_cache.pack();				// FLUSH QUERY CACHE
-    options &= ~REFRESH_QUERY_CACHE;    // Don't flush cache, just free memory
-  }
-  if (options & (REFRESH_TABLES | REFRESH_QUERY_CACHE))
-  {
-    query_cache.flush();			// RESET QUERY CACHE
-  }
 
-  assert(!thd || thd->locked_tables_mode ||
-         !thd->mdl_context.has_locks() ||
-         thd->handler_tables_hash.records ||
-         thd->mdl_context.has_locks(MDL_key::USER_LEVEL_LOCK) ||
-         thd->mdl_context.has_locks(MDL_key::LOCKING_SERVICE) ||
-         thd->global_read_lock.is_acquired() ||
-         thd->backup_tables_lock.is_acquired() ||
-         thd->backup_binlog_lock.is_acquired());
+  DBUG_ASSERT(!thd || thd->locked_tables_mode ||
+              !thd->mdl_context.has_locks() ||
+              !thd->handler_tables_hash.empty() ||
+              thd->mdl_context.has_locks(MDL_key::USER_LEVEL_LOCK) ||
+              thd->mdl_context.has_locks(MDL_key::LOCKING_SERVICE) ||
+              thd->mdl_context.has_locks(MDL_key::BACKUP_LOCK) ||
+              thd->global_read_lock.is_acquired() ||
+              thd->backup_tables_lock.is_acquired());
 
   /*
     Note that if REFRESH_READ_LOCK bit is set then REFRESH_TABLES is set too
     (see sql_yacc.yy)
   */
-  if (options & (REFRESH_TABLES | REFRESH_READ_LOCK)) 
-  {
-    if ((options & REFRESH_READ_LOCK) && thd)
-    {
+  if (options & (REFRESH_TABLES | REFRESH_READ_LOCK)) {
+    if ((options & REFRESH_READ_LOCK) && thd) {
       /*
         On the first hand we need write lock on the tables to be flushed,
         on the other hand we must not try to aspire a global read lock
         if we have a write locked table as this would lead to a deadlock
         when trying to reopen (and re-lock) the table after the flush.
       */
-      if (thd->locked_tables_mode)
-      {
+      if (thd->locked_tables_mode) {
         my_error(ER_LOCK_OR_ACTIVE_TRANSACTION, MYF(0));
         return 1;
       }
       /*
-	Writing to the binlog could cause deadlocks, as we don't log
-	UNLOCK TABLES
+        Writing to the binlog could cause deadlocks, as we don't log
+        UNLOCK TABLES
       */
-      tmp_write_to_binlog= 0;
-      if (thd->global_read_lock.lock_global_read_lock(thd))
-	return 1;                               // Killed
+      tmp_write_to_binlog = 0;
+      if (thd->global_read_lock.lock_global_read_lock(thd)) return 1;  // Killed
       if (close_cached_tables(thd, tables,
-                              ((options & REFRESH_FAST) ?  FALSE : TRUE),
-                              thd->variables.lock_wait_timeout))
-      {
+                              ((options & REFRESH_FAST) ? false : true),
+                              thd->variables.lock_wait_timeout)) {
         /*
           NOTE: my_error() has been already called by reopen_tables() within
           close_cached_tables().
         */
-        result= 1;
+        result = 1;
       }
 
-      if (thd->global_read_lock.make_global_read_lock_block_commit(thd)) // Killed
+      if (thd->global_read_lock.make_global_read_lock_block_commit(
+              thd))  // Killed
       {
         /* Don't leave things in a half-locked state */
         thd->global_read_lock.unlock_global_read_lock(thd);
         return 1;
       }
-    }
-    else
-    {
-      if (thd && thd->locked_tables_mode)
-      {
+    } else {
+      if (thd && thd->locked_tables_mode) {
         /*
           If we are under LOCK TABLES we should have a write
           lock on tables which we are going to flush.
         */
-        if (tables)
-        {
-          for (TABLE_LIST *t= tables; t; t= t->next_local)
+        if (tables) {
+          for (TABLE_LIST *t = tables; t; t = t->next_local)
             if (!find_table_for_mdl_upgrade(thd, t->db, t->table_name, false))
               return 1;
-        }
-        else
-        {
+        } else {
           /*
             It is not safe to upgrade the metadata lock without GLOBAL IX lock.
-            This can happen with FLUSH TABLES <list> WITH READ LOCK as we in these
-            cases don't take a GLOBAL IX lock in order to be compatible with
-            global read lock.
+            This can happen with FLUSH TABLES <list> WITH READ LOCK as we in
+            these cases don't take a GLOBAL IX lock in order to be compatible
+            with global read lock.
           */
           if (thd->open_tables &&
-              !thd->mdl_context.owns_equal_or_stronger_lock(MDL_key::GLOBAL,
-                                  "", "", MDL_INTENTION_EXCLUSIVE))
-          {
+              !thd->mdl_context.owns_equal_or_stronger_lock(
+                  MDL_key::GLOBAL, "", "", MDL_INTENTION_EXCLUSIVE)) {
             my_error(ER_TABLE_NOT_LOCKED_FOR_WRITE, MYF(0),
                      thd->open_tables->s->table_name.str);
             return true;
           }
 
-          for (TABLE *tab= thd->open_tables; tab; tab= tab->next)
-          {
-            if (! tab->mdl_ticket->is_upgradable_or_exclusive())
-            {
+          for (TABLE *tab = thd->open_tables; tab; tab = tab->next) {
+            if (!tab->mdl_ticket->is_upgradable_or_exclusive()) {
               my_error(ER_TABLE_NOT_LOCKED_FOR_WRITE, MYF(0),
                        tab->s->table_name.str);
               return 1;
@@ -291,160 +265,124 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
         }
       }
 
-      if (close_cached_tables(thd, tables,
-                              ((options & REFRESH_FAST) ?  FALSE : TRUE),
-                              (thd ? thd->variables.lock_wait_timeout :
-                               LONG_TIMEOUT)))
-      {
+      if (close_cached_tables(
+              thd, tables, ((options & REFRESH_FAST) ? false : true),
+              (thd ? thd->variables.lock_wait_timeout : LONG_TIMEOUT))) {
         /*
           NOTE: my_error() has been already called by reopen_tables() within
           close_cached_tables().
         */
-        result= 1;
+        result = 1;
       }
     }
-    my_dbopt_cleanup();
   }
-  if (options & REFRESH_HOSTS)
-    hostname_cache_refresh();
-  if (thd && (options & REFRESH_STATUS))
-    refresh_status(thd);
-#ifndef EMBEDDED_LIBRARY
+  if (options & REFRESH_HOSTS) hostname_cache_refresh();
+  if (thd && (options & REFRESH_STATUS)) refresh_status();
   if (options & REFRESH_THREADS)
     Per_thread_connection_handler::kill_blocked_pthreads();
-#endif
-#ifdef HAVE_REPLICATION
-  if (options & REFRESH_MASTER)
-  {
-    assert(thd);
-    tmp_write_to_binlog= 0;
-    if (reset_master(thd))
-    {
+  if (options & REFRESH_MASTER) {
+    DBUG_ASSERT(thd);
+    tmp_write_to_binlog = 0;
+    /*
+      RESET MASTER acquired global read lock (if the thread is not acquired
+      already) to make sure no transaction commits are getting executed
+      while the operation is in process. If (and only if) it is
+      acquired by RESET MASTER internal process (options will contain
+      REFRESH_READ_LOCK flag in this case), unlock the global read lock
+      in reset_master().
+    */
+    if (reset_master(thd, options & REFRESH_READ_LOCK)) {
       /* NOTE: my_error() has been already called by reset_master(). */
-      result= 1;
+      result = 1;
+    }
+  }
+  if (options & REFRESH_OPTIMIZER_COSTS) reload_optimizer_cost_constants();
+  if (options & REFRESH_SLAVE) {
+    tmp_write_to_binlog = 0;
+    if (reset_slave_cmd(thd)) {
+      /*NOTE: my_error() has been already called by reset_slave() */
+      result = 1;
     }
   }
-#endif
-#ifdef HAVE_OPENSSL
-   if (options & REFRESH_DES_KEY_FILE)
-   {
-     if (des_key_file && load_des_key_file(des_key_file))
-     {
-       /* NOTE: my_error() has been already called by load_des_key_file(). */
-       result= 1;
-     }
-   }
-#endif
-  if (options & REFRESH_OPTIMIZER_COSTS)
-    reload_optimizer_cost_constants();
-#ifdef HAVE_REPLICATION
- if (options & REFRESH_SLAVE)
- {
-   tmp_write_to_binlog= 0;
-   if (reset_slave_cmd(thd))
-   {
-     /*NOTE: my_error() has been already called by reset_slave() */
-     result= 1;
-   }
- }
-#endif
   if (options & REFRESH_USER_RESOURCES)
-    reset_mqh((LEX_USER *) NULL, 0);             /* purecov: inspected */
-#ifndef EMBEDDED_LIBRARY
-  if (options & REFRESH_TABLE_STATS)
-  {
+    reset_mqh(thd, nullptr, 0); /* purecov: inspected */
+  if (options & REFRESH_TABLE_STATS) {
     mysql_mutex_lock(&LOCK_global_table_stats);
     free_global_table_stats();
     init_global_table_stats();
     mysql_mutex_unlock(&LOCK_global_table_stats);
   }
-  if (options & REFRESH_INDEX_STATS)
-  {
+  if (options & REFRESH_INDEX_STATS) {
     mysql_mutex_lock(&LOCK_global_index_stats);
     free_global_index_stats();
     init_global_index_stats();
     mysql_mutex_unlock(&LOCK_global_index_stats);
   }
-  if (options & (REFRESH_USER_STATS | REFRESH_CLIENT_STATS | REFRESH_THREAD_STATS))
-  {
+  if (options &
+      (REFRESH_USER_STATS | REFRESH_CLIENT_STATS | REFRESH_THREAD_STATS)) {
     mysql_mutex_lock(&LOCK_global_user_client_stats);
-    if (options & REFRESH_USER_STATS)
-    {
+    if (options & REFRESH_USER_STATS) {
       free_global_user_stats();
       init_global_user_stats();
     }
-    if (options & REFRESH_CLIENT_STATS)
-    {
+    if (options & REFRESH_CLIENT_STATS) {
       free_global_client_stats();
       init_global_client_stats();
     }
-    if (options & REFRESH_THREAD_STATS)
-    {
+    if (options & REFRESH_THREAD_STATS) {
       free_global_thread_stats();
       init_global_thread_stats();
     }
     mysql_mutex_unlock(&LOCK_global_user_client_stats);
   }
-#endif
-  if (options & REFRESH_FLUSH_PAGE_BITMAPS)
-  {
-    result= ha_flush_changed_page_bitmaps();
-    if (result)
-    {
+  if (options & REFRESH_FLUSH_PAGE_BITMAPS) {
+    result = ha_flush_changed_page_bitmaps();
+    if (result) {
       my_error(ER_UNKNOWN_ERROR, MYF(0), "FLUSH CHANGED_PAGE_BITMAPS");
     }
   }
-  if (options & REFRESH_RESET_PAGE_BITMAPS)
-  {
-    result= ha_purge_changed_page_bitmaps(0);
-    if (result)
-    {
+  if (options & REFRESH_RESET_PAGE_BITMAPS) {
+    result = ha_purge_changed_page_bitmaps(0);
+    if (result) {
       my_error(ER_UNKNOWN_ERROR, MYF(0), "RESET CHANGED_PAGE_BITMAPS");
     }
   }
- if (*write_to_binlog != -1)
- {
-   if (thd == NULL || thd->security_context() == NULL)
-   {
-     *write_to_binlog=
-       opt_binlog_skip_flush_commands ? 0 : tmp_write_to_binlog;
-   }
-   else if (thd->security_context()->check_access(SUPER_ACL))
-   {
-     /*
-       For users with 'SUPER' privilege 'FLUSH XXX' statements must not be
-       binlogged if 'super_read_only' is set to 'ON'.
-     */
-     if (opt_super_readonly)
-       *write_to_binlog= 0;
-     else
-       *write_to_binlog=
-         opt_binlog_skip_flush_commands ? 0 : tmp_write_to_binlog;
-   }
-   else
-   {
-     /*
-       For users without 'SUPER' privilege 'FLUSH XXX' statements must not be
-       binlogged if 'read_only' or 'super_read_only' is set to 'ON'.
-       Checking only 'opt_readonly' here as in 'super_read_only' mode this
-       variable is implicitly set to 'true'.
-     */
-     if (opt_readonly)
-       *write_to_binlog= 0;
-     else
-       *write_to_binlog=
-         opt_binlog_skip_flush_commands ? 0 : tmp_write_to_binlog;
-   }
- }
- /*
-   If the query was killed then this function must fail.
- */
- return result || (thd ? thd->killed : 0);
+  if (*write_to_binlog != -1) {
+    if (thd == nullptr || thd->security_context() == nullptr) {
+      *write_to_binlog =
+          opt_binlog_skip_flush_commands ? 0 : tmp_write_to_binlog;
+    } else if (thd->security_context()->check_access(SUPER_ACL)) {
+      /*
+        For users with 'SUPER' privilege 'FLUSH XXX' statements must not be
+        binlogged if 'super_read_only' is set to 'ON'.
+      */
+      if (opt_super_readonly)
+        *write_to_binlog = 0;
+      else
+        *write_to_binlog =
+            opt_binlog_skip_flush_commands ? 0 : tmp_write_to_binlog;
+    } else {
+      /*
+        For users without 'SUPER' privilege 'FLUSH XXX' statements must not be
+        binlogged if 'read_only' or 'super_read_only' is set to 'ON'.
+        Checking only 'opt_readonly' here as in 'super_read_only' mode this
+        variable is implicitly set to 'true'.
+      */
+      if (opt_readonly)
+        *write_to_binlog = 0;
+      else
+        *write_to_binlog =
+            opt_binlog_skip_flush_commands ? 0 : tmp_write_to_binlog;
+    }
+  }
+  /*
+    If the query was killed then this function must fail.
+  */
+  return result || (thd ? thd->killed : 0);
 }
 
-
 /**
-  Implementation of FLUSH TABLES <table_list> WITH READ LOCK.
+  Implementation of FLUSH TABLES @<table_list@> WITH READ LOCK.
 
   In brief: take exclusive locks, expel tables from the table
   cache, reopen the tables, enter the 'LOCKED TABLES' mode,
@@ -464,9 +402,9 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
   ---------------------------------------
   We don't wait for the GRL, since neither the
   5.1 combination that this new statement is intended to
-  replace (LOCK TABLE <list> WRITE; FLUSH TABLES;),
+  replace (LOCK TABLE @<list@> WRITE; FLUSH TABLES;),
   nor FLUSH TABLES WITH READ LOCK do.
-  @todo: this is not implemented, Dmitry disagrees.
+  @todo This is not implemented, Dmitry disagrees.
   Currently we wait for GRL in another connection,
   but are compatible with a GRL in our own connection.
 
@@ -485,7 +423,7 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
   new transactions will be able to read the tables, but not
   write to them.
 
-  Differences from FLUSH TABLES <list>
+  Differences from FLUSH TABLES @<list@>
   -------------------------------------
   - you can't flush WITH READ LOCK a non-existent table
   - you can't flush WITH READ LOCK under LOCK TABLES
@@ -517,8 +455,7 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
   are implicitly flushed (lose their position).
 */
 
-bool flush_tables_with_read_lock(THD *thd, TABLE_LIST *all_tables)
-{
+bool flush_tables_with_read_lock(THD *thd, TABLE_LIST *all_tables) {
   Lock_tables_prelocking_strategy lock_tables_prelocking_strategy;
   TABLE_LIST *table_list;
 
@@ -527,8 +464,7 @@ bool flush_tables_with_read_lock(THD *thd, TABLE_LIST *all_tables)
     been committed implicitly.
   */
 
-  if (thd->locked_tables_mode)
-  {
+  if (thd->locked_tables_mode) {
     my_error(ER_LOCK_OR_ACTIVE_TRANSACTION, MYF(0));
     goto error;
   }
@@ -538,22 +474,19 @@ bool flush_tables_with_read_lock(THD *thd, TABLE_LIST *all_tables)
     IX and database-scope IX locks on the tables as this will make
     this statement incompatible with FLUSH TABLES WITH READ LOCK.
   */
-  if (lock_table_names(thd, all_tables, NULL,
-                       thd->variables.lock_wait_timeout,
+  if (lock_table_names(thd, all_tables, NULL, thd->variables.lock_wait_timeout,
                        MYSQL_OPEN_SKIP_SCOPED_MDL_LOCK))
     goto error;
 
-  DEBUG_SYNC(thd,"flush_tables_with_read_lock_after_acquire_locks");
+  DEBUG_SYNC(thd, "flush_tables_with_read_lock_after_acquire_locks");
 
-  for (table_list= all_tables; table_list;
-       table_list= table_list->next_global)
-  {
+  for (table_list = all_tables; table_list;
+       table_list = table_list->next_global) {
     /* Request removal of table from cache. */
-    tdc_remove_table(thd, TDC_RT_REMOVE_UNUSED,
-                     table_list->db,
-                     table_list->table_name, FALSE);
+    tdc_remove_table(thd, TDC_RT_REMOVE_UNUSED, table_list->db,
+                     table_list->table_name, false);
     /* Reset ticket to satisfy asserts in open_tables(). */
-    table_list->mdl_request.ticket= NULL;
+    table_list->mdl_request.ticket = NULL;
   }
 
   /*
@@ -568,11 +501,10 @@ bool flush_tables_with_read_lock(THD *thd, TABLE_LIST *all_tables)
   */
   if (open_and_lock_tables(thd, all_tables, MYSQL_OPEN_SKIP_SCOPED_MDL_LOCK,
                            &lock_tables_prelocking_strategy) ||
-      thd->locked_tables_list.init_locked_tables(thd))
-  {
+      thd->locked_tables_list.init_locked_tables(thd)) {
     goto error;
   }
-  thd->variables.option_bits|= OPTION_TABLE_LOCK;
+  thd->variables.option_bits |= OPTION_TABLE_LOCK;
 
   /*
     We don't downgrade MDL_SHARED_NO_WRITE here as the intended
@@ -581,13 +513,12 @@ bool flush_tables_with_read_lock(THD *thd, TABLE_LIST *all_tables)
     thd->sql_command= SQLCOM_LOCK_TABLES hacks to enter the LTM.
   */
 
-  return FALSE;
+  return false;
 
 error:
-  return TRUE;
+  return true;
 }
 
-
 /**
   Prepare tables for export (transportable tablespaces) by
   a) waiting until write transactions/DDL operations using these
@@ -606,8 +537,7 @@ error:
   @retval true   Error
 */
 
-bool flush_tables_for_export(THD *thd, TABLE_LIST *all_tables)
-{
+bool flush_tables_for_export(THD *thd, TABLE_LIST *all_tables) {
   Lock_tables_prelocking_strategy lock_tables_prelocking_strategy;
 
   /*
@@ -615,8 +545,7 @@ bool flush_tables_for_export(THD *thd, TABLE_LIST *all_tables)
     been committed implicitly.
   */
 
-  if (thd->locked_tables_mode)
-  {
+  if (thd->locked_tables_mode) {
     my_error(ER_LOCK_OR_ACTIVE_TRANSACTION, MYF(0));
     return true;
   }
@@ -631,39 +560,33 @@ bool flush_tables_for_export(THD *thd, TABLE_LIST *all_tables)
     creation/deletion of metadata file.
   */
   if (open_and_lock_tables(thd, all_tables, MYSQL_OPEN_SKIP_SCOPED_MDL_LOCK,
-                           &lock_tables_prelocking_strategy))
-  {
+                           &lock_tables_prelocking_strategy)) {
     return true;
   }
 
   // Check if all storage engines support FOR EXPORT.
-  for (TABLE_LIST *table_list= all_tables; table_list;
-       table_list= table_list->next_global)
-  {
-    if (!(table_list->table->file->ha_table_flags() & HA_CAN_EXPORT))
-    {
+  for (TABLE_LIST *table_list = all_tables; table_list;
+       table_list = table_list->next_global) {
+    if (!(table_list->table->file->ha_table_flags() & HA_CAN_EXPORT)) {
       my_error(ER_ILLEGAL_HA, MYF(0), table_list->table_name);
       return true;
     }
   }
 
   // Notify the storage engines that the tables should be made ready for export.
-  for (TABLE_LIST *table_list= all_tables; table_list;
-       table_list= table_list->next_global)
-  {
-    handler *handler_file= table_list->table->file;
-    int error= handler_file->extra(HA_EXTRA_EXPORT);
-    if (error)
-    {
+  for (TABLE_LIST *table_list = all_tables; table_list;
+       table_list = table_list->next_global) {
+    handler *handler_file = table_list->table->file;
+    int error = handler_file->extra(HA_EXTRA_EXPORT);
+    if (error) {
       handler_file->print_error(error, MYF(0));
       return true;
     }
   }
 
   // Enter LOCKED TABLES mode.
-  if (thd->locked_tables_list.init_locked_tables(thd))
-    return true;
-  thd->variables.option_bits|= OPTION_TABLE_LOCK;
+  if (thd->locked_tables_list.init_locked_tables(thd)) return true;
+  thd->variables.option_bits |= OPTION_TABLE_LOCK;
 
   return false;
 }
