@@ -325,7 +325,7 @@ extern int MYSQLdebug;
     LIST_SYM = 535,
     LOAD = 536,
     LOCAL_SYM = 537,
-    LOCATOR_SYM = 538,
+    OBSOLETE_TOKEN_538 = 538,
     LOCKS_SYM = 539,
     LOCK_SYM = 540,
     LOGFILE_SYM = 541,
@@ -542,7 +542,7 @@ extern int MYSQLdebug;
     SERIAL_SYM = 752,
     SESSION_SYM = 753,
     SERVER_SYM = 754,
-    SERVER_OPTIONS = 755,
+    OBSOLETE_TOKEN_755 = 755,
     SET_SYM = 756,
     SET_VAR = 757,
     SHARE_SYM = 758,
@@ -635,7 +635,7 @@ extern int MYSQLdebug;
     TRUNCATE_SYM = 845,
     TYPES_SYM = 846,
     TYPE_SYM = 847,
-    UDF_RETURNS_SYM = 848,
+    OBSOLETE_TOKEN_848 = 848,
     ULONGLONG_NUM = 849,
     UNCOMMITTED_SYM = 850,
     UNDEFINED_SYM = 851,
@@ -764,24 +764,28 @@ extern int MYSQLdebug;
     INACTIVE_SYM = 974,
     LATERAL_SYM = 975,
     OPTIONAL_SYM = 976,
-    SECONDARY_ENGINE_SYM = 977,
-    SECONDARY_LOAD_SYM = 978,
-    SECONDARY_UNLOAD_SYM = 979,
-    RETAIN_SYM = 980,
-    OLD_SYM = 981,
-    CHANGED_PAGE_BITMAPS_SYM = 982,
-    CLIENT_STATS_SYM = 983,
-    CLUSTERING_SYM = 984,
-    COMPRESSION_DICTIONARY_SYM = 985,
-    INDEX_STATS_SYM = 986,
-    TABLE_STATS_SYM = 987,
-    THREAD_STATS_SYM = 988,
-    USER_STATS_SYM = 989,
-    ENCRYPTION_KEY_ID_SYM = 990,
-    EFFECTIVE_SYM = 991,
-    CONDITIONLESS_JOIN = 992,
-    SUBQUERY_AS_EXPR = 993,
-    EMPTY_FROM_CLAUSE = 994
+    SECONDARY_SYM = 977,
+    SECONDARY_ENGINE_SYM = 978,
+    SECONDARY_LOAD_SYM = 979,
+    SECONDARY_UNLOAD_SYM = 980,
+    RETAIN_SYM = 981,
+    OLD_SYM = 982,
+    ENFORCED_SYM = 983,
+    OJ_SYM = 984,
+    NETWORK_NAMESPACE_SYM = 985,
+    CHANGED_PAGE_BITMAPS_SYM = 986,
+    CLIENT_STATS_SYM = 987,
+    CLUSTERING_SYM = 988,
+    COMPRESSION_DICTIONARY_SYM = 989,
+    INDEX_STATS_SYM = 990,
+    TABLE_STATS_SYM = 991,
+    THREAD_STATS_SYM = 992,
+    USER_STATS_SYM = 993,
+    ENCRYPTION_KEY_ID_SYM = 994,
+    EFFECTIVE_SYM = 995,
+    CONDITIONLESS_JOIN = 996,
+    SUBQUERY_AS_EXPR = 997,
+    EMPTY_FROM_CLAUSE = 998
   };
 #endif
 /* Tokens.  */
@@ -1065,7 +1069,7 @@ extern int MYSQLdebug;
 #define LIST_SYM 535
 #define LOAD 536
 #define LOCAL_SYM 537
-#define LOCATOR_SYM 538
+#define OBSOLETE_TOKEN_538 538
 #define LOCKS_SYM 539
 #define LOCK_SYM 540
 #define LOGFILE_SYM 541
@@ -1282,7 +1286,7 @@ extern int MYSQLdebug;
 #define SERIAL_SYM 752
 #define SESSION_SYM 753
 #define SERVER_SYM 754
-#define SERVER_OPTIONS 755
+#define OBSOLETE_TOKEN_755 755
 #define SET_SYM 756
 #define SET_VAR 757
 #define SHARE_SYM 758
@@ -1375,7 +1379,7 @@ extern int MYSQLdebug;
 #define TRUNCATE_SYM 845
 #define TYPES_SYM 846
 #define TYPE_SYM 847
-#define UDF_RETURNS_SYM 848
+#define OBSOLETE_TOKEN_848 848
 #define ULONGLONG_NUM 849
 #define UNCOMMITTED_SYM 850
 #define UNDEFINED_SYM 851
@@ -1504,24 +1508,28 @@ extern int MYSQLdebug;
 #define INACTIVE_SYM 974
 #define LATERAL_SYM 975
 #define OPTIONAL_SYM 976
-#define SECONDARY_ENGINE_SYM 977
-#define SECONDARY_LOAD_SYM 978
-#define SECONDARY_UNLOAD_SYM 979
-#define RETAIN_SYM 980
-#define OLD_SYM 981
-#define CHANGED_PAGE_BITMAPS_SYM 982
-#define CLIENT_STATS_SYM 983
-#define CLUSTERING_SYM 984
-#define COMPRESSION_DICTIONARY_SYM 985
-#define INDEX_STATS_SYM 986
-#define TABLE_STATS_SYM 987
-#define THREAD_STATS_SYM 988
-#define USER_STATS_SYM 989
-#define ENCRYPTION_KEY_ID_SYM 990
-#define EFFECTIVE_SYM 991
-#define CONDITIONLESS_JOIN 992
-#define SUBQUERY_AS_EXPR 993
-#define EMPTY_FROM_CLAUSE 994
+#define SECONDARY_SYM 977
+#define SECONDARY_ENGINE_SYM 978
+#define SECONDARY_LOAD_SYM 979
+#define SECONDARY_UNLOAD_SYM 980
+#define RETAIN_SYM 981
+#define OLD_SYM 982
+#define ENFORCED_SYM 983
+#define OJ_SYM 984
+#define NETWORK_NAMESPACE_SYM 985
+#define CHANGED_PAGE_BITMAPS_SYM 986
+#define CLIENT_STATS_SYM 987
+#define CLUSTERING_SYM 988
+#define COMPRESSION_DICTIONARY_SYM 989
+#define INDEX_STATS_SYM 990
+#define TABLE_STATS_SYM 991
+#define THREAD_STATS_SYM 992
+#define USER_STATS_SYM 993
+#define ENCRYPTION_KEY_ID_SYM 994
+#define EFFECTIVE_SYM 995
+#define CONDITIONLESS_JOIN 996
+#define SUBQUERY_AS_EXPR 997
+#define EMPTY_FROM_CLAUSE 998
 
 /* Value type.  */
 
