@@ -1156,7 +1156,7 @@ class Sys_var_double : public sys_var {
     option.var_type = GET_DOUBLE;
     option.min_value = (longlong)getopt_double2ulonglong(min_val);
     option.max_value = (longlong)getopt_double2ulonglong(max_val);
-    global_var(double) = (double)option.def_value;
+    global_var(double) = getopt_ulonglong2double(option.def_value);
     DBUG_ASSERT(min_val <= max_val);
     DBUG_ASSERT(min_val <= def_val);
     DBUG_ASSERT(max_val >= def_val);
@@ -1304,7 +1304,7 @@ class Sys_var_flagset : public Sys_var_typelib {
       if (!(res = var->value->val_str(&str)))
         return true;
       else {
-        char *error;
+        const char *error;
         uint error_len;
 
         var->save_result.ulonglong_value = find_set_from_flags(
@@ -1855,6 +1855,40 @@ class Sys_var_have : public sys_var {
   bool check_update_type(Item_result) { return false; }
 };
 
+/**
+   A subclass of @ref Sys_var_have to return dynamic values
+
+   All the usual restrictions for @ref Sys_var_have apply.
+   But instead of reading a global variable it calls a function
+   to return the value.
+ */
+class Sys_var_have_func : public Sys_var_have {
+ public:
+  /**
+    Construct a new variable.
+
+    @param name_arg The name of the variable
+    @param comment  Explanation of what the variable does
+    @param func     The function to call when in need to read the global value
+  */
+  Sys_var_have_func(const char *name_arg, const char *comment,
+                    enum SHOW_COMP_OPTION (*func)(THD *))
+      /*
+        Note: it doesn't really matter what variable we use, as long as we are
+        using one. So we use a local static dummy
+      */
+      : Sys_var_have(name_arg, comment,
+                     READ_ONLY NON_PERSIST GLOBAL_VAR(dummy_), NO_CMD_LINE),
+        func_(func) {}
+
+  uchar *global_value_ptr(THD *thd, LEX_STRING *) {
+    return (uchar *)show_comp_option_name[func_(thd)];
+  }
+
+ protected:
+  enum SHOW_COMP_OPTION (*func_)(THD *);
+  static enum SHOW_COMP_OPTION dummy_;
+};
 /**
   Generic class for variables for storing entities that are internally
   represented as structures, have names, and possibly can be referred to by
@@ -2062,6 +2096,19 @@ class Sys_var_enum_binlog_checksum : public Sys_var_enum {
   virtual bool global_update(THD *thd, set_var *var);
 };
 
+class Sys_var_enum_default_table_encryption : public Sys_var_enum {
+ public:
+  Sys_var_enum_default_table_encryption(
+      const char *name_arg, const char *comment, int flag_args, ptrdiff_t off,
+      size_t size, CMD_LINE getopt, const char *values[], uint def_val,
+      PolyLock *lock, enum binlog_status_enum binlog_status_arg,
+      on_check_function on_check_func = 0)
+      : Sys_var_enum(name_arg, comment, flag_args, off, size, getopt, values,
+                     def_val, lock, binlog_status_arg, on_check_func, nullptr) {
+  }
+  virtual bool global_update(THD *thd, set_var *var);
+};
+
 /**
   Class for gtid_next.
 */
@@ -2451,8 +2498,4 @@ class Sys_var_binlog_encryption : public Sys_var_bool {
   virtual bool global_update(THD *thd, set_var *var) override;
 };
 
-extern void init_log_slow_verbosity() noexcept;
-extern void init_slow_query_log_use_global_control() noexcept;
-extern void init_log_slow_sp_statements() noexcept;
-
 #endif /* SYS_VARS_H_INCLUDED */
