@@ -96,6 +96,15 @@ enum use_secondary_engine {
   SECONDARY_ENGINE_FORCED = 2
 };
 
+// Values for default_table_encryption
+enum enum_default_table_encryption {
+  DEFAULT_TABLE_ENC_OFF = 0,
+  DEFAULT_TABLE_ENC_ON = 1,
+  DEFAULT_TABLE_ENC_KEYRING_ON = 2,
+  DEFAULT_TABLE_ENC_ONLINE_TO_KEYRING = 3,
+  DEFAULT_TABLE_ENC_ONLINE_FROM_KEYRING_TO_UNENCRYPTED = 4
+};
+
 /* Bits for different SQL modes modes (including ANSI mode) */
 #define MODE_REAL_AS_FLOAT 1
 #define MODE_PIPES_AS_CONCAT 2
@@ -393,6 +402,14 @@ struct System_variables {
   /** Used for controlling preparation of queries against secondary engine. */
   ulong use_secondary_engine;
 
+  /**
+    Used for controlling which statements to execute in a secondary
+    storage engine. Only queries with an estimated cost higher than
+    this value will be attempted executed in a secondary storage
+    engine.
+  */
+  double secondary_engine_cost_threshold;
+
   /** Used for controlling Group Replication consistency guarantees */
   ulong group_replication_consistency;
 
@@ -409,6 +426,12 @@ struct System_variables {
     in the replication topology.
   */
   uint32_t immediate_server_version;
+
+  /**
+    Used to determine if the database or tablespace should be encrypted by
+    default.
+  */
+  ulong default_table_encryption;
 };
 
 /**
@@ -520,7 +543,8 @@ const int COUNT_GLOBAL_STATUS_VARS =
 void add_diff_to_status(System_status_var *to_var, System_status_var *from_var,
                         System_status_var *dec_var);
 
-void add_to_status(System_status_var *to_var, System_status_var *from_var,
-                   bool reset_from_var);
+void add_to_status(System_status_var *to_var, System_status_var *from_var);
+
+void reset_system_status_vars(System_status_var *status_vars);
 
 #endif  // SYSTEM_VARIABLES_INCLUDED
