@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -284,8 +284,17 @@ bool trans_commit(THD *thd, bool ignore_global_read_lock) {
     table statistics during CREATE TABLE ... SELECT, otherwise the
     uncommitted object added by DDL would be removed by I_S query.
   */
-  if (!thd->is_attachable_rw_transaction_active())
-    thd->dd_client()->commit_modified_objects();
+  if (!thd->is_attachable_rw_transaction_active()) {
+    /*
+      If the SE failed to commit the transaction, we must rollback the
+      modified dictionary objects to make sure the DD cache, the DD
+      tables and the state in the SE stay in sync.
+    */
+    if (res)
+      thd->dd_client()->rollback_modified_objects();
+    else
+      thd->dd_client()->commit_modified_objects();
+  }
 
   thd->locked_tables_list.adjust_renamed_tablespace_mdls(&thd->mdl_context);
 
@@ -360,8 +369,17 @@ bool trans_commit_implicit(THD *thd, bool ignore_global_read_lock) {
     table statistics during CREATE TABLE ... SELECT, otherwise the
     uncommitted object added by DDL would be removed by I_S query.
   */
-  if (!thd->is_attachable_rw_transaction_active())
-    thd->dd_client()->commit_modified_objects();
+  if (!thd->is_attachable_rw_transaction_active()) {
+    /*
+      If the SE failed to commit the transaction, we must rollback the
+      modified dictionary objects to make sure the DD cache, the DD
+      tables and the state in the SE stay in sync.
+    */
+    if (res)
+      thd->dd_client()->rollback_modified_objects();
+    else
+      thd->dd_client()->commit_modified_objects();
+  }
 
   thd->locked_tables_list.adjust_renamed_tablespace_mdls(&thd->mdl_context);
   DBUG_RETURN(res);
