@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -201,7 +201,7 @@ uchar *hp_search_next(HP_INFO *info, HP_KEYDEF *keyinfo, const uchar *key,
     Array index, in [0..maxlength)
 */
 
-ulong hp_mask(ulong hashnr, ulong buffmax, ulong maxlength) {
+uint64 hp_mask(uint64 hashnr, uint64 buffmax, uint64 maxlength) {
   if ((hashnr & (buffmax - 1)) < maxlength) return (hashnr & (buffmax - 1));
   return (hashnr & ((buffmax >> 1) - 1));
 }
@@ -224,9 +224,9 @@ void hp_movelink(HASH_INFO *pos, HASH_INFO *next_link, HASH_INFO *newlink) {
 
 /* Calc hashvalue for a key */
 
-ulong hp_hashnr(HP_KEYDEF *keydef, const uchar *key) {
+uint64 hp_hashnr(HP_KEYDEF *keydef, const uchar *key) {
   /*register*/
-  ulong nr = 1, nr2 = 4;
+  uint64 nr = 1, nr2 = 4;
   HA_KEYSEG *seg, *endseg;
 
   for (seg = keydef->seg, endseg = seg + keydef->keysegs; seg < endseg; seg++) {
@@ -280,19 +280,19 @@ ulong hp_hashnr(HP_KEYDEF *keydef, const uchar *key) {
       key += pack_length;
     } else {
       for (; pos < (uchar *)key; pos++) {
-        nr ^= (ulong)((((uint)nr & 63) + nr2) * ((uint)*pos)) + (nr << 8);
+        nr ^= (uint64)((((uint)nr & 63) + nr2) * ((uint)*pos)) + (nr << 8);
         nr2 += 3;
       }
     }
   }
-  DBUG_PRINT("exit", ("hash: 0x%lx", nr));
-  return ((ulong)nr);
+  DBUG_PRINT("exit", ("hash: 0x%llx", nr));
+  return nr;
 }
 
 /* Calc hashvalue for a key in a record */
 
-ulong hp_rec_hashnr(HP_KEYDEF *keydef, const uchar *rec) {
-  ulong nr = 1, nr2 = 4;
+uint64 hp_rec_hashnr(HP_KEYDEF *keydef, const uchar *rec) {
+  uint64 nr = 1, nr2 = 4;
   HA_KEYSEG *seg, *endseg;
 
   for (seg = keydef->seg, endseg = seg + keydef->keysegs; seg < endseg; seg++) {
@@ -343,12 +343,12 @@ ulong hp_rec_hashnr(HP_KEYDEF *keydef, const uchar *rec) {
       cs->coll->hash_sort(cs, pos, length, &nr, &nr2);
     } else {
       for (; pos < end; pos++) {
-        nr ^= (ulong)((((uint)nr & 63) + nr2) * ((uint)*pos)) + (nr << 8);
+        nr ^= (uint64)((((uint)nr & 63) + nr2) * ((uint)*pos)) + (nr << 8);
         nr2 += 3;
       }
     }
   }
-  DBUG_PRINT("exit", ("hash: 0x%lx", nr));
+  DBUG_PRINT("exit", ("hash: 0x%llx", nr));
   return (nr);
 }
 
