@@ -155,7 +155,7 @@ static ulint buf_read_page_low(dberr_t *err, bool sync, ulint type, ulint mode,
       buf_read_page_handle_error(bpage);
       return (0);
     } else if (*err == DB_IO_DECRYPT_FAIL) {
-      bpage->encrypted= true;
+      bpage->encrypted = true;
     }
 
     SRV_CORRUPT_TABLE_CHECK(bpage->encrypted, bpage->is_corrupt = true;);
@@ -332,7 +332,7 @@ released by the i/o-handler thread.
 @param[in]	page_size	page size
 @retval DB_SUCCESS if the page was read and is not corrupted,
 @retval DB_PAGE_CORRUPTED if page based on checksum check is corrupted,
-@retval DB_DECRYPTION_FAILED if page post encryption checksum matches but
+@retval DB_IO_DECRYPT_FAIL if page post encryption checksum matches but
 after decryption normal page checksum does not match.
 @retval DB_TABLESPACE_DELETED if tablespace .ibd file is missing */
 dberr_t buf_read_page(const page_id_t &page_id, const page_size_t &page_size,
@@ -640,7 +640,7 @@ ulint buf_read_ahead_linear(const page_id_t &page_id,
                                 << page_id_t(page_id.space(), i)
                                 << " in nonexisting or being-dropped"
                                    " tablespace";
-      } else if (err == DB_DECRYPTION_FAILED) {
+      } else if (err == DB_IO_DECRYPT_FAIL) {
         ib::error() << "linear readahead failed to"
                        " read or decrypt "
                     << page_id_t(page_id.space(), i);
@@ -720,7 +720,7 @@ void buf_read_ibuf_merge_pages(
       /* We have deleted or are deleting the single-table
       tablespace: remove the entries for that page */
       ibuf_merge_or_delete_for_page(NULL, page_id, &page_size, FALSE);
-    } else if (err == DB_DECRYPTION_FAILED) {
+    } else if (err == DB_IO_DECRYPT_FAIL) {
       ib::error() << "Failed to read or decrypt " << page_id
                   << " for change buffer merge";
     }
@@ -813,12 +813,11 @@ void buf_read_recv_pages(bool sync, space_id_t space_id,
                         cur_page_id, page_size, true, nullptr, false);
     }
 
-    if (err == DB_DECRYPTION_FAILED) {
-      ib::error() << "Recovery failed to decrypt page "
-                  << cur_page_id;
+    if (err == DB_IO_DECRYPT_FAIL) {
+      ib::error() << "Recovery failed to decrypt page " << cur_page_id
+                  << ". Are you using the correct keyring?";
     } else if (err == DB_PAGE_CORRUPTED) {
-      ib::error() << "Recovery failed due to corrupted page "
-                  << cur_page_id;
+      ib::error() << "Recovery failed due to corrupted page " << cur_page_id;
     }
   }
 
