@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2015, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2015, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -278,6 +278,7 @@ const innodb_dd_table_t innodb_dd_table[] = {
 
     INNODB_DD_TABLE("catalogs", 2),
     INNODB_DD_TABLE("character_sets", 3),
+    INNODB_DD_TABLE("check_constraints", 3),
     INNODB_DD_TABLE("collations", 3),
     INNODB_DD_TABLE("column_statistics", 3),
     INNODB_DD_TABLE("column_type_elements", 1),
@@ -601,7 +602,7 @@ void dd_update_v_cols(dd::Table *dd_table, table_id_t id);
 @param[in]	fsp_flags	InnoDB tablespace flags
 @param[in]	state		InnoDB tablespace state */
 void dd_write_tablespace(dd::Tablespace *dd_space, space_id_t space_id,
-                         ulint fsp_flags, dd_space_states state);
+                         uint32_t fsp_flags, dd_space_states state);
 
 /** Add fts doc id column and index to new table
 when old table has hidden fts doc id without fulltext index
@@ -766,7 +767,7 @@ bool dd_process_dd_indexes_rec_simple(mem_heap_t *heap, const rec_t *rec,
 @return true if data is retrived */
 bool dd_process_dd_tablespaces_rec(mem_heap_t *heap, const rec_t *rec,
                                    space_id_t *space_id, char **name,
-                                   uint *flags, uint32 *server_version,
+                                   uint32_t *flags, uint32 *server_version,
                                    uint32 *space_version, bool *is_encrypted,
                                    dd::String_type *state,
                                    dict_table_t *dd_spaces);
@@ -961,7 +962,7 @@ void dd_filename_to_spacename(const char *space_name,
 @retval true on failure */
 bool dd_create_tablespace(dd::cache::Dictionary_client *dd_client, THD *thd,
                           const char *dd_space_name, space_id_t space_id,
-                          ulint flags, const char *filename, bool discarded,
+                          uint32_t flags, const char *filename, bool discarded,
                           dd::Object_id &dd_space_id);
 
 /** Create metadata for implicit tablespace
@@ -1222,17 +1223,23 @@ bool dd_tablespace_update_cache(THD *thd);
 @return true if it does. */
 bool dd_is_table_in_encrypted_tablespace(const dict_table_t *table);
 
+bool dd_is_table_in_encrypted_tablespace(const char *name);
+
 /* Sets tablespace's DD encryption flag.
 @param[in] Thread       THD
 @param[in] space_name   name of the space for which DD encryption flag is to be
+@param[in] *is_space_being_removed - whether space is being removed
 set */
-bool dd_set_encryption_flag(THD *thd, const char *space_name);
+bool dd_set_encryption_flag(THD *thd, const char *space_name,
+                            volatile bool *is_space_being_removed);
 
 /* Clears tablespace's DD encryption flag.
 @param[in] Thread       THD
 @param[in] space_name   name of the space for which DD encryption flag is to be
+@param[in] *is_space_being_removed - whether space is being removed
 cleared */
-bool dd_clear_encryption_flag(THD *thd, const char *space_name);
+bool dd_clear_encryption_flag(THD *thd, const char *space_name,
+                              volatile bool *is_space_being_removed);
 
 /* If mysql_ibd's DD encryption flag is different from the encryption flag in
  * space_flag   the mysql_ibd's encryption flag will be set to the
