@@ -160,7 +160,7 @@ dberr_t dict_stats_rename_index(
 /** Report an error if updating table statistics failed because
 .ibd file is missing, table decryption failed or table is corrupted.
 @param[in,out]	table	Table
-@retval DB_DECRYPTION_FAILED if decryption of the table failed
+@retval DB_IO_DECRYPT_FAIL if decryption of the table failed
 @retval DB_TABLESPACE_DELETED if .ibd file is missing
 @retval DB_CORRUPTION if table is marked as corrupted */
 dberr_t dict_stats_report_error(dict_table_t* table)
