@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2013, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -97,7 +97,7 @@ class Datafile {
     m_handle.m_file = OS_FILE_CLOSED;
   }
 
-  Datafile(const char *name, ulint flags, page_no_t size, ulint order)
+  Datafile(const char *name, uint32_t flags, page_no_t size, ulint order)
       : m_name(mem_strdup(name)),
         m_filename(),
         m_open_flags(OS_FILE_OPEN),
@@ -204,7 +204,7 @@ class Datafile {
   /** Initialize the name and flags of this datafile.
   @param[in]	name	tablespace name, will be copied
   @param[in]	flags	tablespace flags */
-  void init(const char *name, ulint flags);
+  void init(const char *name, uint32_t flags);
 
   /** Release the resources. */
   void shutdown();
@@ -275,7 +275,7 @@ class Datafile {
   @param[in]	for_import	is it for importing
   @retval DB_SUCCESS if tablespace is valid, DB_ERROR if not.
   m_is_valid is also set true on success, else false. */
-  ValidateOutput validate_to_dd(space_id_t space_id, ulint flags,
+  ValidateOutput validate_to_dd(space_id_t space_id, uint32_t flags,
                                 bool for_import)
       MY_ATTRIBUTE((warn_unused_result));
 
@@ -340,7 +340,7 @@ class Datafile {
 
   /** Get Datafile::m_flags.
   @return m_flags */
-  ulint flags() const { return (m_flags); }
+  uint32_t flags() const { return (m_flags); }
 
   /**
   @return true if m_handle is open, false if not */
@@ -391,7 +391,7 @@ class Datafile {
 
   /** Set th tablespace flags
   @param[in]	fsp_flags	Tablespace flags */
-  void set_flags(ulint flags) { m_flags = flags; }
+  void set_flags(uint32_t flags) { m_flags = flags; }
 #endif /* UNIV_HOTBACKUP */
 
  private:
@@ -479,7 +479,7 @@ class Datafile {
   /** Tablespace flags. Contained in the datafile header.
   If this is a system tablespace, FSP_SPACE_FLAGS are only valid
   in the first datafile. */
-  ulint m_flags;
+  uint32_t m_flags;
 
   /** true if file already existed on startup */
   bool m_exists;
