@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2013, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -39,6 +39,8 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "ut0rnd.h"
 #include "ut0ut.h"
 
+#include <atomic>
+
 /** OS mutex for tracking lock/unlock for debugging */
 template <template <typename> class Policy = NoPolicy>
 struct OSTrackMutex {
@@ -222,7 +224,7 @@ struct TTASFutexMutex {
     them up. Reset the lock state to unlocked so that waiting
     threads can test for success. */
 
-    os_rmb;
+    std::atomic_thread_fence(std::memory_order_acquire);
 
     if (state() == MUTEX_STATE_WAITERS) {
       m_lock_word = MUTEX_STATE_UNLOCKED;
@@ -320,7 +322,7 @@ struct TTASFutexMutex {
   @return value of lock word before locking. */
   lock_word_t ttas(uint32_t max_spins, uint32_t max_delay,
                    uint32_t &n_spins) UNIV_NOTHROW {
-    os_rmb;
+    std::atomic_thread_fence(std::memory_order_acquire);
 
     for (n_spins = 0; n_spins < max_spins; ++n_spins) {
       if (!is_locked()) {
@@ -344,7 +346,7 @@ struct TTASFutexMutex {
 
   /** lock_word is the target of the atomic test-and-set instruction
   when atomic operations are enabled. */
-  lock_word_t m_lock_word MY_ALIGNED(MY_ALIGNOF(ulint));
+  alignas(ulint) lock_word_t m_lock_word;
 };
 
 #endif /* HAVE_IB_LINUX_FUTEX */
@@ -451,7 +453,7 @@ struct TTASMutex {
     uint32_t i = 0;
     const uint32_t step = max_spins;
 
-    os_rmb;
+    std::atomic_thread_fence(std::memory_order_acquire);
 
     do {
       while (is_locked()) {
@@ -497,7 +499,7 @@ struct TTASEventMutex {
     ut_ad(!((ulint)&m_lock_word % sizeof(ulint)));
   }
 
-  ~TTASEventMutex() UNIV_NOTHROW { ut_ad(m_lock_word == MUTEX_STATE_UNLOCKED); }
+  ~TTASEventMutex() UNIV_NOTHROW { ut_ad(!m_lock_word.load()); }
 
   /** Called when the mutex is "created". Note: Not from the constructor
   but when the mutex is initialised.
@@ -505,7 +507,7 @@ struct TTASEventMutex {
   @param[in]	filename	File where mutex was created
   @param[in]	line		Line in filename */
   void init(latch_id_t id, const char *filename, uint32_t line) UNIV_NOTHROW {
-    ut_a(m_lock_word == MUTEX_STATE_UNLOCKED);
+    ut_a(!m_lock_word.load(std::memory_order_relaxed));
 
     m_policy.init(*this, id, filename, line);
   }
@@ -514,14 +516,17 @@ struct TTASEventMutex {
   its desctructor will be called on exit(). We can't call
   os_event_destroy() at that stage. */
   void destroy() UNIV_NOTHROW {
-    ut_ad(m_lock_word == MUTEX_STATE_UNLOCKED);
+    ut_ad(!m_lock_word.load(std::memory_order_relaxed));
 
     m_policy.destroy();
   }
 
   /** Try and lock the mutex. Note: POSIX returns 0 on success.
   @return true on success */
-  bool try_lock() UNIV_NOTHROW { return (tas_lock()); }
+  bool try_lock() UNIV_NOTHROW {
+    bool expected = false;
+    return (m_lock_word.compare_exchange_strong(expected, true));
+  }
 
   /** Release the mutex. */
   void exit() UNIV_NOTHROW {
@@ -530,11 +535,12 @@ struct TTASEventMutex {
     field next, the read must be serialized in memory
     after the reset. A speculative processor might
     perform the read first, which could leave a waiting
-                thread hanging indefinitely. */
+    thread hanging indefinitely. */
 
-    tas_unlock();
+    m_lock_word.store(false);
+    std::atomic_thread_fence(std::memory_order_acquire);
 
-    if (m_waiters != 0) {
+    if (m_waiters.load(std::memory_order_acquire)) {
       signal();
     }
   }
@@ -551,8 +557,10 @@ struct TTASEventMutex {
     }
   }
 
-  /** @return the lock state. */
-  lock_word_t state() const UNIV_NOTHROW { return (m_lock_word); }
+  /** @return true if locked. */
+  bool state() const UNIV_NOTHROW {
+    return (m_lock_word.load(std::memory_order_relaxed));
+  }
 
   /** The event that the mutex will wait in sync0arr.cc
   @return even instance */
@@ -560,7 +568,7 @@ struct TTASEventMutex {
 
   /** @return true if locked by some thread */
   bool is_locked() const UNIV_NOTHROW {
-    return (m_lock_word != MUTEX_STATE_UNLOCKED);
+    return (m_lock_word.load(std::memory_order_relaxed));
   }
 
 #ifdef UNIV_DEBUG
@@ -624,8 +632,6 @@ struct TTASEventMutex {
     uint32_t n_waits = 0;
     const uint32_t step = max_spins;
 
-    os_rmb;
-
     for (;;) {
       /* If the lock was free then try and acquire it. */
 
@@ -667,46 +673,37 @@ struct TTASEventMutex {
   }
 
   /** @return the value of the m_waiters flag */
-  lock_word_t waiters() UNIV_NOTHROW { return (m_waiters); }
+  lock_word_t waiters() UNIV_NOTHROW {
+    return (m_waiters.load(std::memory_order_relaxed));
+  }
 
   /** Note that there are threads waiting on the mutex */
   void set_waiters() UNIV_NOTHROW {
-    m_waiters = 1;
-    os_wmb;
+    m_waiters.store(true, std::memory_order_release);
   }
 
   /** Note that there are no threads waiting on the mutex */
   void clear_waiters() UNIV_NOTHROW {
-    m_waiters = 0;
-    os_wmb;
+    m_waiters.store(false, std::memory_order_release);
   }
 
-  /** Try and acquire the lock using TestAndSet.
-  @return	true if lock succeeded */
-  bool tas_lock() UNIV_NOTHROW {
-    return (TAS(&m_lock_word, MUTEX_STATE_LOCKED) == MUTEX_STATE_UNLOCKED);
-  }
-
-  /** In theory __sync_lock_release should be used to release the lock.
-  Unfortunately, it does not work properly alone. The workaround is
-  that more conservative __sync_lock_test_and_set is used instead. */
-  void tas_unlock() UNIV_NOTHROW { TAS(&m_lock_word, MUTEX_STATE_UNLOCKED); }
-
   /** Wakeup any waiting thread(s). */
   void signal() UNIV_NOTHROW;
 
  private:
   /** Disable copying */
-  TTASEventMutex(const TTASEventMutex &);
-  TTASEventMutex &operator=(const TTASEventMutex &);
+  TTASEventMutex(TTASEventMutex &&) = delete;
+  TTASEventMutex(const TTASEventMutex &) = delete;
+  TTASEventMutex &operator=(TTASEventMutex &&) = delete;
+  TTASEventMutex &operator=(const TTASEventMutex &) = delete;
 
   /** lock_word is the target of the atomic test-and-set instruction
   when atomic operations are enabled. */
-  lock_word_t m_lock_word;
+  std::atomic_bool m_lock_word;
 
-  /** Set to 0 or 1. 1 if there are (or may be) threads waiting
+  /** true if there are (or may be) threads waiting
   in the global wait array for this mutex to be released. */
-  lock_word_t m_waiters;
+  std::atomic_bool m_waiters;
 
   /** Used by sync0arr.cc for the wait queue */
   struct os_event m_event;
