@@ -458,6 +458,7 @@ struct Encryption {
   @param[in]      algorithm       Encryption algorithm to check
   @return true if no algorithm explicitly requested */
   static bool none_explicitly_specified(
+      ulong create_info_used_fields,
       const char *algorithm) noexcept MY_ATTRIBUTE((warn_unused_result));
 
   static bool is_master_key_encryption(const char *algorithm)
@@ -469,6 +470,12 @@ struct Encryption {
   static bool is_keyring(const char *algoritm)
       MY_ATTRIBUTE((warn_unused_result));
 
+  static bool is_online_encryption_on() MY_ATTRIBUTE((warn_unused_result));
+
+  static bool should_be_keyring_encrypted(ulong create_info_used_fields,
+                                          const char *algorithm)
+      MY_ATTRIBUTE((warn_unused_result));
+
   /** Generate random encryption value for key and iv.
   @param[in,out]	value	Encryption value */
   static void random_value(byte *value);
