@@ -93,11 +93,11 @@ enum monitor_type_t {
   MONITOR_NO_AVERAGE = 4,      /*!< Set this status if we don't want to
                                calculate the average value for the counter */
   MONITOR_DISPLAY_CURRENT = 8, /*!< Display current value of the
-                          counter, rather than incremental value
-                          over the period. Mostly for counters
-                          displaying current resource usage */
+                               counter, rather than incremental value
+                               over the period. Mostly for counters
+                               displaying current resource usage */
   MONITOR_GROUP_MODULE = 16,   /*!< Monitor can be turned on/off
-                            only as a module, but not individually */
+                               only as a module, but not individually */
   MONITOR_DEFAULT_ON = 32,     /*!< Monitor will be turned on by default at
                                server start up */
   MONITOR_SET_OWNER = 64,      /*!< Owner of "monitor set", a set of
@@ -319,6 +319,19 @@ enum monitor_id_t {
   MONITOR_DML_PURGE_DELAY,
   MONITOR_PURGE_STOP_COUNT,
   MONITOR_PURGE_RESUME_COUNT,
+  MONITOR_PURGE_TRUNCATE_HISTORY_COUNT,
+  MONITOR_PURGE_TRUNCATE_HISTORY_MICROSECOND,
+
+  /* Undo tablespace truncation */
+  MONITOR_UNDO_TRUNCATE,
+  MONITOR_UNDO_TRUNCATE_COUNT,
+  MONITOR_UNDO_TRUNCATE_SWEEP_COUNT,
+  MONITOR_UNDO_TRUNCATE_SWEEP_MICROSECOND,
+  MONITOR_UNDO_TRUNCATE_START_LOGGING_COUNT,
+  MONITOR_UNDO_TRUNCATE_FLUSH_COUNT,
+  MONITOR_UNDO_TRUNCATE_FLUSH_MICROSECOND,
+  MONITOR_UNDO_TRUNCATE_DONE_LOGGING_COUNT,
+  MONITOR_UNDO_TRUNCATE_MICROSECOND,
 
   /* Recovery related counters */
   MONITOR_MODULE_REDO_LOG,
@@ -335,6 +348,7 @@ enum monitor_id_t {
   MONITOR_OVLD_LOG_WAITS,
   MONITOR_OVLD_LOG_WRITE_REQUEST,
   MONITOR_OVLD_LOG_WRITES,
+  MONITOR_OVLD_LSN_TRACKED,
 
   MONITOR_LOG_FLUSH_TOTAL_TIME,
   MONITOR_LOG_FLUSH_MAX_TIME,
@@ -393,6 +407,7 @@ enum monitor_id_t {
   MONITOR_PAD_DECREMENTS,
 
   /* New monitor variables for page encryption */
+  MONITOR_MODULE_ENCRYPTION,
   MONITOR_OVLD_PAGES_ENCRYPTED,
   MONITOR_OVLD_PAGES_DECRYPTED,
 
@@ -489,6 +504,12 @@ enum monitor_id_t {
   MONITOR_CPU_STIME_PCT,
   MONITOR_CPU_N,
 
+  MONITOR_MODULE_PAGE_TRACK,
+  MONITOR_PAGE_TRACK_RESETS,
+  MONITOR_PAGE_TRACK_PARTIAL_BLOCK_WRITES,
+  MONITOR_PAGE_TRACK_FULL_BLOCK_WRITES,
+  MONITOR_PAGE_TRACK_CHECKPOINT_PARTIAL_FLUSH_REQUEST,
+
   /* This is used only for control system to turn
   on/off and reset all monitor counters */
   MONITOR_ALL_COUNTER,
