@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2018, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 1995, 2019, Oracle and/or its affiliates. All rights reserved.
 Copyright (c) 2008, 2009, Google Inc.
 Copyright (c) 2009, 2016, Percona Inc.
 
@@ -198,6 +198,9 @@ struct Srv_threads {
 
   /** true if tablespace alter encrypt thread is created */
   bool m_ts_alter_encrypt_thread_active;
+
+  /** true if there is keyring encryption thread running */
+  bool m_encryption_threads_active;
 };
 
 struct Srv_cpu_usage {
@@ -376,13 +379,6 @@ extern const page_no_t SRV_UNDO_TABLESPACE_SIZE_IN_PAGES;
 
 extern char *srv_log_group_home_dir;
 
-enum redo_log_encrypt_enum {
-  REDO_LOG_ENCRYPT_OFF = 0,
-  REDO_LOG_ENCRYPT_ON = 1,
-  REDO_LOG_ENCRYPT_MK = 2,
-  REDO_LOG_ENCRYPT_RK = 3,
-};
-
 /** Enable or Disable Encrypt of REDO tablespace. */
 extern ulong srv_redo_log_encrypt;
 
@@ -509,6 +505,9 @@ extern bool srv_log_checksums;
 /** If true then disable checkpointing. */
 extern bool srv_checkpoint_disabled;
 
+/* Used to inject a failure to find a free rollback segment. */
+extern bool srv_inject_too_many_concurrent_trxs;
+
 #endif /* UNIV_DEBUG */
 
 extern ulong srv_flush_log_at_trx_commit;
@@ -762,7 +761,7 @@ extern bool srv_print_lock_wait_timeout_info;
 
 extern bool srv_cmp_per_index_enabled;
 
-extern ulong srv_encrypt_tables;
+extern enum_default_table_encryption srv_default_table_encryption;
 
 /** Number of times secondary index lookup triggered cluster lookup */
 extern std::atomic<ulint> srv_sec_rec_cluster_reads;
@@ -779,7 +778,8 @@ extern srv_stats_t srv_stats;
 /* Keys to register InnoDB threads with performance schema */
 
 #ifdef UNIV_PFS_THREAD
-extern mysql_pfs_key_t archiver_thread_key;
+extern mysql_pfs_key_t log_archiver_thread_key;
+extern mysql_pfs_key_t page_archiver_thread_key;
 extern mysql_pfs_key_t buf_dump_thread_key;
 extern mysql_pfs_key_t buf_lru_manager_thread_key;
 extern mysql_pfs_key_t buf_resize_thread_key;
@@ -1049,10 +1049,15 @@ void srv_purge_coordinator_thread();
 /** Worker thread that reads tasks from the work queue and executes them. */
 void srv_worker_thread();
 
-/** Enable the undo log encryption if needed.  If innodb_undo_log_encrypt
-is ON, this will try to enable the undo log encryption and write the metadata
-to the undo log file header. */
-void srv_enable_undo_encryption_if_set();
+/** Rotate default master key for UNDO tablespace. */
+void undo_rotate_default_master_key();
+
+/** Enable UNDO tablespace encryption.
+@param[in] is_boot	true if it is called during server start up. In this
+                        case, default master key will be used which will be
+                        rotated later with actual master key from kyering.
+@return false for success, true otherwise. */
+bool srv_enable_undo_encryption(THD *thd, bool is_boot);
 
 /** Get count of tasks in the queue.
  @return number of tasks in queue */
@@ -1109,6 +1114,19 @@ void undo_spaces_init();
 called once during thread de-initialization. */
 void undo_spaces_deinit();
 
+/** Enables master key redo encryption.
+ * Doesn't depend on the srv_redo_log_encrypt variable, used by
+ * SET innodb_redo_log_encrypt = MK. */
+bool srv_enable_redo_encryption_mk(THD *thd);
+
+/** Enables master key redo encryption.
+ * Doesn't depend on the srv_redo_log_encrypt variable, used by
+ * SET innodb_redo_log_encrypt = RK. */
+bool srv_enable_redo_encryption_rk(THD *thd);
+
+/** Enables redo log encryption based on srv_redo_log_encrypt. */
+bool srv_enable_redo_encryption(THD *thd);
+
 #ifdef UNIV_DEBUG
 struct SYS_VAR;
 
@@ -1259,6 +1277,9 @@ struct export_var_t {
                                   encrypted */
   int64_t innodb_pages_decrypted; /*!< Number of pages
                                   decrypted */
+
+  /* Current redo log encryption key versison for keyring encryption */
+  int64_t innodb_redo_key_version;
   ulint innodb_encryption_rotation_pages_read_from_cache;
   ulint innodb_encryption_rotation_pages_read_from_disk;
   ulint innodb_encryption_rotation_pages_modified;
