@@ -260,6 +260,7 @@ enum latch_level_t {
   SYNC_LOG_CHECKPOINTER,
   SYNC_LOG_SN,
   SYNC_PAGE_ARCH,
+  SYNC_PAGE_ARCH_CLIENT,
   SYNC_LOG_ARCH,
 
   SYNC_LOG_ONLINE,
@@ -378,6 +379,7 @@ enum latch_id_t {
   LATCH_ID_LOG_ARCH,
   LATCH_ID_PAGE_ARCH,
   LATCH_ID_PAGE_ARCH_OPER,
+  LATCH_ID_PAGE_ARCH_CLIENT,
   LATCH_ID_PERSIST_METADATA_BUFFER,
   LATCH_ID_DICT_PERSIST_DIRTY_TABLES,
   LATCH_ID_PERSIST_AUTOINC,
@@ -407,6 +409,7 @@ enum latch_id_t {
   LATCH_ID_TRX_POOL,
   LATCH_ID_TRX_POOL_MANAGER,
   LATCH_ID_TEMP_POOL_MANAGER,
+  LATCH_ID_TEMP_POOL_TBLSP,
   LATCH_ID_TRX,
   LATCH_ID_LOCK_SYS,
   LATCH_ID_LOCK_SYS_WAIT,
@@ -451,6 +454,7 @@ enum latch_id_t {
   LATCH_ID_FIL_CRYPT_STAT_MUTEX,
   LATCH_ID_FIL_CRYPT_DATA_MUTEX,
   LATCH_ID_FIL_CRYPT_THREADS_MUTEX,
+  LATCH_ID_FIL_CRYPT_THREADS_SET_CNT_MUTEX,
   LATCH_ID_FIL_CRYPT_START_ROTATE_MUTEX,
   LATCH_ID_FILE_OPEN,
   LATCH_ID_CLONE_SYS,
