@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2014, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2014, 2019, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2016, Percona Inc. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
@@ -406,9 +406,10 @@ static constexpr size_t n_auto = UT_ARR_SIZE(auto_event_names);
 extern PSI_memory_key auto_event_keys[n_auto];
 extern PSI_memory_info pfs_info_auto[n_auto];
 
+/** gcc 5 fails to evalutate costexprs at compile time. */
+#if defined(__GNUG__) && (__GNUG__ == 5)
+
 /** Compute whether a string begins with a given prefix, compile-time.
-Has to work recursively due to C++11 constexpr constraints (C++14 is
-more flexible).
 @param[in]	a	first string, taken to be zero-terminated
 @param[in]	b	second string (prefix to search for)
 @param[in]	b_len	length in bytes of second string
@@ -421,8 +422,6 @@ constexpr bool ut_string_begins_with(const char *a, const char *b, size_t b_len,
 }
 
 /** Find the length of the filename without its file extension.
-Has to work recursively due to C++11 constexpr constraints (C++14 is
-more flexible).
 @param[in]	file	filename, with extension but without directory
 @param[in]	index	character index to start scanning for extension
                         separator at
@@ -435,8 +434,6 @@ constexpr size_t ut_len_without_extension(const char *file, size_t index = 0) {
 
 /** Retrieve a memory key (registered with PFS), given the file name of the
 caller.
-Has to work recursively due to C++11 constexpr constraints (C++14 is
-more flexible).
 @param[in]	file	portion of the filename - basename, with extension
 @param[in]	len	length of the filename to check for
 @param[in]	index	index of first PSI key to check
@@ -461,6 +458,74 @@ constexpr PSI_memory_key ut_new_get_key_by_file(const char *file) {
 
 #define UT_NEW_THIS_FILE_PSI_KEY ut_new_get_key_by_file(MY_BASENAME)
 
+#else /* __GNUG__ == 5 */
+
+/** Compute whether a string begins with a given prefix, compile-time.
+@param[in]	a	first string, taken to be zero-terminated
+@param[in]	b	second string (prefix to search for)
+@param[in]	b_len	length in bytes of second string
+@return whether b is a prefix of a */
+constexpr bool ut_string_begins_with(const char *a, const char *b,
+                                     size_t b_len) {
+  for (size_t i = 0; i < b_len; ++i) {
+    if (a[i] != b[i]) {
+      return false;
+    }
+  }
+  return true;
+}
+
+/** Find the length of the filename without its file extension.
+@param[in]	file	filename, with extension but without directory
+@return length, in bytes */
+constexpr size_t ut_len_without_extension(const char *file) {
+  for (size_t i = 0;; ++i) {
+    if (file[i] == '\0' || file[i] == '.') {
+      return i;
+    }
+  }
+}
+
+/** Retrieve a memory key (registered with PFS), given the file name of the
+caller.
+@param[in]	file	portion of the filename - basename, with extension
+@param[in]	len	length of the filename to check for
+@return index to registered memory key or -1 if not found */
+constexpr int ut_new_get_key_by_base_file(const char *file, size_t len) {
+  for (size_t i = 0; i < n_auto; ++i) {
+    if (ut_string_begins_with(auto_event_names[i], file, len)) {
+      return i;
+    }
+  }
+  return -1;
+}
+
+/** Retrieve a memory key (registered with PFS), given the file name of
+the caller.
+@param[in]	file	portion of the filename - basename, with extension
+@return index to memory key or -1 if not found */
+constexpr int ut_new_get_key_by_file(const char *file) {
+  return ut_new_get_key_by_base_file(file, ut_len_without_extension(file));
+}
+
+// Sending an expression through a template variable forces the compiler to
+// evaluate the expression at compile time (constexpr in itself has no such
+// guarantee, only that the compiler is allowed).
+template <int Value>
+struct force_constexpr {
+  static constexpr int value = Value;
+};
+
+#define UT_NEW_THIS_FILE_PSI_INDEX \
+  (force_constexpr<ut_new_get_key_by_file(MY_BASENAME)>::value)
+
+#define UT_NEW_THIS_FILE_PSI_KEY    \
+  (UT_NEW_THIS_FILE_PSI_INDEX == -1 \
+       ? PSI_NOT_INSTRUMENTED       \
+       : auto_event_keys[UT_NEW_THIS_FILE_PSI_INDEX])
+
+#endif /* __GNUG__ == 5 */
+
 #endif /* UNIV_PFS_MEMORY */
 
 /** A structure that holds the necessary data for performance schema
