@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2019, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Google Inc.
 
 This program is free software; you can redistribute it and/or modify
@@ -50,6 +50,7 @@ the file COPYING.Google.
 #include <debug_sync.h>
 #endif /* !UNIV_HOTBACKUP */
 
+#include "arch0arch.h"
 #include "buf0buf.h"
 #include "buf0flu.h"
 #include "dict0boot.h"
@@ -377,7 +378,7 @@ void log_files_downgrade(log_t &log) {
 
 static lsn_t log_determine_checkpoint_lsn(log_t &log) {
   const lsn_t oldest_lsn = log.available_for_checkpoint_lsn;
-  const lsn_t dict_lsn = log.dict_suggest_checkpoint_lsn;
+  const lsn_t dict_lsn = log.dict_max_allowed_checkpoint_lsn;
 
   ut_a(dict_lsn == 0 || dict_lsn >= log.last_checkpoint_lsn);
 
@@ -465,6 +466,8 @@ void log_files_write_checkpoint(log_t &log, lsn_t next_checkpoint_lsn) {
 
   log_update_limits(log);
 
+  log.dict_max_allowed_checkpoint_lsn = 0;
+
   log_writer_mutex_exit(log);
 }
 
@@ -475,6 +478,10 @@ static void log_checkpoint(log_t &log) {
 
   const lsn_t checkpoint_lsn = log_determine_checkpoint_lsn(log);
 
+  if (arch_page_sys != nullptr) {
+    arch_page_sys->flush_at_checkpoint(checkpoint_lsn);
+  }
+
   LOG_SYNC_POINT("log_before_checkpoint_data_flush");
 
 #ifdef _WIN32
@@ -877,6 +884,7 @@ static bool log_consider_checkpoint(log_t &log) {
   }
 
   log_checkpoint(log);
+
   return (true);
 }
 
@@ -1019,8 +1027,6 @@ void log_update_limits(log_t &log) {
     limit_for_start -= margins;
   }
   log.sn_limit_for_start.store(limit_for_start);
-
-  log.dict_suggest_checkpoint_lsn = 0;
 }
 
 bool log_calc_max_ages(log_t &log) {
