@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2019, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Google Inc.
 Copyright (c) 2016, Percona Inc. All Rights Reserved.
 
@@ -613,8 +613,8 @@ void log_start(log_t &log, checkpoint_no_t checkpoint_no, lsn_t checkpoint_lsn,
 
   log_files_update_offsets(log, start_lsn);
 
-  log.write_ahead_end_offset =
-      ut_uint64_align_up(log.current_file_end_offset, srv_log_write_ahead_size);
+  log.write_ahead_end_offset = ut_uint64_align_up(log.current_file_real_offset,
+                                                  srv_log_write_ahead_size);
 
   lsn_t block_lsn;
   byte *block;
@@ -1182,9 +1182,7 @@ void log_position_collect_lsn_info(const log_t &log, lsn_t *current_lsn,
   ut_a(*current_lsn >= *checkpoint_lsn);
 }
 
-static void log_pad_current_log_block(void)
-/*===========================*/
-{
+static void log_pad_current_log_block(void) {
   byte b = MLOG_DUMMY_RECORD;
   ulint pad_length;
   ulint i;
@@ -1207,11 +1205,9 @@ static void log_pad_current_log_block(void)
   ut_a(lsn % OS_FILE_LOG_BLOCK_SIZE == LOG_BLOCK_HDR_SIZE);
 }
 
-static void log_scrub()
-/*=========*/
-{
+static void log_scrub() {
   log_writer_mutex_enter(*log_sys);
-  ulint cur_lbn = log_block_convert_lsn_to_no(log_sys->current_file_lsn);
+  const auto cur_lbn = log_block_convert_lsn_to_no(log_sys->current_file_lsn);
   if (next_lbn_to_pad == cur_lbn) {
     log_pad_current_log_block();
   }
