@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2012, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2012, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -177,7 +177,7 @@ static MY_ATTRIBUTE((nonnull, warn_unused_result)) dberr_t
 
   /* Write the number of indexes in the table. */
   uint32_t num_indexes = 0;
-  ulint flags = fil_space_get_flags(table->space);
+  uint32_t flags = fil_space_get_flags(table->space);
   bool has_sdi = FSP_FLAGS_HAS_SDI(flags);
 
   if (has_sdi) {
@@ -456,8 +456,8 @@ static MY_ATTRIBUTE((warn_unused_result)) dberr_t row_quiesce_write_header(
   }
 
   /* Write the space flags */
-  ulint space_flags = fil_space_get_flags(table->space);
-  ut_ad(space_flags != ULINT_UNDEFINED);
+  uint32_t space_flags = fil_space_get_flags(table->space);
+  ut_ad(space_flags != UINT32_UNDEFINED);
   mach_write_to_4(value, space_flags);
 
   if (fwrite(&value, 1, sizeof(value), file) != sizeof(value)) {
