@@ -68,6 +68,219 @@ static bool row_vers_non_vc_index_entry_match(dict_index_t *index,
                                               const dtuple_t *ientry2,
                                               ulint *n_non_v_col);
 
+/** Checks if a particular version of a record from clustered index matches
+the secondary index record. The match occurs if and only if two condition hold:
+1) the clust_rec exists and is not delete marked
+2) the values in columns in clust_rec match those in sec_rec
+Please note that the delete marker on sec_rec does not play any role in this
+definition!
+@param[in]    clust_index       the clustered index
+@param[in]    clust_rec         the clustered index record, can be null or
+                                delete marked
+@param[in]    clust_vrow        the values of virtual columns, can be NULL if
+                                the clust_rec was stored in undo log by
+                                operation that did not change any secondary
+                                index column (and was not a DELETE operation)
+@param[in]    clust_offsets     the offsets for clust_rec,
+                                rec_get_offsets(clust_rec, clust_index)
+@param[in]    sec_index         the secondary index
+@param[in]    sec_rec           the secondary index record
+@param[in]    sec_offsets       the offsets for secondary index record,
+                                rec_get_offsets(sec_rec, sec_index)
+@param[in]    comp              the compression flag for both the clustered and
+                                the secondary index, as both are assumed equal
+@param[in]    looking_for_match are we looking for match?
+                                false means that we are looking for non-match
+@param[in]    heap              the heap to be used for all allocations
+@return true iff the clust_rec matches sec_rec
+*/
+static bool row_clust_vers_matches_sec(
+    const dict_index_t *const clust_index, const rec_t *const clust_rec,
+    const dtuple_t *const clust_vrow, const ulint *const clust_offsets,
+    const dict_index_t *const sec_index, const rec_t *const sec_rec,
+    const ulint *const sec_offsets, const ulint comp,
+    const bool looking_for_match, mem_heap_t *const heap) {
+  /** If we could not find a clust_rec version, it means it either never existed
+  or was garbage collected, in either case we can interpret it as the row not
+  being present at that point in time. Similarly, if it is delete marked.
+  In all this cases, we report that there is no match. */
+  if (!clust_rec || rec_get_deleted_flag(clust_rec, comp)) {
+    return false;
+  }
+
+  /** If the index involves virtual columns, then we can relay on the assumption
+  that `trx_undo_prev_version_build` will try to retrieve clust_vrow, and the
+  only reason it can not retrieve it is because there was no change to any of
+  the indexed columns. In particular this should mean, that the answer to the
+  question "does this clust_rec match sec_row?" did not change, and we can
+  return the same value as before, which was !looking_for_match. We know it was
+  not looking_for_match because in such case the loop would stop.
+
+  There are some difficulties we should take into consideration here:
+
+  1. It could be the case that there was no "previous iteration".
+  Indeed, it can happen, that this is the first call.
+  If we got to this line, it means that there are at least two versions of the
+  clustered index row: the most recent, which we don't see, and the one passed
+  here as `clust_rec`, which we know has all the important columns equal to the
+  most recent one.
+  Moreover, we know that `clust_rec` is not delete marked.
+  We also know, that the most recent version is also not delete marked,
+  because, if it was delete marked, then the most recent change would be a
+  DELETE operation, and in such cases we always undo log the values of columns,
+  yet `clust_vrow` is null.
+  So, the most recent version, and the version just before it, not only have
+  the same values of indexed columns, but also the same delete mark.
+  If so, then it is impossible, that this particular change created, or
+  removed a secondary index entry.
+  Therefore we need to continue the loop, and to do so we have to return the
+  opposite of what the loop is searching for, thus !looking_for_match.
+
+  Here's a bit different argument, perhaps more persuasive in case we want to
+  prove that the returned value correctly answers the question "does clust_rec
+  match the sec_rec?".
+  Consider two cases, depending on sec_rec delete mark:
+
+  A) sec_rec is delete marked
+  In this case, looking_for_match is set to true, thus we are about to return
+  false. So, our claim is that clust_rec does not match sec_rec.
+  For consider for a moment the opposite, that clust_rec does match sec_rec -
+  it would follow, that also the most recent version matches sec_rec, as it has
+  the same values of columns, and delete mark. But then, we have that two most
+  recent versions of the clustered index record are not delete marked and match
+  the secondary index record, yet for some reason the change was not
+  synchronized to the secondary index, which is still delete marked!
+  This contradicts the assumption that at most one most recent change is not
+  synchronized to the secondary index.
+
+  B) sec_rec is not delete marked
+  In this case, looking_for_match is set to false, thus we are about to return
+  true. So, our claim is that clust_rec does match sec_rec.
+  For consider for a moment the opposite, that clust_rec doesn't match sec_rec -
+  it would follow, that also the most recent version doesn't match sec_rec, as
+  it has the same values of columns, and delete mark. But then, we have that two
+  most recent versions of the clustered index record do not match the secondary
+  index record, yet for some reason the change was not synchronized to the
+  secondary index, which is still not delete marked!
+  This contradicts the assumption that at most one most recent change is not
+  synchronized to the secondary index.
+
+  2. It could (hypothetically) be the case that in previous iteration the answer
+  was different, because the two versions differ in delete mark
+
+  Again, before getting here we've already established that `clust_rec` is not
+  delete marked, and if `clust_vrow` is missing, then it must mean that the
+  later version is also not delete marked, as otherwise we would have to log all
+  columns to the undo log */
+  if (dict_index_has_virtual(sec_index) && !clust_vrow) {
+    return !looking_for_match;
+  }
+
+  /** Reconstruct all the columns */
+  row_ext_t *ext;
+  dtuple_t *row = row_build(ROW_COPY_POINTERS, clust_index, clust_rec,
+                            clust_offsets, NULL, NULL, NULL, &ext, heap);
+  if (dict_index_has_virtual(sec_index)) {
+    ut_ad(row->n_v_fields == clust_vrow->n_v_fields);
+    dtuple_copy_v_fields(row, clust_vrow);
+  }
+  dtuple_t *entry = row_build_index_entry(row, ext, sec_index, heap);
+
+  /** If the reconstructed values do not match the secondary index then we know
+  we should report no match. We compare the strings in binary mode to make it
+  more robust, because a thread which has changed "a" to "A" should prevent
+  concurrent transactions from peeking into the new binary representation,
+  say via CONVERT(column_name, binary). */
+  dtuple_set_types_binary(entry, dtuple_get_n_fields(entry));
+  return (0 == cmp_dtuple_rec(entry, sec_rec, sec_index, sec_offsets));
+}
+
+/**
+Loops through the history of clustered index record in the undo log, stopping
+after the first version which was not created by the given active transaction,
+and reports if it found a version which satisfies criterion specified by
+looking_for_match. If looking_for_match is true, it searches for a version which
+matches the secondary index record. Otherwise it searches for a version which
+does not match.
+@param[in]      looking_for_match are we looking for match?
+                                  false means that we are looking for non-match
+@param[in]      clust_index       the clustered index
+@param[in]      clust_rec         the clustered index record, can be null or
+                                  delete marked
+@param[in]      clust_offsets     the offsets for clust_rec,
+                                  rec_get_offsets(clust_rec, clust_index)
+@param[in]      sec_index         the secondary index
+@param[in]      sec_rec           the secondary index record
+@param[in]      sec_offsets       the offsets for secondary index record,
+                                  rec_get_offsets(sec_rec, sec_index)
+@param[in]      comp              the compression flag for both the clustered
+                                  and the secondary index, as both are assumed
+                                  equal
+@param[in]      trx_id            the active transaction which created the most
+                                  recent version of clustered index record
+@param[in]      mtr               the mtr inside which we are operating
+@param[in,out]  heap              the heap to be used for all allocations. This
+                                  heap might get deallocated, and a newly
+                                  allocated one will be returned, along with its
+                                  ownership
+@return true iff a version of the clust_rec which is in relation specified by
+looking_for_match to the given sec_rec is found among versions created by trx_id
+or the one version before them
+*/
+static bool row_vers_find_matching(
+    bool looking_for_match, const dict_index_t *const clust_index,
+    const rec_t *const clust_rec, ulint *&clust_offsets,
+    const dict_index_t *const sec_index, const rec_t *const sec_rec,
+    const ulint *const sec_offsets, const ulint comp, const trx_id_t trx_id,
+    mtr_t *const mtr, mem_heap_t *&heap) {
+  const rec_t *version = clust_rec;
+  trx_id_t version_trx_id = trx_id;
+
+  while (version_trx_id == trx_id) {
+    mem_heap_t *old_heap = heap;
+    const dtuple_t *clust_vrow = NULL;
+    rec_t *prev_version = NULL;
+
+    /* We keep the semaphore in mtr on the clust_rec page, so
+    that no other transaction can update it and get an
+    implicit x-lock on rec until mtr_commit(mtr). */
+
+    heap = mem_heap_create(1024);
+
+    trx_undo_prev_version_build(
+        clust_rec, mtr, version, clust_index, clust_offsets, heap,
+        &prev_version, NULL,
+        dict_index_has_virtual(sec_index) ? &clust_vrow : NULL, 0, nullptr);
+
+    /* The oldest visible clustered index version must not be
+    delete-marked, because we never start a transaction by
+    inserting a delete-marked record. */
+    ut_ad(prev_version || !rec_get_deleted_flag(version, comp) ||
+          !trx_rw_is_active(trx_id, NULL, false));
+
+    /* Free version and clust_offsets. */
+    mem_heap_free(old_heap);
+
+    version = prev_version;
+
+    if (version == NULL) {
+      version_trx_id = 0;
+    } else {
+      clust_offsets =
+          rec_get_offsets(version, clust_index, NULL, ULINT_UNDEFINED, &heap);
+      version_trx_id = row_get_rec_trx_id(version, clust_index, clust_offsets);
+    }
+
+    if (row_clust_vers_matches_sec(
+            clust_index, version, clust_vrow, clust_offsets, sec_index, sec_rec,
+            sec_offsets, comp, looking_for_match, heap) == looking_for_match) {
+      return true;
+    }
+  }
+
+  return false;
+}
+
 /** Finds out if an active transaction has inserted or modified a secondary
  index record.
  @return 0 if committed, else the active transaction id;
@@ -76,28 +289,204 @@ static bool row_vers_non_vc_index_entry_match(dict_index_t *index,
  trx_is_active() while holding lock_sys->mutex. */
 UNIV_INLINE
 trx_t *row_vers_impl_x_locked_low(
-    const rec_t *clust_rec,    /*!< in: clustered index record */
-    dict_index_t *clust_index, /*!< in: the clustered index */
-    const rec_t *rec,          /*!< in: secondary index record */
-    dict_index_t *index,       /*!< in: the secondary index */
-    const ulint *offsets,      /*!< in: rec_get_offsets(rec, index) */
-    mtr_t *mtr)                /*!< in/out: mini-transaction */
+    const rec_t *const clust_rec,          /*!< in: clustered index record */
+    const dict_index_t *const clust_index, /*!< in: the clustered index */
+    const rec_t *const sec_rec,            /*!< in: secondary index record */
+    const dict_index_t *const sec_index,   /*!< in: the secondary index */
+    const ulint
+        *const sec_offsets, /*!< in: rec_get_offsets(sec_rec, sec_index) */
+    mtr_t *const mtr)       /*!< in/out: mini-transaction */
 {
   trx_id_t trx_id;
   ibool corrupt;
   ulint comp;
-  ulint rec_del;
-  const rec_t *version;
-  rec_t *prev_version = NULL;
+
   ulint *clust_offsets;
   mem_heap_t *heap;
-  dtuple_t *ientry = NULL;
-  mem_heap_t *v_heap = NULL;
-  const dtuple_t *cur_vrow = NULL;
+
+  /** Here's my best understanding of what this code is doing.
+
+  When we call this function we already have `sec_rec` - a row from secondary
+  index `sec_index`, which includes:
+  - obviously the values of columns mentioned in secondary index definition,
+    in particular materialized values of virtual columns
+  - primary key columns not mentioned explicitly in secondary index definition,
+  - information about row format (`comp`)
+  - information if the row is delete marked or not (`rec_del`: 32 or 0)
+
+  We assume that this `sec_rec` really is a record in the secondary index, as
+  opposed to some artificially "made up" sequence of bytes. Moreover we
+  assume that this secondary index row is currently latched (not to be confused
+  with "locked"), so that `sec_rec` is the most current state of this row.
+
+  Also, we assume, that rows in secondary index are either added, or removed,
+  (or delete marked, or delete un-marked) but never modified.
+  Moreover, we assume, that each of these secondary index operations is done
+  after the primary (clustered) index was modified, to reflect the new state of
+  affairs.
+
+  We assume that `clust_rec` is the current version of the clustered index
+  record to which the secondary record `sec_rec` points to.
+
+  To be more precise:
+
+  Let S[f] mean value of field f in the secondary index record S.
+  Let C[t][f] mean value of field f in version t of clustered record C, where we
+  use consecutive natural numbers to denote versions: t=0,1,...,current_version.
+
+  Note: secondary index is not versioned
+
+  Let S.deleted and C[t].deleted be delete markers of these records.
+
+  Definition 1.
+  We say that secondary index row S `points-to` a clustered index row C if and
+  only if:
+    S[pkey] = C[t][pkey] for each primary key column pkey (for any version t)
+
+  Note: it does not matter which version t we pick, as for our purposes primary
+  key fields may be thought as immutable (say, we emulate their modification by
+  combination of delete + insert).
+
+  Definition 2.
+  We say that secondary index row S `matches` a clustered index row C in
+  version t if and only if:
+    (S[f] = C[t][f] for each column f) and not (C[t].deleted)
+
+  Note: In the above definition f might be a virtual column.
+  Note: There might be multiple versions which a single S `matches`, for
+  example when a transaction modifies a row back and forth, or changes columns
+  which are not indexed by secondary index.
+  Note: The definition of `matches` does not depend on S.deleted
+
+  Definition 3.
+  We say that secondary index row S `corresponds-to` a clustered index row C in
+  version t if and only if:
+    (not(S.deleted) and (S `matches` C[t]))
+    or
+    (S.deleted and not (S `matches` C[t]))
+
+  In other words, S `corresponds-to` C[t] means that the state of secondary
+  index row S is synchronized with the state of the row in clustered index in
+  version t.
+
+  Assumption 1.
+    (S `corresponds-to` C[current_version]) or
+    (S `corresponds-to` C[current_version-1])
+  In other words, `sec_rec` `corresponds-to` either the most current_version of
+  the primary record it `points-to` (i.e. the changes in the clustered index
+  were synchronized to the secondary index), or the current_version-1 - (i.e.
+  the changes in the clustered index was not synchronized to the secondary index
+  yet). This belief is supported by reading the source code and observation that
+  to modify secondary index, one has to modify clustered index first, and
+  modifying clustered index and later secondary index requires holding (implicit
+  or explicit) lock on the clustered index record, so there is at most one
+  transaction operating on any given clustered index row, and thus at most one
+  change "unsynchronized" to secondary index yet.
+
+  An equivalent formulation of Assumption 1 in terms of `matches` is:
+  (not(S.deleted) =>
+    ((S `matches` C[current_version]) or (S `matches` C[current_version-1]))
+  ) and (
+    S.deleted =>
+    not((S `matches` C[current_version]) and (S `matches` C[current_version-1]))
+  )
+  So, a non-deleted S implies that one of the two most recent versions
+  `matches` it, and a deleted S, means that at least one of the two most recent
+  versions does not `match` it.
+
+  Definition 4.
+  We say that S `could-be-authored-by` a clustered index row C in
+  version t if and only if:
+    (S `corresponds-to` C[t]) and !(S `corresponds-to` C[t-1])
+
+  This can be equivalently expressed using `matches` relation as:
+    (not(S.deleted) and (S `matches` C[t]) and not(S `matches` C[t-1]))
+    or
+    (S.deleted and not(S `matches` C[t]) and (S `matches` C[t-1]))
+
+
+  Definition 5.
+  We say that secondary index row S `was-authored-by` a clustered index row C in
+  version t if and only if:
+    (S `could-be-authored-by` C[t]) and
+    (for each v > t. not(S `could-be-authored-by` C[v]))
+  So, t is the latest version in which S `could-be-authored-by` C[t].
+
+  Equivalently, one can define `was-authored-by` in terms of `matches`, by
+  identifying the most recent version t for which `matches` relation between
+  S and C[t] has changed in the right direction, that is,
+  in case S.deleted we search for the first change from (S `matches` C[t-1]) to
+  not(S `matches` C[t]), while in case of not(S.deleted) we search for the first
+  change from not(S `matches` C[t-1]) to (S `matches` C[t]).
+
+  We are now ready to explain precisely what the call to
+  row_vers_impl_x_locked_low(C=`clust_rec`,...,S=`sec_rec`,...)
+  tries to achieve.
+
+  Post-condition of row_vers_impl_x_locked_low:
+  =============================================
+
+  If there is t, such that S `was-authored-by` C[t], and C[t].trx_id is active
+  then the return value is C[t].trx_id.
+  Otherwise the return value is 0.
+
+  Explanation of the algorithm in row_vers_impl_x_locked_low:
+  ===========================================================
+
+  The implementation is tricky, as it tries hard to avoid ever looking at the
+  C[current_version], instead looking only at older versions.
+  (One reason for this effort, IMHO, is that virtual columns might be expensive
+  to materialize, and are not stored in clustered index at all. Another reason,
+  I guess, might be to have only one way of reading data - from undo log).
+  Take a moment to realize that this is wonderful that it's even possible, as
+  this is not apparent from the Def 5! After all it might well be the case that
+  the `t` we are looking for is equal to `current_version` in which case the
+  definition of `was-authored-by` used naively would require us to check if S
+  `matches` C[current_version], which in turn done naively forces us to
+  look at fields of C[current_version]!
+
+  So, how can we do that without ever looking at C[current_version] fields?
+
+  We start by reading C[current_version].trx_id, and this is the only piece of
+  information we read from current_version.
+  We store that in `trx_id` variable.
+
+  We check if `trx_id` is active.
+
+  If `trx_id` is not active, then we know that we can return 0.
+  Why? Because it is impossible for any other C[t].trx_id to be still active, if
+  the most recent trx to modify the record is already inactive.
+
+  From now on we assume that `trx_id` is active.
+
+  We observe that the definition of S `was-authored-by` C[t] requires
+  not(S `corresponds-to` C[t-1]).
+  So, one thing we can use to filter interesting versions, is to proceed through
+  most recent versions t=current_version, current_version-1, ...
+  until we find the first t, such that not(S `corresponds-to` C[t-1]).
+  Surprisingly this is the only condition we have to check! Why?
+  Observe, that it must also be the case that (S `corresponds-to` C[t]),
+  because we either have tested that explicitly in the previous step of the loop
+  or in case of first iteration, it follows from Assumption 1.
+  This means, that (S `could-be-authored-by` C[t]), and since the t is maximal,
+  we have (S `was-authored-by` C[t]).
+
+  Therefore our algorithm is to simply loop over versions t, as long as
+  C[t].trx_id = trx_id, and stop as soon as not(S `corresponds-to` C[t-1]) in
+  which case the answer is yes, or if we can't find such a version, the answer
+  is no.
+
+  The reality is however much more complicated, as it needs to deal with:
+  A) incomplete history of versions (we remove old undo log entries from tail)
+  B) missing information about virtual columns (we don't log values of virtual
+  columns to undo log if they had not changed)
+
+  I'll explain our approach to these two problems in comments at the place they
+  are handled.*/
 
   DBUG_ENTER("row_vers_impl_x_locked_low");
 
-  ut_ad(rec_offs_validate(rec, index, offsets));
+  ut_ad(rec_offs_validate(sec_rec, sec_index, sec_offsets));
 
   heap = mem_heap_create(1024);
 
@@ -120,208 +509,22 @@ trx_t *row_vers_impl_x_locked_low(
     DBUG_RETURN(0);
   }
 
-  comp = page_rec_is_comp(rec);
-  ut_ad(index->table == clust_index->table);
-  ut_ad(!!comp == dict_table_is_comp(index->table));
+  comp = page_rec_is_comp(sec_rec);
+  ut_ad(sec_index->table == clust_index->table);
+  ut_ad(!!comp == dict_table_is_comp(sec_index->table));
   ut_ad(!comp == !page_rec_is_comp(clust_rec));
 
-  rec_del = rec_get_deleted_flag(rec, comp);
+  bool looking_for_match = rec_get_deleted_flag(sec_rec, comp);
 
-  if (dict_index_has_virtual(index)) {
-    ulint n_ext;
-    ulint est_size = DTUPLE_EST_ALLOC(index->n_fields);
-
-    /* Allocate the dtuple for virtual columns extracted from undo
-    log with its own heap, so to avoid it being freed as we
-    iterating in the version loop below. */
-    v_heap = mem_heap_create(est_size);
-    ientry = row_rec_to_index_entry(rec, index, offsets, &n_ext, v_heap);
-  }
-
-  /* We look up if some earlier version, which was modified by
-  the trx_id transaction, of the clustered index record would
-  require rec to be in a different state (delete marked or
-  unmarked, or have different field values, or not existing). If
-  there is such a version, then rec was modified by the trx_id
-  transaction, and it has an implicit x-lock on rec. Note that
-  if clust_rec itself would require rec to be in a different
-  state, then the trx_id transaction has not yet had time to
-  modify rec, and does not necessarily have an implicit x-lock
-  on rec. */
-
-  for (version = clust_rec;; version = prev_version) {
-    row_ext_t *ext;
-    dtuple_t *row;
-    dtuple_t *entry;
-    ulint vers_del;
-    trx_id_t prev_trx_id;
-    mem_heap_t *old_heap = heap;
-    const dtuple_t *vrow = NULL;
-
-    /* We keep the semaphore in mtr on the clust_rec page, so
-    that no other transaction can update it and get an
-    implicit x-lock on rec until mtr_commit(mtr). */
-
-    heap = mem_heap_create(1024);
-
-    trx_undo_prev_version_build(clust_rec, mtr, version, clust_index,
-                                clust_offsets, heap, &prev_version, NULL,
-                                dict_index_has_virtual(index) ? &vrow : NULL, 0,
-                                nullptr);
-
-    /* The oldest visible clustered index version must not be
-    delete-marked, because we never start a transaction by
-    inserting a delete-marked record. */
-    ut_ad(prev_version || !rec_get_deleted_flag(version, comp) ||
-          !trx_rw_is_active(trx_id, NULL, false));
-
-    /* Free version and clust_offsets. */
-    mem_heap_free(old_heap);
-
-    if (prev_version == NULL) {
-      /* We reached the oldest visible version without
-      finding an older version of clust_rec that would
-      match the secondary index record.  If the secondary
-      index record is not delete marked, then clust_rec
-      is considered the correct match of the secondary
-      index record and hence holds the implicit lock. */
-
-      if (rec_del) {
-        /* The secondary index record is del marked.
-        So, the implicit lock holder of clust_rec
-        did not modify the secondary index record yet,
-        and is not holding an implicit lock on it.
-
-        This assumes that whenever a row is inserted
-        or updated, the leaf page record always is
-        created with a clear delete-mark flag.
-        (We never insert a delete-marked record.) */
-        trx_release_reference(trx);
-        trx = 0;
-      }
-
-      break;
-    }
-
-    clust_offsets = rec_get_offsets(prev_version, clust_index, NULL,
-                                    ULINT_UNDEFINED, &heap);
-
-    vers_del = rec_get_deleted_flag(prev_version, comp);
-
-    prev_trx_id = row_get_rec_trx_id(prev_version, clust_index, clust_offsets);
-
-    /* The stack of versions is locked by mtr.  Thus, it
-    is safe to fetch the prefixes for externally stored
-    columns. */
-
-    row = row_build(ROW_COPY_POINTERS, clust_index, prev_version, clust_offsets,
-                    NULL, NULL, NULL, &ext, heap);
-
-    if (dict_index_has_virtual(index)) {
-      if (vrow) {
-        /* Keep the virtual row info for the next
-        version */
-        cur_vrow = dtuple_copy(vrow, v_heap);
-        dtuple_dup_v_fld(cur_vrow, v_heap);
-      }
-
-      if (!cur_vrow) {
-        ulint n_non_v_col = 0;
-
-        /* Build index entry out of row */
-        entry = row_build_index_entry(row, ext, index, heap);
-
-        /* entry may be NULL if a record was inserted in place
-        of a deleted record, and the BLOB pointers of the new
-        record were not initialized yet.  But in that case,
-        prev_version should be NULL. */
-
-        ut_a(entry != NULL);
-
-        /* If the indexed virtual columns has changed,
-        there must be log record to generate vrow.
-        Otherwise, it is not changed, so no need
-        to compare */
-        if (!row_vers_non_vc_index_entry_match(index, ientry, entry,
-                                               &n_non_v_col)) {
-          if (rec_del != vers_del) {
-            break;
-          }
-        } else if (!rec_del) {
-          break;
-        }
-
-        goto result_check;
-      } else {
-        ut_ad(row->n_v_fields == cur_vrow->n_v_fields);
-        dtuple_copy_v_fields(row, cur_vrow);
-      }
-    }
-
-    entry = row_build_index_entry(row, ext, index, heap);
-
-    /* entry may be NULL if a record was inserted in place
-    of a deleted record, and the BLOB pointers of the new
-    record were not initialized yet.  But in that case,
-    prev_version should be NULL. */
-
-    ut_a(entry != NULL);
-
-    /* If we get here, we know that the trx_id transaction
-    modified prev_version. Let us check if prev_version
-    would require rec to be in a different state. */
-
-    /* The previous version of clust_rec must be
-    accessible, because clust_rec was not a fresh insert.
-    There is no guarantee that the transaction is still
-    active. */
-
-    /* We check if entry and rec are identified in the alphabetical
-    ordering */
-    if (0 == cmp_dtuple_rec(entry, rec, index, offsets)) {
-      /* The delete marks of rec and prev_version should be
-      equal for rec to be in the state required by
-      prev_version */
-
-      if (rec_del != vers_del) {
-        break;
-      }
-
-      /* It is possible that the row was updated so that the
-      secondary index record remained the same in
-      alphabetical ordering, but the field values changed
-      still. For example, 'abc' -> 'ABC'. Check also that. */
-
-      dtuple_set_types_binary(entry, dtuple_get_n_fields(entry));
-
-      if (0 != cmp_dtuple_rec(entry, rec, index, offsets)) {
-        break;
-      }
-
-    } else if (!rec_del) {
-      /* The delete mark should be set in rec for it to be
-      in the state required by prev_version */
-
-      break;
-    }
-
-  result_check:
-    if (trx->id != prev_trx_id) {
-      /* prev_version was the first version modified by
-      the trx_id transaction: no implicit x-lock */
-
-      trx_release_reference(trx);
-      trx = 0;
-      break;
-    }
+  if (!row_vers_find_matching(looking_for_match, clust_index, clust_rec,
+                              clust_offsets, sec_index, sec_rec, sec_offsets,
+                              comp, trx_id, mtr, heap)) {
+    trx_release_reference(trx);
+    trx = 0;
   }
 
   DBUG_PRINT("info", ("Implicit lock is held by trx:" TRX_ID_FMT, trx_id));
 
-  if (v_heap != NULL) {
-    mem_heap_free(v_heap);
-  }
-
   mem_heap_free(heap);
   DBUG_RETURN(trx);
 }
@@ -333,9 +536,9 @@ trx_t *row_vers_impl_x_locked_low(
  negatives. The caller must confirm all positive results by calling
  trx_is_active() while holding lock_sys->mutex. */
 trx_t *row_vers_impl_x_locked(
-    const rec_t *rec,     /*!< in: record in a secondary index */
-    dict_index_t *index,  /*!< in: the secondary index */
-    const ulint *offsets) /*!< in: rec_get_offsets(rec, index) */
+    const rec_t *rec,          /*!< in: record in a secondary index */
+    const dict_index_t *index, /*!< in: the secondary index */
+    const ulint *offsets)      /*!< in: rec_get_offsets(rec, index) */
 {
   mtr_t mtr;
   trx_t *trx;
