@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -69,9 +69,12 @@ Tablespace::Tablespace()
     : m_space_id(++m_last_used_space_id), m_inited(), m_thread_id() {
   ut_ad(m_space_id <= dict_sys_t::s_max_temp_space_id);
   m_purpose = TBSP_NONE;
+  mutex_create(LATCH_ID_TEMP_POOL_TBLSP, &m_mutex);
 }
 
 Tablespace::~Tablespace() {
+  mutex_destroy(&m_mutex);
+
   if (!m_inited) {
     return;
   }
@@ -94,7 +97,7 @@ dberr_t Tablespace::create() {
   ut_ad(m_space_id > dict_sys_t::s_min_temp_space_id);
 
   /* Create the filespace flags */
-  ulint fsp_flags =
+  uint32_t fsp_flags =
       fsp_flags_init(univ_page_size, /* page sizes and a flag if compressed */
                      false,          /* needed only for compressed tables */
                      false,          /* has DATA_DIR */
@@ -140,8 +143,11 @@ bool Tablespace::truncate() {
     return (false);
   }
 
+  acquire();
+
   bool success = fil_truncate_tablespace(m_space_id, FIL_IBT_FILE_INITIAL_SIZE);
   if (!success) {
+    release();
     return (success);
   }
 
@@ -152,6 +158,8 @@ bool Tablespace::truncate() {
   mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO);
   fsp_header_init(m_space_id, FIL_IBT_FILE_INITIAL_SIZE, &mtr, false);
   mtr_commit(&mtr);
+
+  release();
   return (true);
 }
 
@@ -175,7 +183,7 @@ void Tablespace::decrypt() {
 
   fil_space_t *space = fil_space_get(m_space_id);
 
-  FSP_FLAGS_UNSET_ENCRYPTION(space->flags);
+  fsp_flags_unset_encryption(space->flags);
 
   /* There is no need to empty the encryption info in page 0
   here. This is because the file is just truncated and extended
@@ -193,6 +201,23 @@ void Tablespace::decrypt() {
   rw_lock_x_unlock(&space->latch);
 }
 
+void Tablespace::rotate_encryption_key() {
+  if (!is_encrypted()) {
+    return;
+  }
+
+  acquire();
+
+  fil_space_t *space = fil_space_get(m_space_id);
+
+  bool success = encryption_rotate_low(space);
+  if (!success) {
+    ib::warn(ER_XB_MSG_6, space->name);
+  }
+
+  release();
+}
+
 uint32_t Tablespace::file_id() const {
   return (m_space_id - dict_sys_t::s_min_temp_space_id);
 }
