@@ -439,6 +439,7 @@ LatchDebug::LatchDebug() {
   LEVEL_MAP_INSERT(SYNC_LOG_ARCH);
   LEVEL_MAP_INSERT(SYNC_PAGE_ARCH);
   LEVEL_MAP_INSERT(SYNC_PAGE_ARCH_OPER);
+  LEVEL_MAP_INSERT(SYNC_PAGE_ARCH_CLIENT);
   LEVEL_MAP_INSERT(SYNC_PAGE_CLEANER);
   LEVEL_MAP_INSERT(SYNC_PURGE_QUEUE);
   LEVEL_MAP_INSERT(SYNC_TRX_SYS_HEADER);
@@ -699,6 +700,7 @@ Latches *LatchDebug::check_order(const latch_t *latch,
     case SYNC_LOG_ARCH:
     case SYNC_PAGE_ARCH:
     case SYNC_PAGE_ARCH_OPER:
+    case SYNC_PAGE_ARCH_CLIENT:
     case SYNC_DOUBLEWRITE:
     case SYNC_SEARCH_SYS:
     case SYNC_THREADS:
@@ -1307,6 +1309,9 @@ static void sync_latch_meta_init() UNIV_NOTHROW {
   LATCH_ADD_MUTEX(PAGE_ARCH_OPER, SYNC_PAGE_ARCH_OPER,
                   page_sys_arch_oper_mutex_key);
 
+  LATCH_ADD_MUTEX(PAGE_ARCH_CLIENT, SYNC_PAGE_ARCH_CLIENT,
+                  page_sys_arch_client_mutex_key);
+
   LATCH_ADD_MUTEX(PAGE_CLEANER, SYNC_PAGE_CLEANER, page_cleaner_mutex_key);
 
   LATCH_ADD_MUTEX(PURGE_SYS_PQ, SYNC_PURGE_QUEUE, purge_sys_pq_mutex_key);
@@ -1371,6 +1376,8 @@ static void sync_latch_meta_init() UNIV_NOTHROW {
   LATCH_ADD_MUTEX(TEMP_POOL_MANAGER, SYNC_TEMP_POOL_MANAGER,
                   temp_pool_manager_mutex_key);
 
+  LATCH_ADD_MUTEX(TEMP_POOL_TBLSP, SYNC_NO_ORDER_CHECK, PFS_NOT_INSTRUMENTED);
+
   LATCH_ADD_MUTEX(TRX, SYNC_TRX, trx_mutex_key);
 
   LATCH_ADD_MUTEX(LOCK_SYS, SYNC_LOCK_SYS, lock_mutex_key);
@@ -1477,6 +1484,8 @@ static void sync_latch_meta_init() UNIV_NOTHROW {
                   PFS_NOT_INSTRUMENTED);
   LATCH_ADD_MUTEX(FIL_CRYPT_THREADS_MUTEX, SYNC_NO_ORDER_CHECK,
                   PFS_NOT_INSTRUMENTED);
+  LATCH_ADD_MUTEX(FIL_CRYPT_THREADS_SET_CNT_MUTEX, SYNC_NO_ORDER_CHECK,
+                  PFS_NOT_INSTRUMENTED);
   LATCH_ADD_MUTEX(FIL_CRYPT_START_ROTATE_MUTEX, SYNC_NO_ORDER_CHECK,
                   PFS_NOT_INSTRUMENTED);
 
