@@ -1789,10 +1789,7 @@ int ha_myisam::create(const char *name, TABLE *table_arg,
   TABLE_SHARE *share = table_arg->s;
   uint options = share->db_options_in_use;
   DBUG_ENTER("ha_myisam::create");
-  if (ha_create_info->encrypt_type.length > 0) {
-    set_my_errno(HA_WRONG_CREATE_OPTION);
-    DBUG_RETURN(HA_WRONG_CREATE_OPTION);
-  }
+
   for (i = 0; i < share->keys; i++) {
     if (table_arg->key_info[i].flags & HA_USES_PARSER) {
       create_flags |= HA_CREATE_RELIES_ON_SQL_LAYER;
