@@ -1,3 +1,4 @@
+
 /*
    Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
@@ -825,7 +826,9 @@ static int keys_compare(const void *a, const void *b, const void *c) {
                     USE_WHOLE_KEY, SEARCH_SAME, not_used);
 }
 
-static int keys_free(uchar *key, TREE_FREE mode, bulk_insert_param *param) {
+static void keys_free(void *key_, TREE_FREE mode, const void *param_) {
+  auto *key = static_cast<uchar *>(key_);
+  const auto *param = static_cast<const bulk_insert_param *>(param_);
   /*
     Probably I can use info->lastkey here, but I'm not sure,
     and to be safe I'd better use local lastkey.
@@ -840,19 +843,19 @@ static int keys_free(uchar *key, TREE_FREE mode, bulk_insert_param *param) {
         mysql_rwlock_wrlock(&param->info->s->key_root_lock[param->keynr]);
         param->info->s->keyinfo[param->keynr].version++;
       }
-      return 0;
+      break;
     case free_free:
       keyinfo = param->info->s->keyinfo + param->keynr;
       keylen = _mi_keylength(keyinfo, key);
       memcpy(lastkey, key, keylen);
-      return _mi_ck_write_btree(param->info, param->keynr, lastkey,
-                                keylen - param->info->s->rec_reflength);
+      _mi_ck_write_btree(param->info, param->keynr, lastkey,
+                         keylen - param->info->s->rec_reflength);
+      break;
     case free_end:
       if (param->info->s->concurrent_insert)
         mysql_rwlock_unlock(&param->info->s->key_root_lock[param->keynr]);
-      return 0;
+      break;
   }
-  return -1;
 }
 
 int mi_init_bulk_insert(MI_INFO *info, ulong cache_size, ha_rows rows) {
@@ -899,8 +902,8 @@ int mi_init_bulk_insert(MI_INFO *info, ulong cache_size, ha_rows rows) {
       params->keynr = i;
       /* Only allocate a 16'th of the buffer at a time */
       init_tree(&info->bulk_insert[i], cache_size * key[i].maxlength,
-                cache_size * key[i].maxlength, 0, keys_compare, 0,
-                (tree_element_free)keys_free, (void *)params++);
+                cache_size * key[i].maxlength, 0, keys_compare, 0, keys_free,
+                (void *)params++);
     } else
       info->bulk_insert[i].root = 0;
   }
