@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2012, 2021, Oracle and/or its affiliates.
+Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -19,7 +19,7 @@ GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 /******************************************************************************
@@ -75,7 +75,7 @@ mcc.configuration.calculations.typeSetup = typeSetup;
 
 /****************************** Implementation  *******************************/
 
-// Add processes to the cluster if noone exists already
+// Add processes to the cluster if none exists already
 function autoConfigure() {
     var waitCondition = new dojo.Deferred();
     // If no processes, add
@@ -111,7 +111,7 @@ function autoConfigure() {
             if (!hosts || hosts.length == 0 || 
                     (hosts.length == 1 && hosts[0].getValue("anyHost"))) {
                 alert("No hosts - unable to add default processes")
-                mcc.util.dbg("No hosts - unable tocall add default processes")
+                mcc.util.dbg("No hosts - unable to call add default processes")
                 waitCondition.resolve();
                 return;
             }
@@ -120,7 +120,10 @@ function autoConfigure() {
             var names = [];
             var familyHead = [];    // Ptype hashed on family name
             var typeHead = [];      // Ptype hashed on type name
-            var dataNodeId = 1; 
+            var dataNodeId = 1;
+            var mgmtNodeID = 49;
+            var sqlNodeID = 53;
+            var apiNodeID = 231;
             var otherNodeId = 49; 
 
             // Get ids of all process types
@@ -134,15 +137,36 @@ function autoConfigure() {
                 names[pType.getValue("name")] = pType.getValue("nodeLabel");
             },
             function () {
-                // Add new process
+                // Add new process following same ID rules set in MCCStorage.js::initializeProcessTypeStorage
+                var NID = 0;
                 function newProcess(pname, host) {
+                    switch(pname) {
+                        case "ndb_mgmd":
+                            NID = mgmtNodeID;
+                            mgmtNodeID++;
+                            otherNodeId++;
+                            break;
+                        case "mysqld":
+                            NID = sqlNodeID;
+                            sqlNodeID++;
+                            otherNodeId++;
+                            break;
+                        case "api":
+                            NID = apiNodeID;
+                            apiNodeID++;
+                            otherNodeId++;
+                            break;
+                        default:
+                            NID = dataNodeId;
+                            dataNodeId++;
+                    }
+
                     mcc.storage.processStorage().newItem({
                         name: names[pname] + " " + 
                                 typeHead[pname].getValue("currSeq"),
                         host: host.getId(),
                         processtype: typeIds[pname],
-                        NodeId: (pname == "ndbd" || pname == "ndbmtd") ? 
-                                dataNodeId++ : otherNodeId++,
+                        NodeId: NID,
                         seqno: typeHead[pname].getValue("currSeq")
                     });
                 }
@@ -268,7 +292,7 @@ function hwDepParams(processTypeName) {
     // Single deferred to callback
     var waitCondition = new dojo.Deferred();
 
-    // Array of deferreds to wait for
+    // Array of deferrers to wait for
     var waitConditions= [];
     var waitList; 
 
@@ -318,12 +342,21 @@ function typeSetup(processTypeItem) {
             // Leave process type level datadir undefined
             waitCondition.resolve();
         } else if (processFamilyName == "data") {
+            // Get portbase, set default for ServerPort
+            var spbase = processFamilyItem.getValue("Portbase");
+            if (spbase === undefined) {
+                spbase = mcc.configuration.getPara(processFamilyName, null, 
+                        "ServerPort", "defaultValueType");
+            }
+            mcc.configuration.setPara(processFamilyName, null, "ServerPort",
+                    "defaultValueType", spbase);
 
             // Check parameters that depend on cluster defaults
             mcc.storage.clusterStorage().getItem(0).then(function (cluster) {
                 // Leave process type level datadir undefined
 
-                // Check real time or web mode
+                // Check real time or web mode.
+                // This is OK since all those values are "undefined" in parameters.ja.
                 if (cluster.getValue("apparea") != "realtime") {
                     mcc.configuration.setPara(processFamilyName, null, 
                             "HeartbeatIntervalDbDb", "defaultValueType", 15000);
@@ -337,6 +370,7 @@ function typeSetup(processTypeItem) {
                 }
 
                 // Check read/write load
+                // This is OK since all those values are "undefined" in parameters.ja.
                 if (cluster.getValue("writeload") == "high") {
                     mcc.configuration.setPara(processFamilyName, null, 
                             "SendBufferMemory", "defaultValueType", 8);
@@ -360,15 +394,17 @@ function typeSetup(processTypeItem) {
                             "RedoBuffer", "defaultValueType", 32);
                 }
 
-                // Get disk page buffer memory, assign shared global memory
+                // Get disk page buffer memory, assign shared global memory.
                 var diskBuf = processFamilyItem.getValue(
                         mcc.configuration.getPara(processFamilyName, null, 
                                 "DiskPageBufferMemory", "attribute"));
                 if (!diskBuf) {
+                    // If user didn't set it, read from parameters.js.
                     diskBuf = mcc.configuration.getPara(processFamilyName, null, 
                         "DiskPageBufferMemory", "defaultValueType");
                 }
-
+                
+                // If user didn't set it, calculate.
                 if (diskBuf > 8192) {
                     mcc.configuration.setPara(processFamilyName, null, 
                             "SharedGlobalMemory", "defaultValueType", 1024);
@@ -377,7 +413,7 @@ function typeSetup(processTypeItem) {
                             "SharedGlobalMemory", "defaultValueType", 384);
                 } else {
                     mcc.configuration.setPara(processFamilyName, null, 
-                            "SharedGlobalMemory", "defaultValueType", 20);
+                            "SharedGlobalMemory", "defaultValueType", 32);
                 }
 
                 // Restrict MaxNoOfTables
@@ -385,6 +421,7 @@ function typeSetup(processTypeItem) {
                         mcc.configuration.getPara(processFamilyName, null, 
                                 "MaxNoOfTables", "attribute"));
                 if (maxTab) {
+                    //IF user has set it THEN do check.
                     if (maxTab > 20320) {
                         processFamilyItem.setValue(
                             mcc.configuration.getPara(processFamilyName, null, 
@@ -533,6 +570,23 @@ function ndbd_setup(processItem, processFamilyItem, host, waitCondition) {
             "defaultValueInstance", datadir +
             processItem.getValue("NodeId") + dirSep);
 
+    // Get colleague nodes, find own index on host
+    mcc.util.getColleagueNodes(processItem).then(function (colleagues) {
+        var myIdx = dojo.indexOf(colleagues, processItem.getId());
+
+        // Get type's overridden port base
+        var pbase = processFamilyItem.getValue("Portbase");
+
+        // If not overridden, use type default
+        if (pbase === undefined) {
+            pbase = mcc.configuration.getPara(processFamilyName, null, 
+                    "Portbase", "defaultValueType");
+        }
+        // Set port using retrieved portbase and node index on host
+        mcc.configuration.setPara(processFamilyName, id, "ServerPort",
+                "defaultValueInstance", myIdx + pbase);
+    });
+
     // Get cluster attributes
     mcc.storage.clusterStorage().getItem(0).then(function (cluster) {
         // Get node distribution (deferred)
@@ -730,8 +784,9 @@ function instanceSetup(processFamilyName, processItem) {
             mcc.configuration.setPara(processFamilyName, id, "HostName",
                     "defaultValueInstance", null);
         } else {
+            //Use HostName=internalIP to avoid mixing LOCAL & REMOTE hosts.
             mcc.configuration.setPara(processFamilyName, id, "HostName",
-                    "defaultValueInstance", host.getValue("name"));
+                    "defaultValueInstance", host.getValue("internalIP"));
         }
 
         // Get prototypical process type and do process specific assignments
@@ -754,6 +809,6 @@ function instanceSetup(processFamilyName, processItem) {
 /******************************** Initialize  *********************************/
 
 dojo.ready(function () {
-    mcc.util.dbg("Configuration calulations module initialized");
+    mcc.util.dbg("Configuration calculations module initialized");
 });
 
