@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -473,9 +473,14 @@ NdbSqlUtil::cmpChar(const void* info, const void* p1, unsigned n1, const void* p
   const uchar* v1 = (const uchar*)p1;
   const uchar* v2 = (const uchar*)p2;
   CHARSET_INFO* cs = (CHARSET_INFO*)info;
-  // compare with space padding
-  int k = (*cs->coll->strnncollsp)(cs, v1, n1, v2, n2);
-  return k;
+
+  // Comparing with a NO_PAD collation requires trailing spaces to be stripped.
+  if (cs->pad_attribute == NO_PAD)
+  {
+    n1 = cs->cset->lengthsp(cs, (const char *)p1, n1);
+    n2 = cs->cset->lengthsp(cs, (const char *)p2, n2);
+  }
+  return (*cs->coll->strnncollsp)(cs, v1, n1, v2, n2);
 }
 
 int
@@ -1093,6 +1098,7 @@ strnxfrm_bug7284(const CHARSET_INFO* cs,
 
 int
 NdbSqlUtil::strnxfrm_hash(const CHARSET_INFO* cs,
+                          Uint32 typeId,
                           uchar* dst, unsigned bufLen,
                           const uchar* src, unsigned srcLen,
                           unsigned maxLen)
@@ -1106,8 +1112,16 @@ NdbSqlUtil::strnxfrm_hash(const CHARSET_INFO* cs,
    */
   if (cs->pad_attribute == NO_PAD && cs != &my_charset_bin)
   {
+    assert(typeId == NdbSqlUtil::Type::Char ||
+           typeId == NdbSqlUtil::Type::Varchar ||
+           typeId == NdbSqlUtil::Type::Longvarchar);
+
+    // Fixed length char need trailing spaces to be stripped if NO_PAD
+    if (typeId == NdbSqlUtil::Type::Char)
+      srcLen = cs->cset->lengthsp(cs, reinterpret_cast<const char*>(src), srcLen);
+
     // Hash the string using the collations hash function.
-    ulong hash = 0, n2 = 0;
+    uint64 hash = 0, n2 = 0;
     (*cs->coll->hash_sort)(cs, src, srcLen, &hash, &n2);
 
     if (verify_hash_only_usage)  //Debug only
@@ -1161,7 +1175,7 @@ NdbSqlUtil::strnxfrm_hash_len(const CHARSET_INFO* cs,
   if (cs->pad_attribute == NO_PAD && cs != &my_charset_bin)
   {
     //The hash_sort() value, see strnxfrm_hash
-    return sizeof(ulong);
+    return sizeof(uint64);
   }
   else if (likely(cs->strxfrm_multiply > 0))
   {
