@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -261,6 +261,22 @@ compute_acc_32kpages(const ndb_mgm_configuration_iterator * p)
  * file group and there can only be one such group. It is using overallocating
  * if this happens through an SQL command.
  *
+ * RG_QUERY_MEMORY:
+ * Like transaction memory, query memory may be overallocated. Unlike
+ * transaction memory, query memory may not use the last free 10% of shared
+ * global memory.  This is controlled by setting the reserved memory to zero.
+ * This indicates to memory manager that the resource is low priority and
+ * should not be allowed to starve out other higher priority resource.
+ *
+ * Dbspj is the user of query memory serving join queries and "only" read data.
+ * A bad join query could easily consume a lot of memory.
+ *
+ * Dbtc, which uses transaction memory, on the other hand also serves writes,
+ * and typically memory consumption per request are more limited.
+ *
+ * In situations there there are small amount of free memory left one want to
+ * Dbtc to be prioritized over Dbspj.
+ *
  * Overallocating and total memory
  * -------------------------------
  * The total memory allocated by the global memory manager is the sum of the
@@ -452,6 +468,47 @@ init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
   }
 
   Uint32 transmem = 0;
+  Uint32 tcInstances = 1;
+  if (globalData.ndbMtTcThreads > 1)
+  {
+    tcInstances = globalData.ndbMtTcThreads;
+  }
+
+  Uint32 MaxNoOfConcurrentIndexOperations = 8192;
+  Uint32 MaxNoOfConcurrentOperations = 32768;
+  Uint32 MaxNoOfConcurrentScans = 256;
+  Uint32 MaxNoOfConcurrentTransactions = 4096;
+  Uint32 MaxNoOfFiredTriggers = 4000;
+  Uint32 MaxNoOfLocalScans = 0;
+  Uint32 TransactionBufferMemory = 1048576;
+
+  ndb_mgm_get_int_parameter(p, CFG_DB_NO_INDEX_OPS,
+                            &MaxNoOfConcurrentIndexOperations);
+  ndb_mgm_get_int_parameter(p, CFG_DB_NO_OPS, &MaxNoOfConcurrentOperations);
+  ndb_mgm_get_int_parameter(p, CFG_DB_NO_SCANS, &MaxNoOfConcurrentScans);
+  ndb_mgm_get_int_parameter(p, CFG_DB_NO_TRANSACTIONS, &MaxNoOfConcurrentTransactions);
+  ndb_mgm_get_int_parameter(p, CFG_DB_NO_TRIGGERS, &MaxNoOfFiredTriggers);
+  // Use CFG_TC_LOCAL_SCAN instead of CFG_DB_NO_LOCAL_SCANS since it is
+  // calculated if MaxNoOfLocalScans is not set.
+  ndb_mgm_get_int_parameter(p, CFG_TC_LOCAL_SCAN, &MaxNoOfLocalScans);
+  ndb_mgm_get_int_parameter(p, CFG_DB_TRANS_BUFFER_MEM, &TransactionBufferMemory);
+
+  const Uint32 TakeOverOperations = MaxNoOfConcurrentOperations;
+
+  Uint64 transmem_bytes =
+      globalEmulatorData.theSimBlockList->getTransactionMemoryNeed(
+        tcInstances,
+        p,
+        TakeOverOperations,
+        MaxNoOfConcurrentIndexOperations,
+        MaxNoOfConcurrentOperations,
+        MaxNoOfConcurrentScans,
+        MaxNoOfConcurrentTransactions,
+        MaxNoOfFiredTriggers,
+        MaxNoOfLocalScans,
+        TransactionBufferMemory);
+
+  transmem = transmem_bytes / 32768;
   {
     /**
      * Request extra undo buffer memory to be allocated when
@@ -482,7 +539,7 @@ init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
         Uint32 undopages = Uint32(undo_buffer_size / GLOBAL_PAGE_SIZE);
         g_eventLogger->info("reserving %u extra pages for undo buffer memory",
                             undopages);
-        transmem = undopages;
+        transmem += undopages;
         Resource_limit rl;
         rl.m_min = transmem;
         rl.m_max = 0;
@@ -492,6 +549,21 @@ init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
     }
   }
 
+  {
+    Resource_limit rl;
+    /*
+     * Setting m_min = 0 makes QUERY_MEMORY a low priority resource group
+     * which can not use the last 10% of shared global page memory.
+     *
+     * For example TRANSACTION_MEMORY will have access to those last
+     * percent of global shared global page memory.
+     */
+    rl.m_min = 0;
+    rl.m_max = 0;
+    rl.m_resource_id = RG_QUERY_MEMORY;
+    ed.m_mem_manager->set_resource_limit(rl);
+  }
+
   Uint32 sum = shared_pages + tupmem + filepages + jbpages + sbpages +
     pgman_pages + stpages + transmem;
 
