@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -1612,7 +1612,7 @@ TESTCASE("AbortWithSlowChildScans",
   INITIALIZER(runAbortWithSlowChildScans);
   FINALIZER(runCleanupTable);
 }
-NDBT_TESTSUITE_END(testFK);
+NDBT_TESTSUITE_END(testFK)
 
 int
 main(int argc, const char** argv)
