@@ -1,5 +1,5 @@
-/* Copyright (C) 2008 MySQL AB
-   Use is subject to license terms
+/*
+  Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -19,7 +19,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <NDBT.hpp>
 #include <NDBT_Test.hpp>
@@ -123,6 +123,7 @@ runTestSingleUserMode(NDBT_Context* ctx, NDBT_Step* step)
     CHECK(restarter.restartOneDbNode(nodeId) != 0)
     CHECK(restarter.exitSingleUserMode() == 0);
     CHECK(restarter.waitClusterStarted(timeout) == 0);
+    CHK_NDB_READY(pNdb);
 
     // Test that the single user mode api can do everything
     CHECK(restarter.enterSingleUserMode(pNdb->getNodeId()) == 0);
@@ -139,6 +140,7 @@ runTestSingleUserMode(NDBT_Context* ctx, NDBT_Step* step)
     CHECK(utilTrans.clearTable(pNdb, records/2) == 0);
     CHECK(restarter.exitSingleUserMode() == 0);
     CHECK(restarter.waitClusterStarted(timeout) == 0);
+    CHK_NDB_READY(pNdb);
 
     // Test create index in single user mode
     CHECK(restarter.enterSingleUserMode(pNdb->getNodeId()) == 0);
@@ -153,6 +155,7 @@ runTestSingleUserMode(NDBT_Context* ctx, NDBT_Step* step)
     CHECK(drop_index_on_pk(pNdb, tabName) == 0);
     CHECK(restarter.exitSingleUserMode() == 0);
     CHECK(restarter.waitClusterStarted(timeout) == 0);
+    CHK_NDB_READY(pNdb);
 
     // Test recreate index in single user mode
     CHECK(create_index_on_pk(pNdb, tabName) == 0);
@@ -165,6 +168,7 @@ runTestSingleUserMode(NDBT_Context* ctx, NDBT_Step* step)
     CHECK(create_index_on_pk(pNdb, tabName) == 0);
     CHECK(restarter.exitSingleUserMode() == 0);
     CHECK(restarter.waitClusterStarted(timeout) == 0);
+    CHK_NDB_READY(pNdb);
     CHECK(drop_index_on_pk(pNdb, tabName) == 0);
 
     CHECK(utilTrans.clearTable(GETNDB(step),  records) == 0);
@@ -187,7 +191,7 @@ TESTCASE("SingleUserMode",
   INITIALIZER(runTestSingleUserMode);
   FINALIZER(runClearTable);
 }
-NDBT_TESTSUITE_END(testSingleUserMode);
+NDBT_TESTSUITE_END(testSingleUserMode)
 
 
 int main(int argc, const char** argv){
