@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -1369,8 +1369,7 @@ void PFS_status_variable_cache::manifest(THD *thd,
   for (const SHOW_VAR *show_var_iter = show_var_array;
        show_var_iter && show_var_iter->name; show_var_iter++) {
     // work buffer, must be aligned to handle long/longlong values
-    my_aligned_storage<SHOW_VAR_FUNC_BUFF_SIZE + 1, MY_ALIGNOF(longlong)>
-        value_buf;
+    alignas(longlong) char value_buf[SHOW_VAR_FUNC_BUFF_SIZE + 1];
     SHOW_VAR show_var_tmp;
     const SHOW_VAR *show_var_ptr = show_var_iter; /* preserve array pointer */
 
@@ -1387,7 +1386,7 @@ void PFS_status_variable_cache::manifest(THD *thd,
       */
       for (const SHOW_VAR *var = show_var_ptr; var->type == SHOW_FUNC;
            var = &show_var_tmp) {
-        ((mysql_show_var_func)(var->value))(thd, &show_var_tmp, value_buf.data);
+        ((mysql_show_var_func)(var->value))(thd, &show_var_tmp, value_buf);
       }
       show_var_ptr = &show_var_tmp;
     }
@@ -1517,8 +1516,6 @@ void reset_pfs_status_stats() {
   reset_status_by_account();
   reset_status_by_user();
   reset_status_by_host();
-  /* Clear again, updated by previous aggregations. */
-  reset_global_status();
 }
 
 /**
