@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -1147,25 +1147,25 @@ PFS_connection_status_visitor::~PFS_connection_status_visitor() {}
 void PFS_connection_status_visitor::visit_global() {
   /* NOTE: Requires lock on LOCK_status. */
   mysql_mutex_assert_owner(&LOCK_status);
-  add_to_status(m_status_vars, &global_status_var, false);
+  add_to_status(m_status_vars, &global_status_var);
 }
 
 void PFS_connection_status_visitor::visit_host(PFS_host *pfs) {
-  pfs->m_status_stats.aggregate_to(m_status_vars, true);
+  pfs->m_status_stats.aggregate_to(m_status_vars);
 }
 
 void PFS_connection_status_visitor::visit_user(PFS_user *pfs) {
-  pfs->m_status_stats.aggregate_to(m_status_vars, true);
+  pfs->m_status_stats.aggregate_to(m_status_vars);
 }
 
 void PFS_connection_status_visitor::visit_account(PFS_account *pfs) {
-  pfs->m_status_stats.aggregate_to(m_status_vars, true);
+  pfs->m_status_stats.aggregate_to(m_status_vars);
 }
 
 void PFS_connection_status_visitor::visit_thread(PFS_thread *) {}
 
 void PFS_connection_status_visitor::visit_THD(THD *thd) {
-  add_to_status(m_status_vars, &thd->status_var, false);
+  add_to_status(m_status_vars, &thd->status_var);
 }
 
 PFS_instance_wait_visitor::PFS_instance_wait_visitor() {}
