@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -693,7 +693,7 @@ int PFS_object_row::make_row(PFS_program *pfs) {
 }
 
 int PFS_column_row::make_row(const MDL_key *mdl) {
-  static_assert(MDL_key::NAMESPACE_END == 18,
+  static_assert(MDL_key::NAMESPACE_END == 19,
                 "Adjust performance schema when changing enum_mdl_namespace");
 
   switch (mdl->mdl_namespace()) {
@@ -804,6 +804,11 @@ int PFS_column_row::make_row(const MDL_key *mdl) {
       m_schema_name_length = mdl->db_name_length();
       m_object_name_length = mdl->name_length();
       break;
+    case MDL_key::CHECK_CONSTRAINT:
+      m_object_type = OBJECT_TYPE_CHECK_CONSTRAINT;
+      m_schema_name_length = mdl->db_name_length();
+      m_object_name_length = mdl->name_length();
+      break;
     case MDL_key::NAMESPACE_END:
     default:
       DBUG_ASSERT(false);
@@ -1442,7 +1447,7 @@ int PFS_user_variable_value_row::make_row(const char *val, size_t length) {
 */
 #define COMMON_STATELESS_MATCH               \
   int cmp = 0;                               \
-  if (m_is_null) {                           \
+  if (is_null) {                             \
     cmp = (record_null ? 0 : 1);             \
   } else {                                   \
     if (record_null) {                       \
@@ -1455,7 +1460,7 @@ int PFS_user_variable_value_row::make_row(const char *val, size_t length) {
       cmp = 0;                               \
     }                                        \
   }                                          \
-  switch (m_find_flag) {                     \
+  switch (find_flag) {                       \
     case HA_READ_KEY_EXACT:                  \
       return (cmp == 0);                     \
     case HA_READ_KEY_OR_NEXT:                \
@@ -1472,27 +1477,27 @@ int PFS_user_variable_value_row::make_row(const char *val, size_t length) {
   }
 
 bool PFS_key_long::stateless_match(bool record_null, long record_value,
-                                   bool m_is_null, long m_key_value,
-                                   enum ha_rkey_function m_find_flag) {
+                                   bool is_null, long m_key_value,
+                                   enum ha_rkey_function find_flag) {
   COMMON_STATELESS_MATCH
 }
 
 bool PFS_key_ulong::stateless_match(bool record_null, ulong record_value,
-                                    bool m_is_null, ulong m_key_value,
-                                    enum ha_rkey_function m_find_flag) {
+                                    bool is_null, ulong m_key_value,
+                                    enum ha_rkey_function find_flag) {
   COMMON_STATELESS_MATCH
 }
 
 bool PFS_key_longlong::stateless_match(bool record_null, longlong record_value,
-                                       bool m_is_null, longlong m_key_value,
-                                       enum ha_rkey_function m_find_flag) {
+                                       bool is_null, longlong m_key_value,
+                                       enum ha_rkey_function find_flag) {
   COMMON_STATELESS_MATCH
 }
 
 bool PFS_key_ulonglong::stateless_match(bool record_null,
-                                        ulonglong record_value, bool m_is_null,
+                                        ulonglong record_value, bool is_null,
                                         ulonglong m_key_value,
-                                        enum ha_rkey_function m_find_flag) {
+                                        enum ha_rkey_function find_flag) {
   COMMON_STATELESS_MATCH
 }
 
@@ -1568,10 +1573,10 @@ bool PFS_key_pstring::stateless_match(bool record_null,
                                       const char *record_string,
                                       size_t record_string_length,
                                       const char *m_key_value,
-                                      size_t m_key_value_length, bool m_is_null,
-                                      enum ha_rkey_function m_find_flag) {
-  if (m_find_flag == HA_READ_KEY_EXACT) {
-    if (m_is_null) {
+                                      size_t m_key_value_length, bool is_null,
+                                      enum ha_rkey_function find_flag) {
+  if (find_flag == HA_READ_KEY_EXACT) {
+    if (is_null) {
       return record_null;
     }
 
@@ -1589,7 +1594,7 @@ bool PFS_key_pstring::stateless_match(bool record_null,
 
   int cmp = 0;
 
-  if (m_is_null) {
+  if (is_null) {
     cmp = record_null ? 0 : 1;
   } else {
     if (record_null) {
@@ -1599,7 +1604,7 @@ bool PFS_key_pstring::stateless_match(bool record_null,
     }
   }
 
-  switch (m_find_flag) {
+  switch (find_flag) {
     case HA_READ_KEY_OR_NEXT:
       return (cmp >= 0);
     case HA_READ_KEY_OR_PREV:
