@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -123,9 +123,8 @@ bool PFS_index_rpl_applier_status_by_coord_by_thread::match(Master_info *mi) {
 
     if (mi->rli->slave_running) {
       PSI_thread *psi = thd_get_psi(mi->rli->info_thd);
-      PFS_thread *pfs = reinterpret_cast<PFS_thread *>(psi);
-      if (pfs) {
-        row.thread_id = pfs->m_thread_internal_id;
+      if (psi != nullptr) {
+        row.thread_id = PSI_THREAD_CALL(get_thread_internal_id)(psi);
       }
     }
 
@@ -274,9 +273,8 @@ int table_replication_applier_status_by_coordinator::make_row(Master_info *mi) {
 
   if (mi->rli->slave_running) {
     PSI_thread *psi = thd_get_psi(mi->rli->info_thd);
-    PFS_thread *pfs = reinterpret_cast<PFS_thread *>(psi);
-    if (pfs) {
-      m_row.thread_id = pfs->m_thread_internal_id;
+    if (psi != nullptr) {
+      m_row.thread_id = PSI_THREAD_CALL(get_thread_internal_id)(psi);
       m_row.thread_id_is_null = false;
     }
   }
