@@ -17,7 +17,6 @@
 
 #include <sstream>
 #include <string>
-#include "log.h"
 
 namespace myrocks {
 
@@ -29,7 +28,7 @@ class Rdb_logger : public rocksdb::Logger {
 
   void Logv(const rocksdb::InfoLogLevel log_level, const char *format,
             va_list ap) override {
-    assert(format != nullptr);
+    DBUG_ASSERT(format != nullptr);
 
     enum loglevel mysql_log_level;
 
@@ -50,13 +49,13 @@ class Rdb_logger : public rocksdb::Logger {
     }
 
     // log to MySQL
-    std::string f("LibRocksDB:");
-    f.append(format);
-    error_log_print(mysql_log_level, f.c_str(), ap);
+    char buffer[1024];
+    vsnprintf(buffer, sizeof(buffer), format, ap);
+    LogPluginErrMsg(mysql_log_level, 0, "%s", buffer);
   }
 
   void Logv(const char *format, va_list ap) override {
-    assert(format != nullptr);
+    DBUG_ASSERT(format != nullptr);
     // If no level is specified, it is by default at information level
     Logv(rocksdb::InfoLogLevel::INFO_LEVEL, format, ap);
   }
