@@ -24,9 +24,6 @@
 /* C++ system header files */
 #include <string>
 
-/* MySQL header files */
-#include "log.h"
-
 /* RocksDB header files */
 #include "rocksdb/utilities/convenience.h"
 
@@ -102,7 +99,6 @@ bool Rdb_cf_options::set_default(const std::string &default_config) {
   if (!default_config.empty() && !rocksdb::GetColumnFamilyOptionsFromString(
                                       options, default_config, &options)
                                       .ok()) {
-    // NO_LINT_DEBUG
     fprintf(stderr, "Invalid default column family config: %s\n",
             default_config.c_str());
     return false;
@@ -116,7 +112,8 @@ bool Rdb_cf_options::set_default(const std::string &default_config) {
 void Rdb_cf_options::skip_spaces(const std::string &input, size_t *const pos) {
   DBUG_ASSERT(pos != nullptr);
 
-  while (*pos < input.size() && isspace(input[*pos])) ++(*pos);
+  while (*pos < input.size() && isspace(input[*pos]))
+    ++(*pos);
 }
 
 // Find a valid column family name.  Note that all characters except a
@@ -134,12 +131,13 @@ bool Rdb_cf_options::find_column_family(const std::string &input,
   // Loop through the characters in the string until we see a '='.
   for (; *pos < input.size() && input[*pos] != '='; ++(*pos)) {
     // If this is not a space, move the end position to the current position.
-    if (input[*pos] != ' ') end_pos = *pos;
+    if (input[*pos] != ' ')
+      end_pos = *pos;
   }
 
   if (end_pos == beg_pos - 1) {
-    // NO_LINT_DEBUG
-    sql_print_warning("No column family found (options: %s)", input.c_str());
+    LogPluginErrMsg(WARNING_LEVEL, 0, "No column family found (options: %s)",
+                    input.c_str());
     return false;
   }
 
@@ -157,9 +155,9 @@ bool Rdb_cf_options::find_options(const std::string &input, size_t *const pos,
 
   // Make sure we have an open curly brace at the current position.
   if (*pos < input.size() && input[*pos] != '{') {
-    // NO_LINT_DEBUG
-    sql_print_warning("Invalid cf options, '{' expected (options: %s)",
-                      input.c_str());
+    LogPluginErrMsg(WARNING_LEVEL, 0,
+                    "Invalid cf options, '{' expected (options: %s)",
+                    input.c_str());
     return false;
   }
 
@@ -175,24 +173,24 @@ bool Rdb_cf_options::find_options(const std::string &input, size_t *const pos,
   // number of closing curly braces.
   while (*pos < input.size()) {
     switch (input[*pos]) {
-      case '}':
-        // If this is a closing curly brace and we bring the count down to zero
-        // we can exit the loop with a valid options string.
-        if (--brace_count == 0) {
-          *options = input.substr(beg_pos, *pos - beg_pos);
-          ++(*pos);  // Move past the last closing curly brace
-          return true;
-        }
+    case '}':
+      // If this is a closing curly brace and we bring the count down to zero
+      // we can exit the loop with a valid options string.
+      if (--brace_count == 0) {
+        *options = input.substr(beg_pos, *pos - beg_pos);
+        ++(*pos);  // Move past the last closing curly brace
+        return true;
+      }
 
-        break;
+      break;
 
-      case '{':
-        // If this is an open curly brace increment the count.
-        ++brace_count;
-        break;
+    case '{':
+      // If this is an open curly brace increment the count.
+      ++brace_count;
+      break;
 
-      default:
-        break;
+    default:
+      break;
     }
 
     // Move to the next character.
@@ -201,9 +199,9 @@ bool Rdb_cf_options::find_options(const std::string &input, size_t *const pos,
 
   // We never found the correct number of closing curly braces.
   // Generate an error.
-  // NO_LINT_DEBUG
-  sql_print_warning("Mismatched cf options, '}' expected (options: %s)",
-                    input.c_str());
+  LogPluginErrMsg(WARNING_LEVEL, 0,
+                  "Mismatched cf options, '}' expected (options: %s)",
+                  input.c_str());
   return false;
 }
 
@@ -219,13 +217,14 @@ bool Rdb_cf_options::find_cf_options_pair(const std::string &input,
   skip_spaces(input, pos);
 
   // We should now have a column family name.
-  if (!find_column_family(input, pos, cf)) return false;
+  if (!find_column_family(input, pos, cf))
+    return false;
 
   // If we are at the end of the input then we generate an error.
   if (*pos == input.size()) {
-    // NO_LINT_DEBUG
-    sql_print_warning("Invalid cf options, '=' expected (options: %s)",
-                      input.c_str());
+    LogPluginErrMsg(WARNING_LEVEL, 0,
+                    "Invalid cf options, '=' expected (options: %s)",
+                    input.c_str());
     return false;
   }
 
@@ -235,7 +234,8 @@ bool Rdb_cf_options::find_cf_options_pair(const std::string &input,
 
   // Find the options for this column family.  This should be in the format
   // {<options>} where <options> may contain embedded pairs of curly braces.
-  if (!find_options(input, pos, opt_str)) return false;
+  if (!find_options(input, pos, opt_str))
+    return false;
 
   // Skip any trailing spaces after the option string.
   skip_spaces(input, pos);
@@ -243,9 +243,9 @@ bool Rdb_cf_options::find_cf_options_pair(const std::string &input,
   // We should either be at the end of the input string or at a semicolon.
   if (*pos < input.size()) {
     if (input[*pos] != ';') {
-      // NO_LINT_DEBUG
-      sql_print_warning("Invalid cf options, ';' expected (options: %s)",
-                        input.c_str());
+      LogPluginErrMsg(WARNING_LEVEL, 0,
+                      "Invalid cf options, ';' expected (options: %s)",
+                      input.c_str());
       return false;
     }
 
@@ -275,8 +275,8 @@ bool Rdb_cf_options::parse_cf_options(const std::string &cf_options,
 
     // Generate an error if we have already seen this column family.
     if (option_map->find(cf) != option_map->end()) {
-      // NO_LINT_DEBUG
-      sql_print_warning(
+      LogPluginErrMsg(
+          WARNING_LEVEL, 0,
           "Duplicate entry for %s in override options (options: %s)",
           cf.c_str(), cf_options.c_str());
       return false;
@@ -285,8 +285,8 @@ bool Rdb_cf_options::parse_cf_options(const std::string &cf_options,
     // Generate an error if the <opt_str> is not valid according to RocksDB.
     if (!rocksdb::GetColumnFamilyOptionsFromString(options, opt_str, &options)
              .ok()) {
-      // NO_LINT_DEBUG
-      sql_print_warning(
+      LogPluginErrMsg(
+          WARNING_LEVEL, 0,
           "Invalid cf config for %s in override options (options: %s)",
           cf.c_str(), cf_options.c_str());
       return false;
@@ -312,8 +312,8 @@ bool Rdb_cf_options::set_override(const std::string &override_config) {
   return true;
 }
 
-const rocksdb::Comparator *Rdb_cf_options::get_cf_comparator(
-    const std::string &cf_name) {
+const rocksdb::Comparator *
+Rdb_cf_options::get_cf_comparator(const std::string &cf_name) {
   if (Rdb_cf_manager::is_cf_name_reverse(cf_name.c_str())) {
     return &s_rev_pk_comparator;
   } else {
@@ -321,8 +321,8 @@ const rocksdb::Comparator *Rdb_cf_options::get_cf_comparator(
   }
 }
 
-std::shared_ptr<rocksdb::MergeOperator> Rdb_cf_options::get_cf_merge_operator(
-    const std::string &cf_name) {
+std::shared_ptr<rocksdb::MergeOperator>
+Rdb_cf_options::get_cf_merge_operator(const std::string &cf_name) {
   return (cf_name == DEFAULT_SYSTEM_CF_NAME)
              ? std::make_shared<Rdb_system_merge_op>()
              : nullptr;
