@@ -95,19 +95,17 @@ std::string rdb_pc_stat_types[] = {
     "IO_RANGE_SYNC_NANOS",
     "IO_LOGGER_NANOS"};
 
-#define IO_PERF_RECORD(_field_)                                       \
-  do {                                                                \
-    if (rocksdb::get_perf_context()->_field_ > 0) {                   \
-      counters->m_value[idx] += rocksdb::get_perf_context()->_field_; \
-    }                                                                 \
-    idx++;                                                            \
+#define IO_PERF_RECORD(_field_)                                                \
+  do {                                                                         \
+    if (rocksdb::get_perf_context()->_field_ > 0)                              \
+      counters->m_value[idx] += rocksdb::get_perf_context()->_field_;          \
+    idx++;                                                                     \
   } while (0)
-#define IO_STAT_RECORD(_field_)                                          \
-  do {                                                                   \
-    if (rocksdb::get_iostats_context()->_field_ > 0) {                   \
-      counters->m_value[idx] += rocksdb::get_iostats_context()->_field_; \
-    }                                                                    \
-    idx++;                                                               \
+#define IO_STAT_RECORD(_field_)                                                \
+  do {                                                                         \
+    if (rocksdb::get_iostats_context()->_field_ > 0)                           \
+      counters->m_value[idx] += rocksdb::get_iostats_context()->_field_;       \
+    idx++;                                                                     \
   } while (0)
 
 static void harvest_diffs(Rdb_atomic_perf_counters *const counters) {
@@ -162,17 +160,15 @@ static void harvest_diffs(Rdb_atomic_perf_counters *const counters) {
   IO_PERF_RECORD(key_lock_wait_time);
   IO_PERF_RECORD(key_lock_wait_count);
 
-  if (rocksdb::get_iostats_context() != nullptr) {
-    IO_STAT_RECORD(thread_pool_id);
-    IO_STAT_RECORD(bytes_written);
-    IO_STAT_RECORD(bytes_read);
-    IO_STAT_RECORD(open_nanos);
-    IO_STAT_RECORD(allocate_nanos);
-    IO_STAT_RECORD(write_nanos);
-    IO_STAT_RECORD(read_nanos);
-    IO_STAT_RECORD(range_sync_nanos);
-    IO_STAT_RECORD(logger_nanos);
-  }
+  IO_STAT_RECORD(thread_pool_id);
+  IO_STAT_RECORD(bytes_written);
+  IO_STAT_RECORD(bytes_read);
+  IO_STAT_RECORD(open_nanos);
+  IO_STAT_RECORD(allocate_nanos);
+  IO_STAT_RECORD(write_nanos);
+  IO_STAT_RECORD(read_nanos);
+  IO_STAT_RECORD(range_sync_nanos);
+  IO_STAT_RECORD(logger_nanos);
 }
 
 #undef IO_PERF_DIFF
@@ -205,12 +201,7 @@ bool Rdb_io_perf::start(const uint32_t perf_context_level) {
   }
 
   rocksdb::get_perf_context()->Reset();
-
-  auto *io_stats_context = rocksdb::get_iostats_context();
-  if (io_stats_context != nullptr) {
-    io_stats_context->Reset();
-  }
-
+  rocksdb::get_iostats_context()->Reset();
   return true;
 }
 
