@@ -6,16 +6,14 @@
 #include "rocksdb/sst_file_writer.h"
 
 #include <vector>
-
 #include "db/dbformat.h"
-#include "file/writable_file_writer.h"
-#include "rocksdb/file_system.h"
 #include "rocksdb/table.h"
-#include "table/block_based/block_based_table_builder.h"
+#include "table/block_based_table_builder.h"
 #include "table/sst_file_writer_collectors.h"
-#include "test_util/sync_point.h"
+#include "util/file_reader_writer.h"
+#include "util/sync_point.h"
 
-namespace ROCKSDB_NAMESPACE {
+namespace rocksdb {
 
 const std::string ExternalSstFilePropertyNames::kVersion =
     "rocksdb.external_sst_file.version";
@@ -43,7 +41,7 @@ struct SstFileWriter::Rep {
   std::unique_ptr<WritableFileWriter> file_writer;
   std::unique_ptr<TableBuilder> builder;
   EnvOptions env_options;
-  ImmutableOptions ioptions;
+  ImmutableCFOptions ioptions;
   MutableCFOptions mutable_cf_options;
   Env::IOPriority io_priority;
   InternalKeyComparator internal_comparator;
@@ -70,8 +68,7 @@ struct SstFileWriter::Rep {
       if (internal_comparator.user_comparator()->Compare(
               user_key, file_info.largest_key) <= 0) {
         // Make sure that keys are added in order
-        return Status::InvalidArgument(
-            "Keys must be added in strict ascending order.");
+        return Status::InvalidArgument("Keys must be added in order");
       }
     }
 
@@ -99,7 +96,8 @@ struct SstFileWriter::Rep {
     file_info.largest_key.assign(user_key.data(), user_key.size());
     file_info.file_size = builder->FileSize();
 
-    InvalidatePageCache(false /* closing */).PermitUncheckedError();
+    InvalidatePageCache(false /* closing */);
+
     return Status::OK();
   }
 
@@ -134,32 +132,27 @@ struct SstFileWriter::Rep {
     file_info.num_range_del_entries++;
     file_info.file_size = builder->FileSize();
 
-    InvalidatePageCache(false /* closing */).PermitUncheckedError();
+    InvalidatePageCache(false /* closing */);
+
     return Status::OK();
   }
 
-  Status InvalidatePageCache(bool closing) {
-    Status s = Status::OK();
+  void InvalidatePageCache(bool closing) {
     if (invalidate_page_cache == false) {
       // Fadvise disabled
-      return s;
+      return;
     }
     uint64_t bytes_since_last_fadvise =
       builder->FileSize() - last_fadvise_size;
     if (bytes_since_last_fadvise > kFadviseTrigger || closing) {
       TEST_SYNC_POINT_CALLBACK("SstFileWriter::Rep::InvalidatePageCache",
                                &(bytes_since_last_fadvise));
-      // Tell the OS that we don't need this file in page cache
-      s = file_writer->InvalidateCache(0, 0);
-      if (s.IsNotSupported()) {
-        // NotSupported is fine as it could be a file type that doesn't use page
-        // cache.
-        s = Status::OK();
-      }
+      // Tell the OS that we dont need this file in page cache
+      file_writer->InvalidateCache(0, 0);
       last_fadvise_size = builder->FileSize();
     }
-    return s;
   }
+
 };
 
 SstFileWriter::SstFileWriter(const EnvOptions& env_options,
@@ -184,10 +177,8 @@ SstFileWriter::~SstFileWriter() {
 Status SstFileWriter::Open(const std::string& file_path) {
   Rep* r = rep_.get();
   Status s;
-  std::unique_ptr<FSWritableFile> sst_file;
-  FileOptions cur_file_opts(r->env_options);
-  s = r->ioptions.env->GetFileSystem()->NewWritableFile(
-      file_path, cur_file_opts, &sst_file, nullptr);
+  std::unique_ptr<WritableFile> sst_file;
+  s = r->ioptions.env->NewWritableFile(file_path, &sst_file, r->env_options);
   if (!s.ok()) {
     return s;
   }
@@ -196,24 +187,24 @@ Status SstFileWriter::Open(const std::string& file_path) {
 
   CompressionType compression_type;
   CompressionOptions compression_opts;
-  if (r->mutable_cf_options.bottommost_compression !=
-      kDisableCompressionOption) {
-    compression_type = r->mutable_cf_options.bottommost_compression;
-    if (r->mutable_cf_options.bottommost_compression_opts.enabled) {
-      compression_opts = r->mutable_cf_options.bottommost_compression_opts;
+  if (r->ioptions.bottommost_compression != kDisableCompressionOption) {
+    compression_type = r->ioptions.bottommost_compression;
+    if (r->ioptions.bottommost_compression_opts.enabled) {
+      compression_opts = r->ioptions.bottommost_compression_opts;
     } else {
-      compression_opts = r->mutable_cf_options.compression_opts;
+      compression_opts = r->ioptions.compression_opts;
     }
   } else if (!r->ioptions.compression_per_level.empty()) {
     // Use the compression of the last level if we have per level compression
     compression_type = *(r->ioptions.compression_per_level.rbegin());
-    compression_opts = r->mutable_cf_options.compression_opts;
+    compression_opts = r->ioptions.compression_opts;
   } else {
     compression_type = r->mutable_cf_options.compression;
-    compression_opts = r->mutable_cf_options.compression_opts;
+    compression_opts = r->ioptions.compression_opts;
   }
 
-  IntTblPropCollectorFactories int_tbl_prop_collector_factories;
+  std::vector<std::unique_ptr<IntTblPropCollectorFactory>>
+      int_tbl_prop_collector_factories;
 
   // SstFileWriter properties collector to add SstFileWriter version.
   int_tbl_prop_collector_factories.emplace_back(
@@ -240,36 +231,19 @@ Status SstFileWriter::Open(const std::string& file_path) {
     r->column_family_name = "";
     cf_id = TablePropertiesCollectorFactory::Context::kUnknownColumnFamily;
   }
-  // SstFileWriter is used to create sst files that can be added to database
-  // later. Therefore, no real db_id and db_session_id are associated with it.
-  // Here we mimic the way db_session_id behaves by resetting the db_session_id
-  // every time SstFileWriter is used, and in this case db_id is set to be "SST
-  // Writer".
-  std::string db_session_id = r->ioptions.env->GenerateUniqueId();
-  if (!db_session_id.empty() && db_session_id.back() == '\n') {
-    db_session_id.pop_back();
-  }
+
   TableBuilderOptions table_builder_options(
       r->ioptions, r->mutable_cf_options, r->internal_comparator,
       &int_tbl_prop_collector_factories, compression_type, compression_opts,
-      cf_id, r->column_family_name, unknown_level, false /* is_bottommost */,
-      TableFileCreationReason::kMisc, 0 /* creation_time */,
-      0 /* oldest_key_time */, 0 /* file_creation_time */,
-      "SST Writer" /* db_id */, db_session_id, 0 /* target_file_size */);
-  // XXX: when we can remove skip_filters from the SstFileWriter public API
-  // we can remove it from TableBuilderOptions.
-  table_builder_options.skip_filters = r->skip_filters;
-  FileTypeSet tmp_set = r->ioptions.checksum_handoff_file_types;
-  r->file_writer.reset(new WritableFileWriter(
-      std::move(sst_file), file_path, r->env_options, r->ioptions.clock,
-      nullptr /* io_tracer */, nullptr /* stats */, r->ioptions.listeners,
-      r->ioptions.file_checksum_gen_factory.get(),
-      tmp_set.Contains(FileType::kTableFile)));
+      nullptr /* compression_dict */, r->skip_filters, r->column_family_name,
+      unknown_level);
+  r->file_writer.reset(
+      new WritableFileWriter(std::move(sst_file), file_path, r->env_options));
 
   // TODO(tec) : If table_factory is using compressed block cache, we will
   // be adding the external sst file blocks into it, which is wasteful.
   r->builder.reset(r->ioptions.table_factory->NewTableBuilder(
-      table_builder_options, r->file_writer.get()));
+      table_builder_options, cf_id, r->file_writer.get()));
 
   r->file_info = ExternalSstFileInfo();
   r->file_info.file_path = file_path;
@@ -313,16 +287,11 @@ Status SstFileWriter::Finish(ExternalSstFileInfo* file_info) {
 
   if (s.ok()) {
     s = r->file_writer->Sync(r->ioptions.use_fsync);
-    r->InvalidatePageCache(true /* closing */).PermitUncheckedError();
+    r->InvalidatePageCache(true /* closing */);
     if (s.ok()) {
       s = r->file_writer->Close();
     }
   }
-  if (s.ok()) {
-    r->file_info.file_checksum = r->file_writer->GetFileChecksum();
-    r->file_info.file_checksum_func_name =
-        r->file_writer->GetFileChecksumFuncName();
-  }
   if (!s.ok()) {
     r->ioptions.env->DeleteFile(r->file_info.file_path);
   }
@@ -340,4 +309,4 @@ uint64_t SstFileWriter::FileSize() {
 }
 #endif  // !ROCKSDB_LITE
 
-}  // namespace ROCKSDB_NAMESPACE
+}  // namespace rocksdb
