@@ -363,7 +363,8 @@ inline void TOKUDB_SHARE::init_cardinality_counts(uint32_t rec_per_keys,
                                                   uint64_t *rec_per_key) {
   assert_debug(_mutex.is_owned_by_me());
   // can not change number of keys live
-  assert_always(_rec_per_key == NULL && _rec_per_keys == 0);
+  assert_always(_rec_per_key == nullptr);
+  assert_always(_rec_per_keys == 0);
   _rec_per_keys = rec_per_keys;
   _rec_per_key = rec_per_key;
 }
