@@ -1,37 +1,42 @@
-/* Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2011, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-// Always include (the generated) my_config.h, to get correct platform defines.
-#include "my_config.h"
 #include <gtest/gtest.h>
+#include <limits.h>
 
-#include "test_utils.h"
-#include "my_stacktrace.h"
 #include "m_string.h"
-#include "hash_filo.h"
+#include "my_inttypes.h"
+#include "my_stacktrace.h"
+#include "unittest/gunit/test_utils.h"
+#include "unittest/gunit/thread_utils.h"
 
 namespace segfault_unittest {
 
-using my_testing::Server_initializer;
 using my_testing::Mock_error_handler;
+using my_testing::Server_initializer;
 
-class FatalSignalDeathTest : public ::testing::Test
-{
-protected:
-  virtual void SetUp()
-  {
+class FatalSignalDeathTest : public ::testing::Test {
+ protected:
+  virtual void SetUp() {
     ::testing::FLAGS_gtest_death_test_style = "threadsafe";
     initializer.SetUp();
   }
@@ -40,9 +45,7 @@ protected:
   Server_initializer initializer;
 };
 
-
-TEST_F(FatalSignalDeathTest, Abort)
-{
+TEST_F(FatalSignalDeathTest, Abort) {
 #if defined(_WIN32)
   MY_EXPECT_DEATH_IF_SUPPORTED(abort(), ".* UTC - mysqld got exception.*");
 #else
@@ -50,80 +53,63 @@ TEST_F(FatalSignalDeathTest, Abort)
 #endif
 }
 
-
-TEST_F(FatalSignalDeathTest, Segfault)
-{
-  int *pint= NULL;
+TEST_F(FatalSignalDeathTest, Segfault) {
+  int *pint = NULL;
 #if defined(_WIN32)
   /*
    After upgrading from gtest 1.5 to 1.6 this segfault is no longer
    caught by handle_fatal_signal(). We get an empty error message from the
    gtest library instead.
   */
-  MY_EXPECT_DEATH_IF_SUPPORTED(*pint= 42, "");
+  MY_EXPECT_DEATH_IF_SUPPORTED(*pint = 42, "");
 #elif defined(__SANITIZE_ADDRESS__)
   /* AddressSanitizer */
-  MY_EXPECT_DEATH_IF_SUPPORTED(*pint= 42, ".*ASAN:(DEADLYSIGNAL|SIGSEGV).*");
+  MY_EXPECT_DEATH_IF_SUPPORTED(*pint = 42, ".*ASAN:(DEADLYSIGNAL|SIGSEGV).*");
 #else
   /*
    On most platforms we get SIGSEGV == 11, but SIGBUS == 10 is also possible.
    And on Mac OsX we can get SIGILL == 4 (but only in optmized mode).
   */
-  MY_EXPECT_DEATH_IF_SUPPORTED(*pint= 42, ".* UTC - mysqld got signal .*");
+  MY_EXPECT_DEATH_IF_SUPPORTED(*pint = 42, ".* UTC - mysqld got signal .*");
 #endif
 }
 
-
-// A simple helper function to determine array size.
-template <class T, int size>
-int array_size(const T (&)[size])
-{
-  return size;
-}
-
-
 // Verifies that my_safe_utoa behaves like sprintf(_, "%llu", _)
-TEST(PrintUtilities, Utoa)
-{
+TEST(PrintUtilities, Utoa) {
   char buff[22];
-  ulonglong intarr[]= { 0, 1, 8, 12, 1234, 88888, ULONG_MAX, ULLONG_MAX };
+  ulonglong intarr[] = {0, 1, 8, 12, 1234, 88888, ULONG_MAX, ULLONG_MAX};
   char sprintbuff[22];
-  for (int ix= 0; ix < array_size(intarr); ++ix)
-  {
+  for (size_t ix = 0; ix < array_elements(intarr); ++ix) {
     char *my_res;
     sprintf(sprintbuff, "%llu", intarr[ix]);
-    my_res= my_safe_utoa(10, intarr[ix], &buff[sizeof(buff)-1]);
+    my_res = my_safe_utoa(10, intarr[ix], &buff[sizeof(buff) - 1]);
     EXPECT_STREQ(sprintbuff, my_res);
 
-    if (intarr[ix] <= ULONG_MAX)
-    {
-      sprintf(sprintbuff, "%lu", (ulong) intarr[ix]);
-      my_res= my_safe_utoa(10, (ulong) intarr[ix], &buff[sizeof(buff)-1]);
+    if (intarr[ix] <= ULONG_MAX) {
+      sprintf(sprintbuff, "%lu", (ulong)intarr[ix]);
+      my_res = my_safe_utoa(10, (ulong)intarr[ix], &buff[sizeof(buff) - 1]);
       EXPECT_STREQ(sprintbuff, my_res);
     }
   }
 }
 
-
 // Verifies that my_safe_itoa behaves like sprintf(_, "%lld", _)
-TEST(PrintUtilities, Itoa)
-{
+TEST(PrintUtilities, Itoa) {
   char buff[22];
   char sprintbuff[22];
-  longlong intarr[]= { 0, 1, 8, 12, 1234, 88888, LONG_MAX, LLONG_MAX };
+  longlong intarr[] = {0, 1, 8, 12, 1234, 88888, LONG_MAX, LLONG_MAX};
 
-  for (int ix= 0; ix < array_size(intarr); ++ix)
-  {
+  for (size_t ix = 0; ix < array_elements(intarr); ++ix) {
     char *my_res;
     sprintf(sprintbuff, "%lld", intarr[ix]);
-    my_res= my_safe_itoa(10, intarr[ix], &buff[sizeof(buff)-1]);
+    my_res = my_safe_itoa(10, intarr[ix], &buff[sizeof(buff) - 1]);
     EXPECT_STREQ(sprintbuff, my_res);
 
     ll2str(intarr[ix], buff, 10, 0);
     EXPECT_STREQ(sprintbuff, buff);
 
     sprintf(sprintbuff, "%lld", -intarr[ix]);
-    my_res= my_safe_itoa(10, -intarr[ix], &buff[sizeof(buff)-1]);
+    my_res = my_safe_itoa(10, -intarr[ix], &buff[sizeof(buff) - 1]);
     EXPECT_STREQ(sprintbuff, my_res);
 
     // This one fails ....
@@ -132,43 +118,40 @@ TEST(PrintUtilities, Itoa)
     //  << "failed for " << -intarr[ix];
 
     sprintf(sprintbuff, "%llx", intarr[ix]);
-    my_res= my_safe_itoa(16, intarr[ix], &buff[sizeof(buff)-1]);
+    my_res = my_safe_itoa(16, intarr[ix], &buff[sizeof(buff) - 1]);
     EXPECT_STREQ(sprintbuff, my_res);
 
     ll2str(intarr[ix], buff, 16, 0);
     EXPECT_STREQ(sprintbuff, buff);
 
     sprintf(sprintbuff, "%llx", -intarr[ix]);
-    my_res= my_safe_itoa(16, -intarr[ix], &buff[sizeof(buff)-1]);
-    EXPECT_STREQ(sprintbuff, my_res)
-      << "failed for " << -intarr[ix];
+    my_res = my_safe_itoa(16, -intarr[ix], &buff[sizeof(buff) - 1]);
+    EXPECT_STREQ(sprintbuff, my_res) << "failed for " << -intarr[ix];
 
     ll2str(-intarr[ix], buff, 16, 0);
     EXPECT_STREQ(sprintbuff, buff);
   }
 }
 
-
 // Various tests for my_safe_snprintf.
-TEST(PrintUtilities, Printf)
-{
+TEST(PrintUtilities, Printf) {
   char buff[512];
   char sprintfbuff[512];
-  const char *null_str= NULL;
+  const char *null_str = NULL;
 
   my_safe_snprintf(buff, sizeof(buff), "hello");
   EXPECT_STREQ("hello", buff);
- 
+
   my_safe_snprintf(buff, sizeof(buff), "hello %s hello", "hello");
   EXPECT_STREQ("hello hello hello", buff);
   my_safe_snprintf(buff, sizeof(buff), "hello %s hello", null_str);
   EXPECT_STREQ("hello (null) hello", buff);
- 
+
   my_safe_snprintf(buff, sizeof(buff), "hello %d hello", 42);
   EXPECT_STREQ("hello 42 hello", buff);
   my_safe_snprintf(buff, sizeof(buff), "hello %i hello", 42);
   EXPECT_STREQ("hello 42 hello", buff);
-  my_safe_snprintf(buff, sizeof(buff), "hello %u hello", (unsigned) 42);
+  my_safe_snprintf(buff, sizeof(buff), "hello %u hello", (unsigned)42);
   EXPECT_STREQ("hello 42 hello", buff);
 
   my_safe_snprintf(buff, sizeof(buff), "hello %llu hello", ULLONG_MAX);
@@ -183,33 +166,16 @@ TEST(PrintUtilities, Printf)
   EXPECT_STREQ("hello ffffffd6 hello", buff);
   EXPECT_STREQ(sprintfbuff, buff);
 
-  my_safe_snprintf(buff, sizeof(buff), "hello %llx hello", (longlong) -42);
-  sprintf(sprintfbuff, "hello %llx hello", (longlong) -42);
+  my_safe_snprintf(buff, sizeof(buff), "hello %llx hello", (longlong)-42);
+  sprintf(sprintfbuff, "hello %llx hello", (longlong)-42);
   EXPECT_STREQ("hello ffffffffffffffd6 hello", buff);
   EXPECT_STREQ(sprintfbuff, buff);
 
-  void *p= this;
+  void *p = this;
   my_safe_snprintf(buff, sizeof(buff), "hello 0x%p hello", p);
-  my_snprintf(sprintfbuff, sizeof(sprintfbuff), "hello %p hello", p);
+  snprintf(sprintfbuff, sizeof(sprintfbuff), "hello 0x%llx hello",
+           (long long)p);
   EXPECT_STREQ(sprintfbuff, buff) << "my_safe_snprintf:" << buff;
 }
 
-
-// After the fix for Bug#14689561, this is no longer a death test.
-TEST(HashFiloTest, TestHashFiloZeroSize)
-{
-  hash_filo *t_cache;
-  t_cache= new hash_filo(PSI_NOT_INSTRUMENTED,
-                         5, 0, 0,
-                         (my_hash_get_key) NULL,
-                         (my_hash_free_key) NULL,
-                         NULL);
-  t_cache->clear();
-  t_cache->resize(0);
-  hash_filo_element entry;
-  // After resize (to zero) it tries to dereference last_link which is NULL.
-  t_cache->add(&entry);
-  delete t_cache;
-}
-
-}
+}  // namespace segfault_unittest
