@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -286,6 +286,17 @@ size_t vio_ssl_read(Vio *vio, uchar *buf, size_t size) {
     /* Process the SSL I/O error. */
     if (!ssl_should_retry(vio, ret, &event, &ssl_errno_not_used)) break;
 
+    if (!vio->is_blocking_flag) {
+      switch (event) {
+        case VIO_IO_EVENT_READ:
+          DBUG_RETURN(VIO_SOCKET_WANT_READ);
+        case VIO_IO_EVENT_WRITE:
+          DBUG_RETURN(VIO_SOCKET_WANT_WRITE);
+        default:
+          DBUG_RETURN(VIO_SOCKET_ERROR);
+      }
+    }
+
     /* Attempt to wait for an I/O event. */
     if (vio_socket_io_wait(vio, event)) break;
   }
@@ -318,6 +329,17 @@ size_t vio_ssl_write(Vio *vio, const uchar *buf, size_t size) {
     /* Process the SSL I/O error. */
     if (!ssl_should_retry(vio, ret, &event, &ssl_errno_not_used)) break;
 
+    if (!vio->is_blocking_flag) {
+      switch (event) {
+        case VIO_IO_EVENT_READ:
+          DBUG_RETURN(VIO_SOCKET_WANT_READ);
+        case VIO_IO_EVENT_WRITE:
+          DBUG_RETURN(VIO_SOCKET_WANT_WRITE);
+        default:
+          DBUG_RETURN(VIO_SOCKET_ERROR);
+      }
+    }
+
     /* Attempt to wait for an I/O event. */
     if (vio_socket_io_wait(vio, event)) break;
   }
@@ -397,9 +419,10 @@ typedef int (*ssl_handshake_func_t)(SSL *);
   @return Return value is 1 on success.
 */
 
-static int ssl_handshake_loop(Vio *vio, SSL *ssl, ssl_handshake_func_t func,
-                              unsigned long *ssl_errno_holder) {
-  int ret;
+static size_t ssl_handshake_loop(Vio *vio, SSL *ssl, ssl_handshake_func_t func,
+                                 unsigned long *ssl_errno_holder) {
+  DBUG_ENTER(__func__);
+  size_t ret = -1;
 
   vio->ssl_arg = ssl;
 
@@ -414,12 +437,27 @@ static int ssl_handshake_loop(Vio *vio, SSL *ssl, ssl_handshake_func_t func,
     */
     DBUG_ASSERT(ERR_peek_error() == 0);
 
-    ret = func(ssl);
+    int handshake_ret;
+    handshake_ret = func(ssl);
 
-    if (ret >= 1) break;
+    if (handshake_ret >= 1) {
+      ret = 0;
+      break;
+    }
 
     /* Process the SSL I/O error. */
-    if (!ssl_should_retry(vio, ret, &event, ssl_errno_holder)) break;
+    if (!ssl_should_retry(vio, handshake_ret, &event, ssl_errno_holder)) break;
+
+    if (!vio->is_blocking_flag) {
+      switch (event) {
+        case VIO_IO_EVENT_READ:
+          DBUG_RETURN(VIO_SOCKET_WANT_READ);
+        case VIO_IO_EVENT_WRITE:
+          DBUG_RETURN(VIO_SOCKET_WANT_WRITE);
+        default:
+          DBUG_RETURN(VIO_SOCKET_ERROR);
+      }
+    }
 
     /* Wait for I/O so that the handshake can proceed. */
     if (vio_socket_io_wait(vio, event)) break;
@@ -427,13 +465,13 @@ static int ssl_handshake_loop(Vio *vio, SSL *ssl, ssl_handshake_func_t func,
 
   vio->ssl_arg = NULL;
 
-  return ret;
+  DBUG_RETURN(ret);
 }
 
 static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
-                  ssl_handshake_func_t func, unsigned long *ssl_errno_holder) {
-  int r;
-  SSL *ssl;
+                  ssl_handshake_func_t func, unsigned long *ssl_errno_holder,
+                  SSL **sslptr) {
+  SSL *ssl = nullptr;
   my_socket sd = mysql_socket_getfd(vio->mysql_socket);
 
   /* Declared here to make compiler happy */
@@ -444,59 +482,75 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
   DBUG_ENTER("ssl_do");
   DBUG_PRINT("enter", ("ptr: %p, sd: %d  ctx: %p", ptr, sd, ptr->ssl_context));
 
-  if (!(ssl = SSL_new(ptr->ssl_context))) {
-    DBUG_PRINT("error", ("SSL_new failure"));
-    *ssl_errno_holder = ERR_get_error();
-    DBUG_RETURN(1);
+  if (!sslptr) {
+    sslptr = &ssl;
   }
-  DBUG_PRINT("info", ("ssl: %p timeout: %ld", ssl, timeout));
-  SSL_clear(ssl);
-  SSL_SESSION_set_timeout(SSL_get_session(ssl), timeout);
-  SSL_set_fd(ssl, sd);
+
+  if (*sslptr == nullptr) {
+    if (!(ssl = SSL_new(ptr->ssl_context))) {
+      DBUG_PRINT("error", ("SSL_new failure"));
+      *ssl_errno_holder = ERR_get_error();
+      DBUG_RETURN(1);
+    }
+
+    DBUG_PRINT("info", ("ssl: %p timeout: %ld", ssl, timeout));
+    SSL_clear(ssl);
+    SSL_SESSION_set_timeout(SSL_get_session(ssl), timeout);
+    SSL_set_fd(ssl, sd);
 #if !defined(HAVE_WOLFSSL) && defined(SSL_OP_NO_COMPRESSION)
-  SSL_set_options(ssl, SSL_OP_NO_COMPRESSION); /* OpenSSL >= 1.0 only */
+    SSL_set_options(ssl, SSL_OP_NO_COMPRESSION); /* OpenSSL >= 1.0 only */
 #elif !defined(HAVE_WOLFSSL) && \
     OPENSSL_VERSION_NUMBER >= 0x00908000L /* workaround for OpenSSL 0.9.8 */
-  sk_SSL_COMP_zero(SSL_COMP_get_compression_methods());
+    sk_SSL_COMP_zero(SSL_COMP_get_compression_methods());
 #endif
 
 #if !defined(HAVE_WOLFSSL) && !defined(DBUG_OFF)
-  {
-    STACK_OF(SSL_COMP) *ssl_comp_methods = NULL;
-    ssl_comp_methods = SSL_COMP_get_compression_methods();
-    n = sk_SSL_COMP_num(ssl_comp_methods);
-    DBUG_PRINT("info", ("Available compression methods:\n"));
-    if (n == 0)
-      DBUG_PRINT("info", ("NONE\n"));
-    else
-      for (j = 0; j < n; j++) {
-        SSL_COMP *c = sk_SSL_COMP_value(ssl_comp_methods, j);
+    {
+      STACK_OF(SSL_COMP) *ssl_comp_methods = NULL;
+      ssl_comp_methods = SSL_COMP_get_compression_methods();
+      n = sk_SSL_COMP_num(ssl_comp_methods);
+      DBUG_PRINT("info", ("Available compression methods:\n"));
+      if (n == 0)
+        DBUG_PRINT("info", ("NONE\n"));
+      else
+        for (j = 0; j < n; j++) {
+          SSL_COMP *c = sk_SSL_COMP_value(ssl_comp_methods, j);
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-        DBUG_PRINT("info", ("  %d: %s\n", c->id, c->name));
+          DBUG_PRINT("info", ("  %d: %s\n", c->id, c->name));
 #else  /* OPENSSL_VERSION_NUMBER < 0x10100000L */
-        DBUG_PRINT("info",
-                   ("  %d: %s\n", SSL_COMP_get_id(c), SSL_COMP_get0_name(c)));
+          DBUG_PRINT("info",
+                     ("  %d: %s\n", SSL_COMP_get_id(c), SSL_COMP_get0_name(c)));
 #endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
-      }
-  }
+        }
+    }
 #endif
 
-    /*
-      Since yaSSL does not support non-blocking send operations, use
-      special transport functions that properly handles non-blocking
-      sockets. These functions emulate the behavior of blocking I/O
-      operations by waiting for I/O to become available.
-    */
+      /*
+        Since yaSSL does not support non-blocking send operations, use
+        special transport functions that properly handles non-blocking
+        sockets. These functions emulate the behavior of blocking I/O
+        operations by waiting for I/O to become available.
+      */
 #ifdef HAVE_WOLFSSL
-  /* Set first argument of the transport functions. */
-  wolfSSL_SetIOReadCtx(ssl, vio);
-  wolfSSL_SetIOWriteCtx(ssl, vio);
+    /* Set first argument of the transport functions. */
+    wolfSSL_SetIOReadCtx(ssl, vio);
+    wolfSSL_SetIOWriteCtx(ssl, vio);
 #endif
+    *sslptr = ssl;
+  } else {
+    ssl = *sslptr;
+  }
+
+  size_t loop_ret;
+  if ((loop_ret = ssl_handshake_loop(vio, ssl, func, ssl_errno_holder))) {
+    if (loop_ret != VIO_SOCKET_ERROR) {
+      DBUG_RETURN((int)loop_ret);  // Don't free SSL
+    }
 
-  if ((r = ssl_handshake_loop(vio, ssl, func, ssl_errno_holder)) < 1) {
     DBUG_PRINT("error", ("SSL_connect/accept failure"));
     SSL_free(ssl);
-    DBUG_RETURN(1);
+    *sslptr = nullptr;
+    DBUG_RETURN((int)VIO_SOCKET_ERROR);
   }
 
   /*
@@ -505,6 +559,9 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
     and set pointer to the SSL structure
   */
   if (vio_reset(vio, VIO_TYPE_SSL, SSL_get_fd(ssl), ssl, 0)) DBUG_RETURN(1);
+  if (sslptr != &ssl) {
+    *sslptr = nullptr;
+  }
 
 #ifndef DBUG_OFF
   {
@@ -538,14 +595,14 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
 int sslaccept(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
               unsigned long *ssl_errno_holder) {
   DBUG_ENTER("sslaccept");
-  int ret = ssl_do(ptr, vio, timeout, SSL_accept, ssl_errno_holder);
+  int ret = ssl_do(ptr, vio, timeout, SSL_accept, ssl_errno_holder, nullptr);
   DBUG_RETURN(ret);
 }
 
 int sslconnect(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
-               unsigned long *ssl_errno_holder) {
+               unsigned long *ssl_errno_holder, SSL **ssl) {
   DBUG_ENTER("sslconnect");
-  int ret = ssl_do(ptr, vio, timeout, SSL_connect, ssl_errno_holder);
+  int ret = ssl_do(ptr, vio, timeout, SSL_connect, ssl_errno_holder, ssl);
   DBUG_RETURN(ret);
 }
 
